%%   Eotvos Quantum Transport Utilities - CreateHamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to create and store Hamiltonian of the scattering region
%%
classdef CreateHamiltonians < handle & Messages

    properties ( Access = protected )
        %> An instance of structure param
        param
        %> A list of the sites to be kept after decimation.
        kulso_szabfokok
        %> The matrix of the Hamiltonian.
        Hscatter
        %> The matrix of the Hamiltonian corresponding to the transverse coupling.
        Hscatter_transverse
        %> The matrix of the overlap integrals in the Hamiltonian.
        Sscatter
        %> The matrix of the overlap integrals for the transverse coupling.
        Sscatter_transverse        
        %> The matrix of the Peierls phases.
        fazis_mtx_scatter
        %> The matrix of the Peierls phases for the transverse coupling.
        fazis_mtx_scatter_t
        %> An instance of the structure coordinates.
        coordinates
        %> The length of the scattering region in units of the lattice contant.
        height
        %> The number of sites in the cross section.
        width
        %> The number of sites in one unit cell.
        M
        %> A logical value. True if the Hamiltonian was created, false otherwise.
        HamiltoniansCreated
        %> A logical value. True if the Hamiltonian was decimated, or false otherwise.
        HamiltoniansDecimated
        %> A logical value. True if the overlap integrals were applied, false otherwise.
        OverlapApplied %NEW           
        %> A logical value. True if the vector potential was incorporated into the Hamiltonian or false otherwise.
        MagneticFieldApplied
        %> A logical value. True if a gauge transformation was applied on the Hamiltonians, or false otherwise.
        GaugeTransformationApplied
        %> A transverse momentum.
        q

    end
    

methods ( Access = public )
%% constructorof the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
function  obj = CreateHamiltonians(Opt, param, varargin)
        obj = obj@Messages( Opt );
        obj.Opt = Opt;
        obj.param = param;

        obj.Reset();

        obj.InputParsing( varargin{:} );

end

%% CreateScatterH
%> @brief Creates a Hamiltonian of a rectangle shaped area. The created Hamiltonian and coordinates are stored within the object.
%> @param varargin Optional parameters, see the documenattion for details.
function CreateScatterH( obj, varargin )  
    p = inputParser;
    p.addParamValue('Surface_tmp', []);      
    p.addParamValue('CustomHamiltonian', []);    % NEW  
    p.addParamValue('EF', []);    % NEW      
    p.parse(varargin{:});       
    Surface_tmp     = p.Results.Surface_tmp;
    CustomHamiltonian = p.Results.CustomHamiltonian;
    EF = p.Results.EF;
        
    shape = obj.param.scatter.shape;
    
    obj.width = shape.width;
    obj.height = shape.height; % number of unit cells!!!!!!!!
    
    if isempty(Surface_tmp)
            
        if strcmpi(obj.Opt.Lattice_Type, 'Square')
            createH = Square_Lead_Hamiltonians();
            [H0,H1,H1_transverse,coordinates_unitcell] = createH.SquareLattice_Lead_Hamiltonians(obj.param.scatter, obj.width, 'q', obj.q);
        elseif strcmpi(obj.Opt.Lattice_Type, 'SSH')
            createH = Square_Lead_Hamiltonians();
            [H0,H1,H1_transverse,coordinates_unitcell] = createH.SSH_Lead_Hamiltonians(obj.param.scatter, 'q', obj.q);
        elseif strcmp(obj.Opt.Lattice_Type, 'Lieb')
            createH = Square_Lead_Hamiltonians();
            [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.Lieb_Lead_Hamiltonians(obj.params, obj.width, 'q', obj.q);
            obj.H1adj = obj.H1';                  
        elseif strcmpi(obj.Opt.Lattice_Type, 'Graphene')
            End_Type = obj.param.scatter.End_Type;
            createH = Hex_Lead_Hamiltonians();
            [H0,H1,H1_transverse,coordinates_unitcell] = createH.Graphene_Lead_Hamiltonians(obj.param.scatter, obj.width, End_Type, 'q', obj.q);
        elseif strcmpi(obj.Opt.Lattice_Type, 'Silicene')
            End_Type = obj.param.scatter.End_Type;
            createH = Hex_Lead_Hamiltonians();
            [H0,H1,H1_transverse,coordinates_unitcell] = createH.Silicene_Lead_Hamiltonians(obj.param.scatter, obj.width, End_Type, 'q', obj.q);
        elseif strcmpi(obj.Opt.Lattice_Type, 'Graphene_Bilayer')
            End_Type = obj.param.scatter.End_Type;
            createH = Hex_Lead_Hamiltonians();
            [H0,H1,H1_transverse,coordinates_unitcell] = createH.Graphene_Bilayer_Lead_Hamiltonians(obj.param.scatter, obj.width, End_Type, 'q', obj.q);
        elseif ~isempty( obj.Opt.custom_Hamiltonians )
            obj.queryHamiltonians( CustomHamiltonian );   
            return
        else
            error('Unrecognized lattice type, or the claculation options are not set to use outher DFT source.')
        end
    else
       H0 = Surface_tmp.Read( 'H0' );
       H1 = Surface_tmp.Read( 'H1' );
       H1_transverse = Surface_tmp.Read( 'H1_transverse' );
       coordinates_unitcell = Surface_tmp.Read( 'coordinates' ); 
       obj.MagneticFieldApplied  = Surface_tmp.Read( 'MagneticFieldApplied' );
       obj.GaugeTransformationApplied = Surface_tmp.Read( 'GaugeTransformationApplied' );
    end
    
    H1adj = H1';
    obj.M = size(H0,1);
    M = obj.M;
    height = obj.height;
    coordinates = structures( 'coordinates' );
    coordinates.a = coordinates_unitcell.a;
    coordinates.b = coordinates_unitcell.b;
    coordinates.x = zeros( M*height,1 );
    coordinates.y = zeros( M*height,1 );
    if ~isempty( coordinates_unitcell.z )
        coordinates.z = zeros( M*height,1 );
    end
    
    if isfield( coordinates_unitcell, 'spinup' ) && ~isempty(coordinates_unitcell.spinup)
        coordinates.spinup = false( obj.M*obj.height,1 );
    end
    if isfield( coordinates_unitcell, 'BdG_u' ) && ~isempty(coordinates_unitcell.BdG_u)
        coordinates.BdG_u = false( M*height,1 );
    end
    Hscatter = sparse([],[],[],M*height, M*height);
    if ~isempty(obj.q)
        Hscatter_transverse = sparse([],[],[],M*height, M*height);  
    else
        Hscatter_transverse = [];
    end
    
    for idx = 1:height
        Hscatter( (idx-1)*M+1 : idx*M, (idx-1)*M+1:idx*M ) = H0;
        if ~isempty(obj.q)
            Hscatter_transverse( (idx-1)*M+1 : idx*M, (idx-1)*M+1:idx*M ) = H1_transverse;
        end
         if idx < height
       		Hscatter( idx*M+1:(idx+1)*M, (idx-1)*M+1:idx*M ) = H1adj;
       	end
        
       	if idx > 1
       		Hscatter( (idx-2)*M+1:(idx-1)*M, (idx-1)*M+1:idx*M ) = H1;
        end  
        
        coordinates.x( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.x + coordinates_unitcell.a(1)*(idx-1);
        coordinates.y( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.y + coordinates_unitcell.a(2)*(idx-1); 
        if ~isempty( coordinates_unitcell.z )
            coordinates.z( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.z;
        end
        
        if isfield( obj.coordinates, 'spinup' ) && ~isempty(coordinates.spinup)
            coordinates.spinup( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.spinup;
        end
        
        if isfield( coordinates, 'BdG_u' ) && ~isempty(coordinates.BdG_u)
            coordinates.BdG_u( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.BdG_u;
        end
        
    end
    
    obj.coordinates = coordinates;
    obj.Hscatter = Hscatter;
    obj.Hscatter_transverse = Hscatter_transverse;
    
    obj.HamiltoniansCreated   = 1;
    obj.MagneticFieldApplied  = 0;
    obj.GaugeTransformationApplied = 0;
    
    
end

%% shiftFermiEnergy 
%> @brief Shifts the on-site energies in the Scattering region by a given energy.
%> @param Energy The energy value to be used.
    function shiftFermiEnergy( obj, Energy )  
        if isempty( obj.Sscatter )
            energyshift = speye( size( obj.Hscatter ) )*Energy;
        else
            energyshift = obj.Sscatter*Energy;
        end        
        
        obj.Hscatter = obj.Hscatter - energyshift;
        if ~isempty( obj.param.scatter) && ~isempty( obj.param.scatter.epsilon )
            obj.param.scatter.epsilon = obj.param.scatter.epsilon + Energy;
        end
    end

%% projectHamiltonian2Spin
%> @brief Projects the Hamiltonian to a spin states of $s=\pm1$
%> @param s The quantum index of the spin (\pm1)
    function projectHamiltonian2Spin( obj, s )
        if s == 1
            obj.coordinates.x = obj.coordinates.x(obj.coordinates.spinup);
            obj.coordinates.y = obj.coordinates.y(obj.coordinates.spinup);
            obj.Hscatter = obj.Hscatter( obj.coordinates.spinup, obj.coordinates.spinup);
            if ~isempty(obj.q)
                obj.Hscatter_transverse = obj.Hscatter_transverse( obj.coordinates.spinup, obj.coordinates.spinup);
            end
        elseif s == -1
            obj.coordinates.x = obj.coordinates.x(~obj.coordinates.spinup);
            obj.coordinates.y =obj. coordinates.y(~obj.coordinates.spinup);
            obj.Hscatter = obj.Hscatter( ~obj.coordinates.spinup, ~obj.coordinates.spinup);    
            if ~isempty(obj.q)
                obj.Hscatter_transverse = obj.Hscatter_transverse( ~obj.coordinates.spinup, ~obj.coordinates.spinup);
            end
        else
            display('spin should be +/-1', 1)
        end
        
        obj.coordinates.spinup = [];
        obj.M = obj.M/2;
        
        
    end
    
%% Reset
%> @brief Resets all elements in the object.
    function Reset(obj)
        
        obj.kulso_szabfokok      = [];
        obj.Hscatter             = [];
        obj.Hscatter_transverse  = [];
        obj.Sscatter             = [];
        obj.Sscatter_transverse  = [];
        obj.fazis_mtx_scatter    = [];
        obj.fazis_mtx_scatter_t  = [];
        obj.coordinates          = structures('coordinates');

        obj.height               = [];
        obj.width                = [];
        obj.M                    = []; %number of sites in one slab
        obj.HamiltoniansCreated   = 0; 
        obj.HamiltoniansDecimated = 0;
        obj.MagneticFieldApplied  = 0;
        obj.GaugeTransformationApplied = 0;
        obj.OverlapApplied = 0;
        obj.q                          = [];    
    
    end

%% CreateClone
%> @brief Creates a clone of the present Surface_Green_function object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )
            
        ret = CreateHamiltonians(obj.Opt, obj.param);
        ret.Write( 'kulso_szabfokok', obj.kulso_szabfokok);
        ret.Write( 'Hscatter',obj.Hscatter );
        ret.Write( 'Sscatter',obj.Sscatter );
        ret.Write( 'fazis_mtx_scatter', obj.fazis_mtx_scatter);
        ret.Write( 'Hscatter_transverse',obj.Hscatter_transverse );
        ret.Write( 'Sscatter_transverse',obj.Sscatter_transverse );
        ret.Write( 'fazis_mtx_scatter_t', obj.fazis_mtx_scatter_t);
        ret.Write( 'coordinates', obj.coordinates );
        ret.Write( 'HamiltoniansCreated', obj.HamiltoniansCreated);
        ret.Write( 'HamiltoniansDecimated', obj.HamiltoniansDecimated);
        ret.Write( 'MagneticFieldApplied', obj.MagneticFieldApplied );  
        ret.Write( 'GaugeTransformationApplied', obj.GaugeTransformationApplied);
        ret.Write( 'q', obj.q);
        ret.Write( 'height', obj.height );  
        ret.Write( 'width', obj.width);
        ret.Write( 'M', obj.M);        
        ret.Write( 'OverlapApplied', obj.OverlapApplied);                
    end   
    
    
%% RemoveSites %NEW
%> @brief Removes specific sites from the model of the scattering region.
%> @param indexes Logical array. Values set to true will be removed.
    function RemoveSites( obj, indexes )
        
        % update kulso_szabfokok
        if ~isempty( obj.kulso_szabfokok )
            all_sites = 1:size(obj.Hscatter,1);
            sites2remove = all_sites( indexes );
            
            for idx = length(sites2remove):-1:1
               indexes_tmp = find( obj.kulso_szabfokok == sites2remove(idx), 1 );
               if ~isempty(indexes_tmp)
                   error('EQuUs:CreateHamiltonians:RemoveSites', 'kulso_szabfokok cannot be removed from the Hamiltonian.');
               end
               
               indexes_tmp = obj.kulso_szabfokok > sites2remove(idx);
               obj.kulso_szabfokok(indexes_tmp) = obj.kulso_szabfokok(indexes_tmp) - 1;
            end
        end
        
        obj.Hscatter = obj.Hscatter(~indexes, ~indexes);
        if ~isempty(obj.Hscatter_transverse)
            obj.Hscatter_transverse = obj.Hscatter_transverse(~indexes, ~indexes);
        end
        
        if ~isempty(obj.Sscatter )
            obj.Sscatter = obj.Sscatter(~indexes, ~indexes);
        end
        
        if ~isempty(obj.Sscatter_transverse )
            obj.Sscatter_transverse = obj.Sscatter_transverse(~indexes, ~indexes);
        end
        
        if ~isempty(obj.fazis_mtx_scatter )
            obj.fazis_mtx_scatter = obj.fazis_mtx_scatter(~indexes, ~indexes);
        end
        
        if ~isempty(obj.fazis_mtx_scatter_t )
            obj.fazis_mtx_scatter_t = obj.fazis_mtx_scatter_t(~indexes, ~indexes);
        end
        
        % update coordinates
        fieldnames_coord = fieldnames( obj.coordinates );
        for idx = 1:length(fieldnames_coord)
            if strcmpi(fieldnames_coord{idx}, 'a') || strcmpi(fieldnames_coord{idx}, 'b')
                continue
            end
            
            if ~isempty(obj.coordinates.(fieldnames_coord{idx}))
                obj.coordinates.(fieldnames_coord{idx}) = obj.coordinates.(fieldnames_coord{idx})(~indexes);
            end
        end
        

        
    end       

%% saveScatterHamiltonian
%> @brief Saves the Hamiltonian and other data of the scattering region
%> @param filename The string containing the path to the file. (In octave use absolute paths only)
    function saveScatter( filename )
        if ~exist('filename', 'var')
            filename = 'Hscatter.mat';
        end
        save(filename,'Hscatter', 'Hscatter_transverse', 'coordinates', 'param', 'fazis_mtx_scatte', 'fazis_mtx_scatte_t', 'width', 'height', 'M')
    end

%% LoadHamiltonian
%> @brief Loads the Hamiltonian and other data of the scattering region.
%> @param filename The string containing the path to the file. (In octave use absolute paths only)
    function loadScatter( obj, filename )
    if ~exist('filename', 'var')
        filename = 'Hscatter.mat';
    end    
    disp('Loading scatter Hamiltonian from file')
    HandleForLoad = LoadFromFile( filename );
    obj.Hscatter = HandleForLoad.LoadVariable('Hscatter', 'NoEmpty', 'On');
    obj.Hscatter_transverse = HandleForLoad.LoadVariable('Hscatter_transverse', 'NoEmpty', 'On');
    obj.Sscatter = HandleForLoad.LoadVariable('Sscatter', 'NoEmpty', 'On');
    obj.Sscatter_transverse = HandleForLoad.LoadVariable('Sscatter_transverse', 'NoEmpty', 'On');    
    obj.coordinates = HandleForLoad.LoadVariable('coordinates', 'NoEmpty', 'On');
    obj.fazis_mtx_scatter = HandleForLoad.LoadVariable('fazis_mtx_scatte', 'NoEmpty', 'Off');
    obj.fazis_mtx_scatter_t = HandleForLoad.LoadVariable('fazis_mtx_scatte_t', 'NoEmpty', 'Off');
    obj.param = HandleForLoad.LoadVariable('param', 'NoEmpty', 'On');
    obj.M = HandleForLoad.LoadVariable('M', 'NoEmpty', 'On');
    obj.width = HandleForLoad.LoadVariable('width', 'NoEmpty', 'Off');
    obj.height = HandleForLoad.LoadVariable('height', 'NoEmpty', 'On');
	obj.q = HandleForLoad.LoadVariable('q', 'NoEmpty', 'On');
    HandleForLoad.ClearLoadedVariable();
end

%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write(obj, MemberName, input)
        
        if strcmp(MemberName, 'kulso_szabfokok')
                obj.kulso_szabfokok = input;
        elseif strcmp(MemberName, 'Hscatter')
                obj.Hscatter = input;
        elseif strcmp(MemberName, 'Hscatter_transverse')
                obj.Hscatter_transverse = input;
        elseif strcmp(MemberName, 'Sscatter')
                obj.Sscatter = input;
        elseif strcmp(MemberName, 'Sscatter_transverse')
                obj.Sscatter_transverse = input;        
	    elseif strcmp(MemberName, 'height')
			obj.height = input; 
	    elseif strcmp(MemberName, 'width')			
			obj.width = input;  
        elseif strcmp(MemberName, 'M')			
			obj.M = input;                 
        elseif strcmp(MemberName, 'fazis_mtx_scatter')
                obj.fazis_mtx_scatter = input;
        elseif strcmp(MemberName, 'fazis_mtx_scatter_t')
                obj.fazis_mtx_scatter_t = input;
        elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = input;
        elseif strcmp(MemberName, 'param')
                obj.param = input;
		elseif strcmp(MemberName, 'HamiltoniansCreated')
			obj.HamiltoniansCreated   = input; 
	    elseif strcmp(MemberName, 'HamiltoniansDecimated')
			obj.HamiltoniansDecimated = input; 
	    elseif strcmp(MemberName, 'MagneticFieldApplied')			
			obj.MagneticFieldApplied = input;  
        elseif strcmp(MemberName, 'GaugeTransformationApplied')			
			obj.GaugeTransformationApplied = input; 
        elseif strcmp(MemberName, 'OverlapApplied')			
			obj.OverlapApplied = input;                        
        elseif strcmp(MemberName, 'q')			
			obj.q = input;                               
        else
            disp(['No member to write with name ',MemberName]);
        end
    end

%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        if strcmp(MemberName, 'kulso_szabfokok')
            ret = obj.kulso_szabfokok;
        elseif strcmp(MemberName, 'param')
            ret = obj.param;
        elseif strcmp(MemberName, 'Hscatter')
            ret = obj.Hscatter;
        elseif strcmp(MemberName, 'Hscatter_transverse')
            ret = obj.Hscatter_transverse;
        elseif strcmp(MemberName, 'Sscatter')
            ret = obj.Sscatter;
        elseif strcmp(MemberName, 'Sscatter_transverse')
            ret = obj.Sscatter_transverse;            
         elseif strcmp(MemberName, 'height')
            ret = obj.height;
         elseif strcmp(MemberName, 'width')
            ret = obj.width;
         elseif strcmp(MemberName, 'M')
            ret = obj.M;   
         elseif strcmp(MemberName, 'coordinates')
            ret = obj.coordinates;   
		 elseif strcmp(MemberName, 'HamiltoniansCreated')
			ret = obj.HamiltoniansCreated;
	     elseif strcmp(MemberName, 'HamiltoniansDecimated')
			ret = obj.HamiltoniansDecimated; 
         elseif strcmp(MemberName, 'MagneticFieldApplied')	             
			ret = obj.MagneticFieldApplied; 
         elseif strcmp(MemberName, 'OverlapApplied')			            
            ret = obj.OverlapApplied; 
         elseif strcmp(MemberName, 'GaugeTransformationApplied')			
			ret = obj.GaugeTransformationApplied; 
         elseif strcmp(MemberName, 'fazis_mtx_scatter')
            ret = obj.fazis_mtx_scatter;     
         elseif strcmp(MemberName, 'fazis_mtx_sactter_t')
            ret = obj.fazis_mtx_scatter_t;
         elseif strcmp(MemberName, 'q')
            ret = obj.q;
        else
            disp(['No member to read with name ',MemberName]);
        end
    end

%% MemberClear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        if strcmp(MemberName, 'kulso_szabfokok')
            obj.kulso_szabfokok = [];
        elseif strcmp(MemberName, 'Hscatter')
            obj.Hscatter = [];
         elseif strcmp(MemberName, 'Hscatter_transverse')
            obj.Hscatter_transverse = [];
        elseif strcmp(MemberName, 'Sscatter')
            obj.Sscatter = [];
         elseif strcmp(MemberName, 'Sscatter_transverse')
            obj.Sscatter_transverse = [];            
         elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = [];           
        elseif strcmp(MemberName, 'fazis_mtx_scatter')
            obj.fazis_mtx_scatter = [];          
        elseif strcmp(MemberName, 'fazis_mtx_scatter_t')
            obj.fazis_mtx_scatter_t = [];          
        else
            disp(['No member to clear with name ',MemberName]);
        end
        
    end

end % methods public

methods ( Access = private )
    
%% queryHamiltonians
%> @brief Obtains Hamiltonians from the object Custom_Hamiltonians
%> @param cCustom_Hamiltonians An instance of class Custom_Hamiltonians
    function queryHamiltonians( obj, cCustom_Hamiltonians ) 
        if isempty( cCustom_Hamiltonians )
            cCustom_Hamiltonians = Custom_Hamiltonians( obj.Opt ); 
        end
        
        if ~cCustom_Hamiltonians.Read( 'Hamiltonians_loaded' )
            cCustom_Hamiltonians.LoadHamiltonians();
        end
            
        obj.coordinates         = cCustom_Hamiltonians.Read( 'coordinates_scatter' );
        obj.Hscatter            = cCustom_Hamiltonians.Read( 'Hscatter' );
        obj.Hscatter_transverse = cCustom_Hamiltonians.Read( 'Hscatter_transverse' );
        obj.Sscatter            = cCustom_Hamiltonians.Read( 'Sscatter' );
        obj.Sscatter_transverse = cCustom_Hamiltonians.Read( 'Sscatter_transverse' );        
        obj.HamiltoniansCreated   = 1;
        obj.MagneticFieldApplied  = 0;
        obj.GaugeTransformationApplied = 0;        
        
        % determine kulso_szabfokok
        Hcoupling = cCustom_Hamiltonians.Read( 'Hcoupling' );
        
        if iscell( Hcoupling )
            for idx = 1:length( Hcoupling )
               coupled_sites = CoupledSites( Hcoupling{idx} );
               obj.kulso_szabfokok = [obj.kulso_szabfokok; coupled_sites];
            end
            obj.kulso_szabfokok = unique( obj.kulso_szabfokok );
        else
            obj.kulso_szabfokok = CoupledSites( Hcoupling );
        end
        
        
        % tranforming into the BdG model if necessary
        if ~isempty( obj.Opt.BdG ) && obj.Opt.BdG
            obj.Transforms2BdG()
        end
        
        % determine sites coupled to the leads
        function ret = CoupledSites( Hcoupling )
           [rows, cols] =  find(Hcoupling);
           ret = unique( cols );
        end
    end
    
%% Transforms2BdG
%> @brief Transforms the Hamiltonians into a Bogoliubov de Gennes model.
    function Transforms2BdG( obj )
        if isempty( obj.param.scatter.pair_potential )
            error('CreateHamiltonian:Transform2BdG', 'Pair potential need to be set for the scattering region.');
        end
        
        if ~isempty(obj.coordinates.BdG_u)
            obj.display('Hamiltonians already transformed to BdG model.')
            return
        end
        
        fnames = fieldnames( obj.coordinates );
        for idx = 1:length(fnames)
            fname = fnames{idx};
            if strcmp(fname, 'a') || strcmp(fname, 'b')
                continue
            end                    
            obj.coordinates.(fname) = [ obj.coordinates.(fname); obj.coordinates.(fname) ];
        end
                
        obj.coordinates.BdG_u = [ true(size(obj.Hscatter,1),1); false(size(obj.Hscatter,1),1)];
                
        length_kulso_szabfokok = length(obj.kulso_szabfokok);
        obj.kulso_szabfokok = [obj.kulso_szabfokok(1:length_kulso_szabfokok); obj.kulso_szabfokok(1:length_kulso_szabfokok)+size(obj.Hscatter,1) ];        
        
        % transforming the Hamiltonians
        pair_potential = obj.param.scatter.pair_potential;        
        site_num = size( obj.Hscatter, 1 );        
        obj.coordinates.BdG_u = [true( site_num, 1); false( site_num, 1) ];
        
        if isempty( obj.Sscatter )
            Sscatter = speye(size(obj.Hscatter));
        else
            Sscatter = obj.Sscatter;
        end
        
        obj.Hscatter = [ obj.Hscatter, Sscatter*pair_potential;
                         Sscatter*conj(pair_potential), -conj(obj.Hscatter) ];
        
        % transforming the overlap integrals
        if isempty( obj.Sscatter )
            return;
        end
        
        obj.Sscatter = [ obj.Sscatter, sparse([],[],[], site_num, site_num );
                         sparse([],[],[], site_num, site_num ), obj.Sscatter ];
         
        
    end
    
%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin)
        
        p = inputParser;
        p.addParamValue('q', []);
        
        p.parse(varargin{:});

        obj.q = p.Results.q;

    end


end %methods protected

end % Global end

