%%   Eotvos Quantum Transport Utilities - CreateLeadHamiltonians
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%>  Interface to create and store Hamiltonian of the translational invariant leads.
%%
classdef CreateLeadHamiltonians < handle & Messages
    
    
    properties ( Access = protected )
    %> An instance of the structure lead_param
    param
    %> The orientation of the lead. Set +1 is the "incoming" direction of the propagating states is defined in the +x or +y direction, and "-1" otherwise.
    Lead_Orientation
    %> The id number of the current lead.
    Hanyadik_Lead
    %> Function handle to calculate the infinite Greens function.
    End_Type
    %> The number of the sites in the cross section.
    M 
    %> An instance of the structure lead_param
    params
    %> The tranverse momentum for transverse computations.
    q
    %> List of sites in the unit cell that should be kept after decimation.
    kulso_szabfokok
    %> An instance of the structure coordinates.
    coordinates
    %> The Hamiltonian of a unit cell.
    H0
    %> The coupling Hamiltonian between the unit cells.
    H1
    %> The coupling Hamiltonian between the unit cells in the opposite direction as H1. (For complex energies they differ from each other.)
    H1adj
    %> Obsolete
    H00 
    %> The transverse coupling between the slabs for transverse calculations.
    H1_transverse
    %> The overlap integrals of a unit cell.
    S0
    %> The overlap integrals between the unit cells.
    S1
    %> The overlap integrals between the slabs for transverse calculations.
    S1_transverse
    %> The matrix of the Peierls phases in the unit cell.
    fazis_mtx_H0
    %> The matrix of the Peierls phases in the coupling matrix between the unit cells.
    fazis_mtx_H1
    %> The matrix of the Peierls phases in the transverse coupling matrix between the unit cells.
    fazis_mtx_H1t
    %> A logical value. True if the Hamiltonians were created, false otherwise.
    HamiltoniansCreated
    %> A logical value. True if the Hamiltonians were decimated, false otherwise.
    HamiltoniansDecimated 
    %> A logical value. True if the overlap integrals were applied, false otherwise.
    OverlapApplied    
    %> A logical value. True if magnetic field was applied in the Hamiltonians, false otherwise.
    MagneticFieldApplied
    %> A logical value. True if a gauge transformation was incorporated into the Hamiltonians or false otherwise.
    GaugeTransformationApplied
    %> list of optional parameters (see http://www.mathworks.com/help/matlab/ref/varargin.html for details)
    varargin
    end


methods ( Access = public )
%% constructorof the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = CreateLeadHamiltonians(Opt, param, varargin)    
        obj = obj@Messages( Opt );
        obj.param = param;
        obj.varargin = varargin;

        if strcmpi(class(obj), 'CreateLeadHamiltonians')
	        obj.Reset();
        end
        
    end
    
    
    
%% ApplyOverlapMatrices
%> @brief Applies the overlap matrices to the Hamiltonians: K = H-ES
%> @param E The energy value.
    function ApplyOverlapMatrices(obj, E)
        
        if obj.OverlapApplied
            obj.display('Overlap matrices were already applied.');
            return;
        end
        
        if ~isempty( obj.S0 )
            obj.H0 = obj.H0 - E*obj.S0;
            obj.OverlapApplied = true;
        end
        
        if ~isempty( obj.S1 )
            obj.H1 = obj.H1 - E*obj.S1;
            obj.H1adj = obj.H1adj - E*obj.S1';
        end
        
        
        if ~isempty( obj.S1_transverse )
            obj.H1_transverse = obj.H1_transverse - E*obj.S1_transverse;
        end
        
    end
    
    
%% CreateHamiltonians
%> @brief Creates the Hamiltonians H_0 and H_1 of the lead. The created Hamiltonians are stored by within the object. 
%> @param varargin Optional parameters, see the web documentation for details.
    function CreateHamiltonians(obj, varargin )
        
        p = inputParser;
        p.addParamValue('toSave', 0);
        p.addParamValue('CustomHamiltonian', []);  
        p.parse(varargin{:});

        toSave  = p.Results.toSave;
        CustomHamiltonian = p.Results.CustomHamiltonian;
        
        obj.setM();
        
        if obj.Opt.Hleads == 1
            HandleForLoad = LoadFromFile(['Hamiltoni_Lead_',num2str(obj.Hanyadik_Lead),'.mat']);
            obj.H0            = HandleForLoad.LoadVariable('H0', 'NoEmpty', 'On');
            obj.H1            = HandleForLoad.LoadVariable('H1', 'NoEmpty', 'On');
            obj.H1_transverse = HandleForLoad.LoadVariable('H1_transverse', 'NoEmpty', 'On');
            obj.H1adj         = obj.H1';
            obj.kulso_szabfokok = HandleForLoad.LoadVariable('kulso_szabfokok');
            obj.Lead_Orientation = HandleForLoad.LoadVariable('Lead_Orientation');
            obj.params        = HandleForLoad.LoadVariable('params');
            obj.fazis_mtx_H0  = HandleForLoad.LoadVariable('fazis_mtx_H0', 'NoEmpty', 'Off');
            obj.fazis_mtx_H1  = HandleForLoad.LoadVariable('fazis_mtx_H1', 'NoEmpty', 'Off');
            obj.fazis_mtx_H1t = HandleForLoad.LoadVariable('fazis_mtx_H1t', 'NoEmpty', 'Off');
            HandleForLoad.ClearLoadedVariable();            
        else
            if strcmpi(obj.Opt.Lattice_Type, 'Square')
                createH = Square_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.SquareLattice_Lead_Hamiltonians(obj.params, obj.M, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = 1:obj.M;
            elseif strcmpi(obj.Opt.Lattice_Type, 'SSH')
                createH = Square_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.SSH_Lead_Hamiltonians(obj.params, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = 1;
            elseif strcmp(obj.Opt.Lattice_Type, 'Lieb')
                createH = Square_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.Lieb_Lead_Hamiltonians(obj.params, obj.M, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = 3:3:3*obj.M;                
            elseif strcmpi(obj.Opt.Lattice_Type, 'H') || strcmp(obj.Opt.Lattice_Type, 'Graphene')
                createH = Hex_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.Graphene_Lead_Hamiltonians(obj.params, obj.M, obj.End_Type, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = (1:obj.M);
            elseif strcmpi(obj.Opt.Lattice_Type, 'Graphene_Bilayer')
                createH = Hex_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.Graphene_Bilayer_Lead_Hamiltonians(obj.params, obj.M, obj.End_Type, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = [1:obj.M, size(obj.H0,1)/2 + (1:obj.M)];
                obj.M = 2*obj.M;
            elseif strcmpi(obj.Opt.Lattice_Type, 'Silicene')
                createH = Hex_Lead_Hamiltonians();
                [obj.H0,obj.H1,obj.H1_transverse,obj.coordinates] = createH.Silicene_Lead_Hamiltonians(obj.params, obj.M, obj.End_Type, 'q', obj.q);
                obj.H1adj = obj.H1';
                obj.kulso_szabfokok = [1:obj.M, size(obj.H0,1)/2 + (1:obj.M)];          
                obj.M = 2*obj.M;
            elseif ~isempty( obj.Opt.custom_Hamiltonians )
                if isempty( CustomHamiltonian )
                    CustomHamiltonian = Custom_Hamiltonians( obj.Opt ); 
                end
        
                if ~CustomHamiltonian.Read( 'Hamiltonians_loaded' )
                    CustomHamiltonian.LoadHamiltonians();
                end
            
                obj.coordinates   = CustomHamiltonian.Read( 'coordinates' );
                obj.coordinates   = obj.coordinates{obj.Hanyadik_Lead};
                obj.H0            = CustomHamiltonian.Read( 'H0' );
                obj.H0            = obj.H0{obj.Hanyadik_Lead};
                obj.H1            = CustomHamiltonian.Read( 'H1' );
                obj.H1            = obj.H1{obj.Hanyadik_Lead};
                obj.H1adj         = obj.H1';
                obj.H1_transverse = CustomHamiltonian.Read( 'H1_transverse' ); 
                obj.H1_transverse = obj.H1_transverse{obj.Hanyadik_Lead};
                obj.S0            = CustomHamiltonian.Read( 'S0' );
                if iscell( obj.S0 )
                    obj.S0            = obj.S0{obj.Hanyadik_Lead};
                end
                obj.S1            = CustomHamiltonian.Read( 'S1' );
                if iscell( obj.S1 )
                    obj.S1            = obj.S1{obj.Hanyadik_Lead};
                end
                obj.S1_transverse = CustomHamiltonian.Read( 'S1_transverse' ); 
                if iscell( obj.S1_transverse )
                    obj.S1_transverse = obj.S1_transverse{obj.Hanyadik_Lead};      
                end
                obj.M             = size(obj.H0,1);
                obj.kulso_szabfokok = 1:obj.M;
            else
                error('Unrecognized lattice type, or the claculation options are not set to use outher DFT source.')
            end
            
            obj.kulso_szabfokok = [obj.kulso_szabfokok, obj.kulso_szabfokok+size(obj.H0,1)];            

            obj.Transform2BdG();
                        
            if toSave
                saveLeads();
            end
            
            obj.HamiltoniansCreated = 1;
            obj.HamiltoniansDecimated = 0;
            obj.MagneticFieldApplied  = 0;
            obj.GaugeTransformationApplied = 0;
        end
  
        
        
        
    end
    
%% Transform2BdG
%> @brief Transforms the Hamiltonians and the overlap matrices into the BdG model.
    function Transform2BdG( obj )
    
        if isempty(obj.Opt.BdG) || ~obj.Opt.BdG
            return
        end
        
        if ~isempty( obj.coordinates.BdG_u )
            % already transformed into BdG model
            return
        end
        
        fnames = fieldnames( obj.coordinates );
        for idx = 1:length(fnames)
            fname = fnames{idx};
            if strcmp(fname, 'a') || strcmp(fname, 'b')
                continue
            end                    
            obj.coordinates.(fname) = [ obj.coordinates.(fname); obj.coordinates.(fname) ];
        end
                
        obj.coordinates.BdG_u = [ true(size(obj.H0,1),1); false(size(obj.H0,1),1)];
                
        length_kulso_szabfokok = length(obj.kulso_szabfokok)/2;
        obj.kulso_szabfokok = [obj.kulso_szabfokok(1:length_kulso_szabfokok), obj.kulso_szabfokok(1:length_kulso_szabfokok)+size(obj.H0,1), ...
                   obj.kulso_szabfokok(length_kulso_szabfokok+1:end) + size(obj.H0,1), obj.kulso_szabfokok(length_kulso_szabfokok+1:end) + 2*size(obj.H0,1)];
               
        pair_potential = obj.params.pair_potential;
        
        % transforming the Hamiltonians
        if isempty(obj.S0)
            S0 = speye(size(obj.H0));
        else
            S0 = obj.S0;
        end
        
        obj.H0 = [obj.H0, S0*pair_potential; S0*conj(pair_potential), -conj(obj.H0)];
        
        if isempty(obj.S1)
            S1 = sparse([],[],[], size(obj.H1,1), size(obj.H1,2));
        else
            S1 = obj.S1;
        end        
        
        obj.H1 = [obj.H1, S1*pair_potential; S1*conj(pair_potential), -conj(obj.H1)];
        obj.H1adj = [obj.H1adj, S1'*pair_potential; S1'*conj(pair_potential), -conj(obj.H1adj)];
        
        if isempty(obj.S1_transverse)
            S1_transverse = sparse([],[],[], size(obj.H1_transverse,1), size(obj.H1_transverse,2));
        else
            S1_transverse = obj.S1_transverse;
        end                
        
        obj.H1_transverse = [obj.H1_transverse, S1_transverse*pair_potential; S1_transverse*conj(pair_potential), -conj(obj.H1_transverse)];
        
        % transforming th eoverlap integrals  
        if ~isempty( obj.S0 )
            obj.S0 = [obj.S0, sparse([], [], [], size(obj.S0,1), size(obj.S0,2)); sparse([], [], [], size(obj.S0,1), size(obj.S0,2)), obj.S0];
        end
        
        if ~isempty(obj.S1)
            obj.S1 = [obj.S1, sparse([], [], [], size(obj.S1,1), size(obj.S1,2)); sparse([], [], [], size(obj.S1,1), size(obj.S1,2)), obj.S1];
        end
        
        if ~isempty( obj.S1_transverse )
            obj.S1_transverse = [obj.S1_transverse, sparse([], [], [], size(obj.S1_transverse,1), size(obj.S1_transverse,2)); ...
                sparse([], [], [], size(obj.S1_transverse,2), size(obj.S1_transverse,1)), obj.S1_transverse];
        end
                
                
        obj.M = 2*obj.M;        
        
    end
        
%% CalcSpektrum
%> @brief Calculates the band structure of the lead.
%> @param varargin Optional parameters, see the web documentation for details.
    function [spectrum, WaveFnc] = CalcSpektrum( obj, varargin )
        
        p = inputParser;
        p.addParamValue('toPlot', 0, @isscalar);  
        p.addParamValue('ka_min', -pi, @isscalar);  
        p.addParamValue('ka_max', pi, @isscalar);  
        p.addParamValue('ka_num', 300, @isscalar);
        p.addParamValue('ka_vec', [] );
        p.addParamValue('center', 0.001);  
        p.addParamValue('db', min([10,size(obj.H0,1)]), @isscalar); 
		p.addParamValue('offset', mean(diag(obj.H0)) ); 
        p.addParamValue('calcWaveFnc', false ); 
        
        p.parse(varargin{:});    
        toPlot     = p.Results.toPlot;
        ka_min     = p.Results.ka_min;
        ka_max     = p.Results.ka_max;
        ka_num     = p.Results.ka_num;
        ka_vec     = p.Results.ka_vec;
        center = p.Results.center;
        db             = p.Results.db;
		offset     = p.Results.offset;
        calcWaveFnc = p.Results.calcWaveFnc;
        
        
	if obj.HamiltoniansDecimated
		obj.display(' Hamiltonians are decimated. Please recreate Hamiltonians to calculate spectrum.')
		spectrum = NaN;
		return
    end
    
    if obj.OverlapApplied
		obj.display('Overlap integrals were applied to Hamiltonians. Please recreate Hamiltonians to calculate spectrum.')
		spectrum = NaN;
		return
    end
    
    obj.display('Calculating spectrum')   
    
    
    deltak = (ka_max-ka_min)/ka_num;

    ka = ka_min;
    H1loc = obj.H1;
    H0loc = obj.H0 - eye(size(obj.H0))*offset;
    H1_transverse_loc = obj.H1_transverse;
    S0_loc = obj.S0;
    S1_loc = obj.S1;
    S1_transverse_loc = obj.S1_transverse;
    
    q = obj.q;
    
    tic
    hsecular_H =@secular_H;
    
    if isempty(ka_vec)
        ka_vec = ka_min:deltak:ka_max;
    end    
    
    spectrum = cell(length(ka_vec),1);
    if calcWaveFnc
        WaveFnc = struct( ...
            'WaveFnc', cell(length(ka_vec),1), ...
            'ka', cell(length(ka_vec),1), ...
            'E', cell(length(ka_vec),1) ...
            );
    else
        WaveFnc = [];
    end
    
    
    for idx=1:length(ka_vec)
        H    = secular_H( H0loc, H1loc, H1_transverse_loc, ka_vec(idx));
        if isempty( S0_loc ) && isempty( S1_loc )
            if calcWaveFnc
                [WaveFnc_tmp, E]  = eigs(H, db, center);
                E = diag(E);
                WaveFnc(idx).WaveFnc = WaveFnc_tmp;
                WaveFnc(idx).E = E;
                WaveFnc(idx).ka = ka_vec(idx);
            else
                E    = eigs(H, db, center);%2.97*sqrt(9/2*eta_B));
            end
        else
            S    = secular_H( S0_loc, S1_loc, S1_transverse_loc, ka_vec(idx));
            if calcWaveFnc
                [WaveFnc_tmp, E]  = eigs(H, S, db, center);
                E = diag(E);
                WaveFnc.WaveFnc{idx} = WaveFnc_tmp;
                WaveFnc.E{idx} = E;
                WaveFnc.ka(idx) = ka_vec(idx);
            else
                E    = eigs(H, S, db, center);%2.97*sqrt(9/2*eta_B));
            end
        end
        spectrum{idx} = [ones(length(E),1)*ka_vec(idx), E];        
    end
    spectrum = cell2mat(spectrum);
    toc
 
    obj.display('Spectrum calculated')
    
    if ~toPlot
        return
    end
    
    figure1 = figure();
    fontsize = 9;
    
    spectrum(:,2) = real(spectrum(:,2));
    x_lim = [min(spectrum(:,1)) max(spectrum(:,1))];
    y_lim = [min(spectrum(:,2)) max(spectrum(:,2))];

    Position = [0.5 0.58 0.33 0.4];
    axes_spectrum = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,...
                'xlim', x_lim,...
                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
    plot(spectrum(:,1), spectrum(:,2),'.', 'MarkerSize', 4, 'Parent', axes_spectrum )  
        
    % Create xlabel
    xlabel_position = [0 0 0];
    xlabel('ka','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
    xlabel_handle = get(axes_spectrum,'XLabel');  
    set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

    % Create ylabel
    ylabel_position = [0 0 0];
    ylabel('E [eV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
    ylabel_handle = get(axes_spectrum,'YLabel'); 
    set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);  

    
        % --------------------------------------
        % Hamiltonian for the secular equation
        function H = secular_H(H0,H1, H1_transverse, k)
            
            if ~isempty(q) && ~obj.HamiltoniansDecimated
               H0 = H0 + H1_transverse*diag(exp(-1i*q)) + H1_transverse'*diag(exp(1i*q)); 
            end
            H = H0 + H1*exp(1i*k) + H1'*exp(-1i*k);
        
        end

 
    end   
    
%% saveLeads
%> @brief Save Lead Hamiltonians  into a file 'Hamiltoni_Lead_' + num2str(Hanyadik_Lead) + '.mat'.  
    function saveLeads( obj )
        save(['Hamiltoni_Lead_',num2str(obj.Hanyadik_Lead),'.mat'], 'H0', 'H1', 'kulso_szabfokok', 'Lead_Orientation', 'fazis_mtx_H0', 'fazis_mtx_H1', 'params');
    end        
    
%% ShiftCoordinates
%> @brief Shifts the coordinates of the sites by an integer multiple of the lattice vector.
%> @param shift Integer by which the coordinates are shifted.
    function ShiftCoordinates( obj, shift )    
        if isempty(shift) || shift == 0
            return
        end
        obj.coordinates.x = obj.coordinates.x + shift*obj.coordinates.a(1);
        obj.coordinates.y = obj.coordinates.y + shift*obj.coordinates.a(2);        
    end    
    
    
    
%% ShiftLead
%> @brief Shifts the on-site energies in the leads by a given energy.
%> @param Energy The enrgy value.
    function ShiftLead( obj, Energy )        
        obj.H0 = obj.H0 + sparse(1:size(obj.H0,1), 1:size(obj.H0,2),Energy,size(obj.H0,1),size(obj.H0,2)); 
        obj.params.epsilon = obj.params.epsilon + Energy;
    end

%% AddPotential
%> @brief Adds on-site potential to the Hamiltonian H0.
%> @param V The potential calculated on the sites.
    function AddPotential( obj, V )        

        if ( size(V,2) == 1 ) || ( size(V,1) == 1 )
            if length(V) == size(obj.H0,1)
                obj.H0 = obj.H0 + sparse(1:size(obj.H0,1), 1:size(obj.H0,2),V,size(obj.H0,1),size(obj.H0,2));
            else
                disp(' Wrong dimension of potential: 1')
                return
            end
        elseif norm(size(V) - size(obj.H0)) < 1e-6
            obj.H0 = obj.H0 + V;            
        else
            disp(' Wrong dimension of potential: 2')
            return
        end
        
        obj.display(' Potential added to lead')
    end

%% isSuperconducting
%> @brief Test, whether the lead is in the superconducting phase or not.
%> @return True if the lead is superconducting, false otherwise.
    function ret = isSuperconducting( obj )
        
        if ~obj.Opt.BdG
            ret = false;
            return;
        end
        
        if abs(obj.params.pair_potential) >= 1e-10
            ret = true;
        else
            ret = false;
        end
        
        return
            
    
    end

%% CreateClone
%> @brief Creates a clone of the present Surface_Green_function object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )
        
        
        ret = CreateLeadHamiltonians(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', obj.Hanyadik_Lead,...
                                            'Lead_Orientation', obj.Lead_Orientation, ...
                                            'q', obj.q);                    
                                        
        ret.Write( 'H0', obj.H0); 
        ret.Write( 'H1', obj.H1);
        ret.Write( 'H1adj', obj.H1adj);
        ret.Write( 'H1_transverse', obj.H1_transverse);
        ret.Write( 'fazis_mtx_H0', obj.fazis_mtx_H0); 
        ret.Write( 'fazis_mtx_H1', obj.fazis_mtx_H1);        
        ret.Write( 'fazis_mtx_H1t', obj.fazis_mtx_H1t);
        ret.Write( 'H00', obj.H00);
        ret.Write( 'kulso_szabfokok', obj.kulso_szabfokok);
		ret.Write( 'coordinates', obj.coordinates);
        ret.Write( 'params', obj.params);
        ret.Write( 'HamiltoniansCreated', obj.HamiltoniansCreated);
        ret.Write( 'OverlapApplied', obj.OverlapApplied);
        ret.Write( 'HamiltoniansDecimated', obj.HamiltoniansDecimated);
        ret.Write( 'MagneticFieldApplied', obj.MagneticFieldApplied); 
        ret.Write( 'GaugeTransformationApplied', obj.GaugeTransformationApplied);
        ret.Write( 'M', obj.M); 
        
    end   
    
%% Reset
%> @brief Resets all elements in the object.
    function Reset( obj )
        obj.Lead_Orientation  = [];
        obj.Hanyadik_Lead     = [];
        obj.End_Type     = [];
        obj.M             = [];
        obj.params        = [];  %Lead parameters
        obj.q             = []; % the transverse momentum
        
        obj.inputparser( obj.varargin{:});

        obj.kulso_szabfokok = [];
        obj.coordinates   = []; 
        obj.H0            = [];
        obj.H1            = [];
        obj.H1adj         = [];
        obj.H00           = [];
        obj.H1_transverse = [];
        obj.fazis_mtx_H0  = [];
        obj.fazis_mtx_H1  = [];
        obj.fazis_mtx_H1t = [];

        
        obj.HamiltoniansCreated   = 0; 
        obj.HamiltoniansDecimated = 0;
        obj.OverlapApplied = 0;
        obj.MagneticFieldApplied  = 0;
        obj.GaugeTransformationApplied = 0;
        
        
        
        obj.setM();
        
        if isempty( obj.Hanyadik_Lead )
            obj.params        = obj.param.scatter;  %Lead parameters
        else
            obj.params        = obj.param.Leads{obj.Hanyadik_Lead};  %Lead parameters
        end
        
        obj.End_Type = obj.params.End_Type;        
    end    
    

    
%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write(obj, MemberName, input)
        
        if strcmp(MemberName, 'H0')
            obj.H0 = input;
        elseif strcmp(MemberName, 'H1')
            obj.H1 = input; 
        elseif strcmp(MemberName, 'H1adj')
            obj.H1adj = input;    
        elseif strcmp(MemberName, 'H00')
            obj.H00 = input;
        elseif strcmp(MemberName, 'H1_transverse')
            obj.H1_transverse = input;      
        elseif strcmp(MemberName, 'S0')
            obj.S0 = input;
        elseif strcmp(MemberName, 'S1')
            obj.S1 = input;          
        elseif strcmp(MemberName, 'S1_transverse')
            obj.S1_transverse = input;              
        elseif strcmp(MemberName, 'M')
            obj.M = input;            
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            obj.fazis_mtx_H0 = input; 
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            obj.fazis_mtx_H1 = input;  
        elseif strcmp(MemberName, 'fazis_mtx_H1t')            
            obj.fazis_mtx_H1t = input;              
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
            M = obj.M;
            obj.H00   = input(1:M,1:M);
            obj.H1    = input(1:M, M+1:2*M);
            obj.H0    = input(M+1:2*M, M+1:2*M);
            obj.H1adj = input( M+1:2*M, 1:M);            
		elseif strcmp(MemberName, 'HamiltoniansCreated')
			obj.HamiltoniansCreated   = input; 
	    elseif strcmp(MemberName, 'OverlapApplied')
			obj.OverlapApplied = input;    
        elseif strcmp(MemberName, 'HamiltoniansDecimated')
			obj.HamiltoniansDecimated = input;    
	    elseif strcmp(MemberName, 'MagneticFieldApplied')			
			obj.MagneticFieldApplied = input; 
        elseif strcmp(MemberName, 'GaugeTransformationApplied')			
			obj.GaugeTransformationApplied = input; 
        elseif strcmp(MemberName, 'kulso_szabfokok')
            obj.kulso_szabfokok = input;
        elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = input;
        elseif strcmp(MemberName, 'epsilon')
            obj.params.epsilon = input;
        elseif strcmp(MemberName, 'vargamma')
            obj.params.vargamma = input;
        elseif strcmp(MemberName, 'params')
            obj.params = input;
        elseif strcmp(MemberName, 'Hanyadik_Lead')
            obj.Hanyadik_Lead = input; 
        elseif strcmp(MemberName, 'Lead_Orientation')
            obj.Lead_Orientation = input;
        elseif strcmp(MemberName, 'param')
            obj.param = input;
            obj.Reset()
         elseif strcmp(MemberName, 'q')
            obj.q = input; 
        else
            obj.display(['No member to write with name ',MemberName]);
        end
    end
%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        if strcmp(MemberName, 'H0')
            ret = obj.H0;
        elseif strcmp(MemberName, 'H1')
            ret = obj.H1;
        elseif strcmp(MemberName, 'H1adj')
            ret = obj.H1adj;
        elseif strcmp(MemberName, 'H00')
            ret = obj.H00;
        elseif strcmp(MemberName, 'H1_transverse')
            ret = obj.H1_transverse;            
        elseif strcmp(MemberName, 'S0')
            ret = obj.S0;
        elseif strcmp(MemberName, 'S1')
            ret = obj.S1;        
        elseif strcmp(MemberName, 'S1_transverse')
            ret = obj.S1_transverse;                
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            ret = obj.fazis_mtx_H0; 
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            ret = obj.fazis_mtx_H1;
        elseif strcmp(MemberName, 'fazis_mtx_H1t')            
            ret = obj.fazis_mtx_H1t;            
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
            if ~isempty(obj.q) && ~obj.HamiltoniansDecimated
                H0loc = obj.H0 + obj.H1_transverse*diag(exp(-1i*obj.q)) + obj.H1_transverse'*diag(exp(1i*obj.q));
            else
                H0loc = obj.H0;
            end            
            ret = [H0loc,obj.H1;obj.H1adj,H0loc];
		elseif strcmp(MemberName, 'HamiltoniansCreated')
			ret = obj.HamiltoniansCreated; 
	    elseif strcmp(MemberName, 'HamiltoniansDecimated')
			ret = obj.HamiltoniansDecimated;   
        elseif strcmp(MemberName, 'OverlapApplied')
			ret = obj.OverlapApplied;   
	    elseif strcmp(MemberName, 'MagneticFieldApplied')			
			ret = obj.MagneticFieldApplied;  
        elseif strcmp(MemberName, 'GaugeTransformationApplied')			
			ret = obj.GaugeTransformationApplied;  
        elseif strcmp(MemberName, 'kulso_szabfokok')
            ret = obj.kulso_szabfokok;
        elseif strcmp(MemberName, 'coordinates')
            ret = obj.coordinates;
        elseif strcmp(MemberName, 'M')
            ret = obj.M;
        elseif strcmp(MemberName, 'epsilon')
            ret = obj.params.epsilon;
        elseif strcmp(MemberName, 'vargamma')
            ret = obj.params.vargamma;            
        elseif strcmp(MemberName, 'params')
            ret = obj.params;
        elseif strcmp(MemberName, 'param')
            ret = obj.param;
        elseif strcmp(MemberName, 'Lead_Orientation')
            ret = obj.Lead_Orientation;
        elseif strcmp(MemberName, 'Hanyadik_Lead')
            ret = obj.Hanyadik_Lead;      
        elseif strcmp(MemberName, 'q')
            ret = obj.q;  
        else            
            obj.display(['No member to read with name ',MemberName]);
            %ret = [];
        end
    end
%% Clear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        if strcmp(MemberName, 'H0')
            obj.H0 = [];
        elseif strcmp(MemberName, 'H1')
            obj.H1 = [];
        elseif strcmp(MemberName, 'H1adj')
            obj.H1adj = [];
        elseif strcmp(MemberName, 'H00')
            obj.H00 = [];
        elseif strcmp(MemberName, 'H1_transverse')
            obj.H1_transverse = [];
        elseif strcmp(MemberName, 'S0')
            obj.S0 = [];
        elseif strcmp(MemberName, 'S1')
            obj.S1 = [];
        elseif strcmp(MemberName, 'S1_transverse')
            obj.S1_transverse = [];            
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            obj.fazis_mtx_H0 = [];            
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            obj.fazis_mtx_H1 = [];
        elseif strcmp(MemberName, 'fazis_mtx_H1t')            
            obj.fazis_mtx_H1t = [];    
        elseif strcmp(MemberName, 'kulso_szabfokok')
            obj.kulso_szabfokok = [];
        elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = [];  
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
        else
            obj.display(['No member to clear with name ',MemberName]);
        end
        
    end    
    
end % public methods


methods ( Access = protected )
   
%% setM
%> @brief Updates the number of sites in the cross section.
    function setM( obj )
        if isempty( obj.Hanyadik_Lead )
            obj.M             = obj.param.scatter.shape.width;
        else
            obj.M             = obj.param.Leads{obj.Hanyadik_Lead}.M;
        end  
    end
    
end % protected methods
    
    
methods (Access=private)        
    

%% input parsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function inputparser( obj, varargin )
        p = inputParser;
        p.addParamValue('Hanyadik_Lead', []);
        p.addParamValue('Lead_Orientation', 1);
        p.addParamValue('q', []);
        p.parse(varargin{:});

        obj.Lead_Orientation  = p.Results.Lead_Orientation;
        obj.Hanyadik_Lead     = p.Results.Hanyadik_Lead;
        obj.q                 = p.Results.q;
        

        
        
    end

end % private methods


    
    
    
end
