%%   Eotvos Quantum Transport Utilities - Custom_Hamiltonians
%    Copyright (C) 2015-2016 Peter Rakyta, Ph.D., David Visontai, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%   Interface to import custom Hamiltonians provided by other codes or created
%   manually
%%
classdef Custom_Hamiltonians < handle & Messages

properties (Access = protected )
    % An instance of structure param
    param
    % Hamiltonian of one slab in the leads
    H0 = []; 
    % coupling between the slabs of the leads
    H1 = []; 
    % transverse coupling between the unit cells of the lead
    H1_transverse = [];
    % Hamiltonian of the scattering region
    Hscatter = []; 
    % transverse coupling for the scattering region
    Hscatter_transverse = [];
    % coupling to the leads
    Hcoupling = []; 
    % overlap integrals of one slab in the leads
    S0 = []; 
    % overlap integrals between the slabs of the leads
    S1 = []; 
    % overlap integrals for the transverse coupling between the unit cells of the lead
    S1_transverse = [];
    % overlap integrals of the scattering region
    Sscatter = []; 
    % overlap integrals for the transverse coupling in the scattering region
    Sscatter_transverse = [];
    % overlap integrals between the scattering region and the leads
    Scoupling = []; 
    %coordinates of the leads
    coordinates = []; 
    % coordinates of the scattering region
    coordinates_scatter = []; 
    % logical value: true if Hamiltonians are loaded, false otherwise.
    Hamiltonians_loaded = false;
    % The Fermi energy
    EF
    % Function Handle for the custom Hamiltonians
    CustomHamiltoniansHandle
end



methods (Access = public )
    
    
%% Constructor of the class
function obj = Custom_Hamiltonians( Opt, param, varargin )
    obj = obj@Messages( Opt );
    obj.param = param;
    obj.Reset();
    obj.InputParsing( varargin{:} )
    
end


%% LoadHamiltonians
%> @brief Obtain the Hamiltonians from the external source
%> @param varargin Optional parameters, see the documentation.
function LoadHamiltonians(obj, varargin)
    
    p = inputParser;
    
    p.addParamValue('q', []);
    p.addParamValue('Extmol', 'Extended_Molecule.mat');
    p.addParamValue('GollumInput', 'input.mat');
    p.addParamValue('WorkingDir', pwd);
    
    p.parse(varargin{:});
    
    
    q = p.Results.q;
    Extmol = p.Results.Extmol;
    GollumInput = p.Results.GollumInput;
    WorkingDir  = p.Results.WorkingDir;     
    
    % Siesta Hamiltonians
    if strcmpi( obj.Opt.custom_Hamiltonians, 'siesta' )

    
        % Coordinatzes needs to be determined : TODO
        [obj.H0, obj.S0, obj.H1, obj.S1, obj.coordinates, ...
            obj.Hscatter, obj.Sscatter, obj.coordinates_scatter, obj.Hcoupling, obj.Scoupling] = obj.Siesta_Equus_interface(WorkingDir, GollumInput, Extmol);

        %!!!!!!!!!!!!!!!!!! Correct in the Gollum_Equus_interface
        for idx = 1: length(obj.Hcoupling)
            obj.Hcoupling{idx} = obj.Hcoupling{idx}';
            obj.Scoupling{idx} = obj.Scoupling{idx}';            
        end
             
    
        obj.H1_transverse = cell(size(obj.H0));
        
        
        
    %**************************************** develpoment lines *******************************************************
%obj.H0{1} = obj.H0{1} + 0.5*obj.S0{1};
%obj.H1{1} = obj.H1{1} + 0.5*obj.S1{1};
%obj.H0{2} = obj.H0{2} + speye(size(obj.H0{2}));
%     for idx = 1:length(obj.H0)
%         obj.S0{idx} = [];speye(size(obj.S0{idx}));
%         obj.S1{idx} = [];sparse([],[],[], size(obj.H1{idx},1), size(obj.H1{idx},2));
%     end
%     obj.Sscatter = [];speye(size(obj.Hscatter));
%     for idx =1:length(obj.Hcoupling)
%        obj.Scoupling{idx} = [];sparse([],[],[], size(obj.Hcoupling{idx},1), size(obj.Hcoupling{idx},2));
%     end
    
    
    
    % Custom created Hamiltonians
    elseif strcmpi( obj.Opt.custom_Hamiltonians, 'custom' )
        
        if isempty(obj.CustomHamiltoniansHandle)
            error('EQuUs:Custom_Hamiltonians:LoadHamiltonians:undefinied function handle for the custom Hamiltonians')
        end
        
        [obj.H0, obj.S0, obj.H1, obj.S1, obj.H1_transverse, obj.coordinates, ...
            obj.Hscatter, obj.Sscatter, obj.coordinates_scatter, obj.Hcoupling, obj.Scoupling] = obj.CustomHamiltoniansHandle();
           
    
    end
    
    
    % setting the Fermi energy
    if ~isempty( obj.EF )        
        for idx = 1:length( obj.S0 )
            if ~isempty( obj.S0{idx} )
                obj.H0{idx} = obj.H0{idx} - obj.EF*obj.S0{idx};
                obj.H1{idx} = obj.H1{idx} - obj.EF*obj.S1{idx};
            else
                obj.H0{idx} = obj.H0{idx} - obj.EF*speye(size(obj.H0{idx}));
            end
        end  
        
        for idx = 1:length( obj.Hcoupling )
            if ~isempty( obj.Scoupling{idx} )
                obj.Hcoupling{idx} = obj.Hcoupling{idx} - obj.EF*obj.Scoupling{idx};
            end            
        end
        
        if ~isempty(obj.Sscatter)
            obj.Hscatter = obj.Hscatter - obj.EF*obj.Sscatter;      
        else            
            obj.Hscatter = obj.Hscatter - obj.EF*speye(size(obj.Hscatter));      
        end
    end
    
    
    obj.Hamiltonians_loaded = true;
    
end





%% Reset
%> @brief Resets all elements in the object.
    function Reset( obj )
        obj.H0 = []; % Hamiltonian of one slab in the leads
        obj.H1 = []; % coupling between the slabs of the leads
        obj.H1_transverse = [];
        obj.Hscatter = []; % Hamiltonian of the scattering region
        obj.Hscatter_transverse = [];
        obj.Hcoupling = []; % coupling to the leads


        obj.S0 = []; % overlap integrals of one slab in the leads
        obj.S1 = []; % overlap integrals between the slabs of the leads
        obj.S1_transverse = [];
        obj.Sscatter = []; % overlap integrals of the scattering region
        obj.Sscatter_transverse = []; 
        obj.Scoupling = []; % overlap integrals between the scattering region and the leads

        obj.coordinates = []; %coordinates of the leads
        obj.coordinates_scatter = []; % coordinates of the scattering region


        obj.Hamiltonians_loaded = false;
        obj.EF = 0;
    end



%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write( obj, MemberName, input)
        
        if strcmp(MemberName, 'H0')
            obj.H0 = input;
        elseif strcmp(MemberName, 'H1')
            obj.H1 = input;
        elseif strcmp(MemberName, 'Hscatter')
            obj.Hscatter = input;
        elseif strcmp(MemberName, 'Hscatter_transverse')
            obj.Hscatter_transverse = input;             
        elseif strcmp(MemberName, 'H1_transverse')
            obj.H1_transverse = input;
        elseif strcmp(MemberName, 'Hcoupling')
            obj.Hcoupling = input;    
        elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = input;
        elseif strcmp(MemberName, 'S0')
            obj.S0 = input;
        elseif strcmp(MemberName, 'S1')
            obj.S1 = input;
        elseif strcmp(MemberName, 'Sscatter')
            obj.Sscatter = input;
        elseif strcmp(MemberName, 'Sscatter_transverse')
            obj.Sscatter_transverse = input;            
        elseif strcmp(MemberName, 'S1_transverse')
            obj.S1_transverse = input;
        elseif strcmp(MemberName, 'Scoupling')
            obj.Scoupling = input;    
        elseif strcmp(MemberName, 'coordinates_scatter')
            obj.coordinates_scatter = input;
        else
            disp(['No member to write with name ',MemberName]);
        end
    end
%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        if strcmp(MemberName, 'H0')
            ret = obj.H0;
        elseif strcmp(MemberName, 'H1')
            ret = obj.H1;
        elseif strcmp(MemberName, 'Hscatter')
            ret = obj.Hscatter;
        elseif strcmp(MemberName, 'Hscatter_transverse')
            ret = obj.Hscatter_transverse;            
        elseif strcmp(MemberName, 'H1_transverse')
            ret = obj.H1_transverse;
        elseif strcmp(MemberName, 'Hcoupling')
            ret = obj.Hcoupling;    
        elseif strcmp(MemberName, 'coordinates')
            ret = obj.coordinates;
        elseif strcmp(MemberName, 'S0')
            ret = obj.S0;
        elseif strcmp(MemberName, 'S1')
            ret = obj.S1;
        elseif strcmp(MemberName, 'Sscatter')
            ret = obj.Sscatter;
        elseif strcmp(MemberName, 'Sscatter_transverse')
            ret = obj.Sscatter_transverse;            
        elseif strcmp(MemberName, 'S1_transverse')
            ret = obj.S1_transverse;
        elseif strcmp(MemberName, 'Scoupling')
            ret = obj.Scoupling;    
        elseif strcmp(MemberName, 'coordinates_scatter')
            ret = obj.coordinates_scatter; 
        elseif strcmp(MemberName, 'Hamiltonians_loaded')
            ret = obj.Hamiltonians_loaded; 
        else            
            disp(['No member to read with name ',MemberName]);
        end
    end
%% Clear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        if strcmp(MemberName, 'H0')
            obj.H0 = [];
        elseif strcmp(MemberName, 'H1')
            obj.H1 = [];
        elseif strcmp(MemberName, 'Hscatter')
            obj.Hscatter = [];
        elseif strcmp(MemberName, 'Hscatter_transverse')
            obj.Hscatter_transverse = [];            
        elseif strcmp(MemberName, 'H1_transverse')
            obj.H1_transverse = [];
        elseif strcmp(MemberName, 'Hcoupling')
            obj.Hcoupling = [];    
        elseif strcmp(MemberName, 'coordinates')
            obj.coordinates = [];
        elseif strcmp(MemberName, 'S0')
            obj.S0 = [];
        elseif strcmp(MemberName, 'S1')
            obj.S1 = [];
        elseif strcmp(MemberName, 'Sscatter')
            obj.Sscatter = [];
        elseif strcmp(MemberName, 'S1_transverse')
            obj.S1_transverse = [];
        elseif strcmp(MemberName, 'Scoupling')
            obj.Scoupling = [];    
        elseif strcmp(MemberName, 'coordinates_scatter')
            obj.coordinates_scatter = [];
        else
            obj.display(['No member to clear with name ',MemberName]);
        end
        
    end

end % methods public


methods ( Access = private ) 
    
%% Clear
%> @brief Creates the Hamiltonians and overlap integrals from the loaded data from the Siesta package.
%> @param TODO
%> @return The 
    function [HL0,SL0,HL1,SL1,coordsLeads,HSC,SSC,coordsSC,HC,SC] = Siesta_Equus_interface( obj, Directory,input,extmol)
        
        %obj.param.scatter

        %SCATTERING REGION
        %----------------------------------------------------------------------------------------
        HSC=0; SSC=0;

        % bug fixed: variable atom confronting with object atom
        % see doc. MATLAB/Octave language support for Atom at https://atom.io/packages/language-matlab

        HandleForLoad = LoadFromFile( fullfile(Directory, input) );
        atom = HandleForLoad.LoadVariable('atom', 'NoEmpty', 'On');
        leadp = HandleForLoad.LoadVariable('leadp', 'NoEmpty', 'On');
        HandleForLoad.ClearLoadedVariables();

        HandleForLoad = LoadFromFile( fullfile(Directory, extmol) );
        HSM = HandleForLoad.LoadVariable('HSM', 'NoEmpty', 'On');
        iorb = HandleForLoad.LoadVariable('iorb', 'NoEmpty', 'On');
        kpoints_EM = HandleForLoad.LoadVariable('kpoints_EM', 'NoEmpty', 'On');
        nspin = HandleForLoad.LoadVariable('nspin', 'NoEmpty', 'On');
        HandleForLoad.ClearLoadedVariables();
        
        

        numLead=size(leadp,1); 
        iorb = cat(2,atom(iorb(:,1),2:3),iorb);                                     	% iorb: Region PL atom-lead atom n l m z
        norb=size(iorb,1);                                                          	% Define # of orbitals
        orb_molecule = find(iorb(:,1)==0);                                          	% Orbitals in Extended Molecule

        %STRUCTURES FOR COORDINATES
        coordsLeads = cell( 1, length(leadp(:,1)) );
        for i=1:length(leadp(:,1))
            coordsLeads{i} = structures('coordinates');
        end
        coordsSC = structures('coordinates');

        %orb_scissors = intersect(find(iorb(:,1)==0),find(iorb(:,2)==0));            	% Orbitals in Molecule

        %GATHER ORBITALS FOR LEADS
        for lead=1:numLead                                                          	% Orbitals in PLs up to TPL
            aux = intersect(find(iorb(:,1)==lead),find(iorb(:,2)==1));
            for PL=2:leadp(lead,1) 
                aux=cat(2,aux,intersect(find(iorb(:,1)==lead),find(iorb(:,2)==PL)));
            end
            switch lead 
                case(1) 
                    orb1=aux; 
                case(2) 
                    orb2=aux; 
                case(3) 
                    orb3=aux; 
                case(4) 
                    orb4=aux; 
            end; 

%    coordsLeads(lead).a=1;
%    if (size(iorb)(2)>7)
%      coordsLeads(lead).x=iorb(aux(:,leadp(lead,2)),8);
%      coordsLeads(lead).y=iorb(aux(:,leadp(lead,2)),9);
%      coordsLeads(lead).z=iorb(aux(:,leadp(lead,2)),10);

%MY WAY OF INTERPRETING THE INPUT FILE
%    coordsLeads(lead).x=iorb(aux(leadp(lead,2)+1:leadp(lead,1)),8);
%    coordsLeads(lead).y=iorb(aux(leadp(lead,2)+1:leadp(lead,1)),9);
%    coordsLeads(lead).z=iorb(aux(leadp(lead,2)+1:leadp(lead,1)),10);
            aux = [];
        end

        %LOAD HAMILTONIANS FROM SEPARATE LEAD FILES
        %-------------------------------------------------------------------------------
        for lead=1:numLead  

    
            if (leadp(lead,3)~=0)      
                HandleForLoad = LoadFromFile( fullfile(Directory, ['Lead_', int2str(lead), '.mat']) );
                HSL          = HandleForLoad.LoadVariable('HSL', 'NoEmpty', 'On');
                kpoints_Lead = HandleForLoad.LoadVariable('kpoints_Lead', 'NoEmpty', 'On');
                HandleForLoad.ClearLoadedVariables();
                switch lead 
                    case(1) 
                        HSL_1=HSL; kpoints_1=kpoints_Lead;
                    case(2) 
                        HSL_2=HSL; kpoints_2=kpoints_Lead;
                    case(3) 
                        HSL_3=HSL; kpoints_3=kpoints_Lead;
                    case(4) 
                        HSL_4=HSL; kpoints_4=kpoints_Lead;
                end;
            end; 
        end



        for spin = 1:nspin  
            for k_EM = 1:size(kpoints_EM,1)
                H = [];
                S = [];
                HL0 = [];
                HL1 = [];
                SL0 = [];
                SL1 = [];
                G = [];
                for i=1:size(HSM,1) 
                    if (find(HSM(i,1)==k_EM))                                         % H & S for given k & spin
                        H(HSM(i,2),HSM(i,3))=HSM(i,4+2*spin)+1.0i*HSM(i,5+2*spin);
                        S(HSM(i,2),HSM(i,3))=HSM(i,4)+1.0i*HSM(i,5);
                    end
                end
%      if ( scissors(1) == 1 )                                             % Add scissor corrections
%         scissor_corrections; 
%      end;                  
                for lead=1:numLead 
                    k_choice = [];
                    kj = [];
                    HSL = [];
                    kpoints_Lead = [];
                    HL01 = [];
                    SL0t = [];
                    HL1t = [];
                    SL1t = [];
                    aux2 = [];
                    
                    if (leadp(lead,3)~=0)
                        switch lead 
                        case(1) 
                            HSL=HSL_1; kpoints_Lead=kpoints_1; 
                        case(2) 
                            HSL=HSL_2; kpoints_Lead=kpoints_2;
                        case(3) 
                            HSL=HSL_3; kpoints_Lead=kpoints_3;
                        case(4) 
                            HSL=HSL_4; kpoints_Lead=kpoints_4;
                        end;
                    
                        for k_Lead=1:size(kpoints_Lead,1)
                            if (abs(kpoints_EM(k_EM,1)+kpoints_Lead(k_Lead,1))<1d-6 && abs(kpoints_EM(k_EM,2)+kpoints_Lead(k_Lead,2))<1d-6) 
                               k_choice=k_Lead; kj=1;
                            elseif (abs(kpoints_EM(k_EM,1)-kpoints_Lead(k_Lead,1))<1d-6 && abs(kpoints_EM(k_EM,2)-kpoints_Lead(k_Lead,2))<1d-6)
                                k_choice=k_Lead; kj=0;
                            end
                        end
                    
                        for i=1:size(HSL,1) 
                            if (find(HSL(i,1)==k_choice))
                                HL0t(HSL(i,2),HSL(i,3))=HSL(i,4+2*spin)           + (-1)^kj*1.0i*HSL(i,5+2*spin);
                                SL0t(HSL(i,2),HSL(i,3))=HSL(i,4)                  + (-1)^kj*1.0i*HSL(i,5);
                                HL1t(HSL(i,2),HSL(i,3))=HSL(i,4+2*(nspin+spin+1)) + (-1)^kj*1.0i*HSL(i,5+2*(nspin+spin+1));
                                SL1t(HSL(i,2),HSL(i,3))=HSL(i,4+2*(nspin+1))      + (-1)^kj*1.0i*HSL(i,5+2*(nspin+1));
                            end;
                            HL0{lead}=HL0t;
                            SL0{lead}=SL0t;
                            HL1{lead}=HL1t;
                            SL1{lead}=SL1t;

 
                        end;
                    
                        aux2 = intersect(find(iorb(:,1)==lead),find(iorb(:,2)==leadp(lead,2)));

%WE HAVE TO APPLY THIS SOMEWHERE (ONLY WHEN USING SEPARATE LEAD CALCULATION)
                    correction(lead)=HL0{lead}(1,1)-H(aux2(1),aux2(1));
                    end; %if
                end; %for numLeads

	%EM PART, THE SCATTERING REGION
	%IN CASE OF JOSEPHSON CURRENT MODE, THIS EXCLUDES THE INTERFACE
	%WHICH IS THE PART OF THE ELECTRODES, CLOSEST TO EM, THAT IS NOT USED 
	%FOR THE LEADS CALCULATION
                HSC = H(orb_molecule,orb_molecule);
                SSC = S(orb_molecule,orb_molecule);
                p_mol=length(HSC);	% Define pointers

                for lead=1:numLead
                    switch lead 
                    case(1) 
                        aux=orb1; 
                    case(2) 
                        aux=orb2; 
                    case(3) 
                        aux=orb3; 
                    case(4) 
                        aux=orb4; 
                    end

	%COUPLING BETWEEN EM AND THE FRIST LEAD UNIT CELL 
                    B=reshape(aux,numel(aux),1); C=aux(:,1);
                    HC{lead}=H(orb_molecule,C);   %Attach coupling to Lead
                    SC{lead}=S(orb_molecule,C);   %Attach coupling to Lead

                    if ( leadp(lead,3) == 0 )
                        HL0{lead}=H(aux(:,leadp(lead,1)),aux(:,leadp(lead,1)));      %K0=<TPL|K|TPL>
                        HL1{lead}=H(aux(:,leadp(lead,1)),aux(:,leadp(lead,1)-1));    %K1=<TPL|K|TPL-1>+coords="in"
                        SL0{lead}=S(aux(:,leadp(lead,1)),aux(:,leadp(lead,1)));        %Needed to normalize states
                        SL1{lead}=S(aux(:,leadp(lead,1)-1),aux(:,leadp(lead,1)));
                    end;   

    
                end;%Lead loop
            end %k loop
        end % spin loop

end %function


% Input parsing
    function InputParsing( obj, varargin )
    
        p = inputParser;
        p.addParamValue('EF', []);
        p.addParamValue('CustomHamiltoniansHandle', []);
        
        p.parse(varargin{:});

       
        obj.EF       = p.Results.EF;
        obj.CustomHamiltoniansHandle = p.Results.CustomHamiltoniansHandle;
                
        
    end
    

end % methods private

end
