%%   Eotvos Quantum Transport Utilities - EigenProblemLead
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Class to solve the eigenproblem of a translational invariant leads and calculate the group velocities.
%%
classdef EigenProblemLead < handle & CreateLeadHamiltonians
    
    
properties ( Access = protected )
%> true for calculating the retarded Green function, or false for the advanced Green function.
    retarted
%> The unsorted right-sided eigenstates.
    modusmtx
%> The unsorted left-sided eigenstates.
    modusmtx_left
%> The unsorted wave numbers in form exp(1i*k).
    expk
%> The unsorted group velocities.
    csoportseb
%> The wave numbers of the eigenstates, that propagates or decays in the positive direction.
    expk_p
%> The wave numbers of the eigenstates in form exp(1i*k), that propagates or decays in the negative direction.
    expk_m
%> The group velocities of the eigenstates in form exp(1i*k), that propagates or decays in the positive direction.
    vcsop_p
%> The group velocities of the eigenstates, that propagates or decays in the negative direction.
    vcsop_m
%> The right sided wave functions of the eigenstates, that propagates or decays in the positive direction.
    modusmtx_p
%> The left-sided wave numbers of the eigenstates, that propagates or decays in the positive direction.
    modusmtx_p_left
%> The dual basis of the right-sided wave functions of the eigenstates, that propagates or decays in the positive direction.
    d_modusmtx_m
%> The right-sided wave functions of the eigenstates, that propagates or decays in the negative direction.
    modusmtx_m
%> The left-sided wave functions of the eigenstates, that propagates or decays in the negative direction.
    modusmtx_m_left
%> The dual basis of the right-sided wave functions of the eigenstates, that propagates or decays in the positive direction.
    d_modusmtx_p
%> A real number corresponding to the tolerance used to sort the left and right moving (decaying) modes.
    sort_tolerance
end




methods ( Access = public )
%% constructorof the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = EigenProblemLead(Opt, param, varargin)    
        obj = obj@CreateLeadHamiltonians( Opt, param, varargin{:} );  

        if strcmpi(class(obj), 'EigenProblemLead')             
            obj.Reset();        
        end
    end


              
%% szetvalaszto
%> @brief Sorts the left and right propagating (decaying) modes.
%> @param tolerance The tolerance to be used during the sort.
    function szetvalaszto( obj, tolerance )
        
        if isempty(obj.csoportseb)
            return
        end
        
            
        if ~exist('tolerance', 'var')
            tolerance  = 1e-6;
        elseif tolerance < 1e-16
            err = MException('Surface_Green_function:szetvalaszto', 'Failed to separate the left and right-going states');
            save('Error_Surface_Green_function_szetvalaszto.mat');
            throw(err); 
        end
            
            if obj.retarted
                indexes = (real(obj.csoportseb*obj.Lead_Orientation) > 0 & abs(abs(obj.expk)-1) < tolerance) | (1-abs(obj.expk))*obj.Lead_Orientation > tolerance;
            else
                indexes = (real(obj.csoportseb*obj.Lead_Orientation) < 0 & abs(abs(obj.expk)-1) < tolerance) | (1-abs(obj.expk))*obj.Lead_Orientation > tolerance;
            end
            obj.expk_p          = obj.expk(indexes);
            obj.vcsop_p         = obj.csoportseb(indexes);
            obj.modusmtx_p      = obj.modusmtx(:,indexes);
            if ~obj.Opt.usingDualModes
                obj.modusmtx_p_left = obj.modusmtx_left(indexes,:);
            end
            
            obj.expk_m          = obj.expk(~indexes);
            obj.vcsop_m         = obj.csoportseb(~indexes); 
            obj.modusmtx_m      = obj.modusmtx(:,~indexes);
            if ~obj.Opt.usingDualModes
                obj.modusmtx_m_left = obj.modusmtx_left(~indexes,:);
            end
            
            obj.sort_tolerance  = tolerance;
            if length(obj.expk_p) ~= length(obj.expk_m)
                obj.Group_Velocity();
                obj.szetvalaszto( obj.sort_tolerance/10 );
            end
                     
            
            obj.modusmtx   = [];
            obj.csoportseb = [];
            obj.expk          = [];
            
        
        
    end
    %---------------------------

%% Group_Velocity
%> @brief Calculates the group velocities corresponding to the propagating states. The calculated group velocities are stored within the object.
    function Group_Velocity(obj)
           
           
        
        usingDualModes = obj.Opt.usingDualModes;        
        
        obj.csoportseb = NaN(2*obj.M,1);
        adj_modusmtx = obj.modusmtx';
        
        modusmtx = obj.modusmtx;
        modusmtx_left = obj.modusmtx_left;
        expk = obj.expk;
        csoportseb = obj.csoportseb;
        
        GroupVelocityCalc();
        obj.csoportseb = csoportseb;

        
        
        
        %------------------------------------------------------------------
        function GroupVelocityCalc()
            
            H1 = obj.Read('H1');
            H1adj = obj.Read('H1adj');
            M = obj.Read('M');
            if usingDualModes                  
                for idx=1:2*M
                    %Eq (18) in GNew J. Phys 093029 (2014), (modes are already normalized with the overlap integrals)
                    csoportseb(idx) = 1i*adj_modusmtx(idx,:)*(H1*expk(idx)-H1adj*(1/expk(idx)) )*modusmtx(:,idx);                    
                end

            else
                %equation (A4) in PRB 78, 035407 (difference in the evanescent group velocities)
                for idx=1:2*M
                    csoportseb(idx) = 1i*modusmtx_left(idx,:)*(H1*expk(idx)-H1adj*(1/expk(idx)) )*modusmtx(:,idx);
                end
            end            
        end
        %------------------------------------------------------------------
        
        
       
    end
    %------------------------------------------------------------------
    

%% TrukkosSajatertekek
%> @brief Calculates the wave numbers corresponding to the propagating states at given energy. The calculated wave numbers are stored within the object.
%> @param E The energy value used in the calculations.
    function TrukkosSajatertekek(obj, E)
    
        if imag(E) < 0
            obj.retarted = false;
        else
            obj.retarted = true;
        end
        
        obj.ApplyOverlapMatrices(E);
        if ~obj.OverlapApplied
            E_matrix = eye(obj.M)*E;
            H0_loc = obj.H0 - E_matrix;
        else
            H0_loc = obj.H0;
        end
        
        if ~isempty(obj.q) && ~obj.HamiltoniansDecimated
            H0_loc = H0_loc + obj.H1_transverse*diag(exp(-1i*obj.q)) + obj.H1_transverse'*diag(exp(1i*obj.q));
        end
        
        usingDualModes = obj.Opt.usingDualModes;
        
        if obj.isSuperconducting() || ~obj.Opt.BdG
            %  if normal system, or if BdG Hamiltonian with nonzero pair potential
            [obj.modusmtx,obj.expk, obj.modusmtx_left] = calcTrickyEigenvalues( H0_loc, obj.H1, obj.H1adj, obj.M );
        else
            % if BdG Hamiltonian with nonzero pair potential
            M = obj.M;
            M0 = M/2;
            H0_e = H0_loc(1:M0, 1:M0);
            H0_h = H0_loc(M0+1:M, M0+1:M);
            H1_e = obj.H1(1:M0, 1:M0);
            H1_h = obj.H1(M0+1:M, M0+1:M);
            H1adj_e = obj.H1adj(1:M0, 1:M0);
            H1adj_h = obj.H1adj(M0+1:M, M0+1:M);
                
            % eigenvalues for electron-like part
            [modusmtx_e, expk_e, modusmtx_left_e] = calcTrickyEigenvalues( H0_e, H1_e, H1adj_e, M0 );
                
            %eigenvalues for hole-like part
            [modusmtx_h, expk_h, modusmtx_left_h] = calcTrickyEigenvalues( H0_h, H1_h, H1adj_h, M0 );
            
            obj.modusmtx = [ modusmtx_e, zeros( size(modusmtx_e,1), size(modusmtx_e,2)); ...
                        zeros(size(modusmtx_h,1), size(modusmtx_e,2)), modusmtx_h ];
                    
            obj.expk = [expk_e;expk_h];
            
            if ~usingDualModes
                obj.modusmtx_left = [ modusmtx_left_e, zeros( size(modusmtx_left_e,1), size(modusmtx_left_e,2)); ...
                            zeros(size(modusmtx_left_h,1), size(modusmtx_left_e,2)), modusmtx_left_h ];
            end
                
        end       

        obj.NormalizeModes();      

        obj.modusmtx_p     = [];
        obj.modusmtx_m     = [];
        obj.d_modusmtx_p     = [];
        obj.d_modusmtx_m     = [];
        
        
      
        
        
        %------------------------------------------------------------------
        % calculates the tricky eigenvalue problem
        function [modusmtx,expk, modusmtx_left] = calcTrickyEigenvalues( H0, H1, H1adj, M )
            [H_eff,B_eff] = Effective_H(H0,H1,H1adj,M);
            
            if usingDualModes                
                [modusmtx,expk]  = eig(H_eff,B_eff);
                expk             = diag(expk); 
                modusmtx_left = []; 
            else
                [modusmtx, modusmtx_left, expk] = obj.eig(H_eff,B_eff);
            end
            
            if ~usingDualModes   
                % Eq (A3) in PRB 78, 035407
                %normfactors = diag(modusmtx_left*B_eff*modusmtx);
                %modusmtx_left  = diag(1./normfactors)*modusmtx_left;
                modusmtx_left = modusmtx_left(:,1:M);
                modusmtx_left  = diag(sqrt(expk))*modusmtx_left;
            end 
                 
        
            % Eq (7) in PRB 78, 035407
            modusmtx       = modusmtx(1:M,:);   
            modusmtx       = modusmtx*diag(1./sqrt(expk));
        end
        %------------------------------------------------------------------
                
                
        %------------------------------------------------------------------
        % Eq (6) PRB 78 035407
        function [H_eff,B_eff] = Effective_H(H0,H1,H1adj,M)
			% eliminating singular values in H1
			H1 = SVDregularization( H1 );
			H1adj = SVDregularization( H1adj );
            
            %H1inv = H1\eye(size(H1));%inv(H1);
            H_eff    = full([-H0,full(-H1adj);eye(M),zeros(M,M)]);  
            B_eff    = full([ H1, zeros(M,M); zeros(M,M), eye(M,M)]);


			function H1svd = SVDregularization( H1 )
				SVDtolerance = 1e-15;
				if rcond(full(H1)) > SVDtolerance
					H1svd = H1;
					return
				end
				
				if issparse(H1)
					[U,S,V] = svd(full(H1));
				else
					[U,S,V] = svd(H1);
				end
				S = diag(S);
				smax = max(abs(S));
				indexes = logical( abs(S) < smax*SVDtolerance );
				S(indexes) = smax*SVDtolerance;
				H1svd = U*diag(S)*V';
			end
        end
        %------------------------------------------------------------------
    end


%% AddPotential
%> @brief Adds on-site potential to the Hamiltonian H0.
%> @param V The potential calculated on the sites.
    function AddPotential( obj, V )        

        AddPotential@CreateLeadHamiltonians( obj, V );
    end

%% CreateClone
%> @brief Creates a clone of the present object.
%> @return Returns with the cloned interface.
    function ret = CreateClone( obj )
        
        
        ret = EigenProblemLead(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', obj.Hanyadik_Lead,...
                                            'Lead_Orientation', obj.Lead_Orientation, ...
                                            'q', obj.q);                    
                                        
        ret.Write( 'H0', obj.H0); 
        ret.Write( 'H1', obj.H1);
        ret.Write( 'H1adj', obj.H1adj);
        ret.Write( 'H1_transverse', obj.H1_transverse);
        ret.Write( 'fazis_mtx_H0', obj.fazis_mtx_H0); 
        ret.Write( 'fazis_mtx_H1', obj.fazis_mtx_H1);        
        ret.Write( 'fazis_mtx_H1t', obj.fazis_mtx_H1t);
        ret.Write( 'H00', obj.H00);
        ret.Write( 'kulso_szabfokok', obj.kulso_szabfokok);
		ret.Write( 'coordinates', obj.coordinates);
        ret.Write( 'vcsop_p', obj.vcsop_p);
        ret.Write( 'vcsop_m', obj.vcsop_m);
        ret.Write( 'modusmtx_p', obj.modusmtx_p);
        ret.Write( 'modusmtx_m', obj.modusmtx_m);
        ret.Write( 'expk_p', obj.expk_p);
        ret.Write( 'expk_m', obj.expk_m);
        ret.Write( 'params', obj.params);
        ret.Write( 'HamiltoniansCreated', obj.HamiltoniansCreated);
        ret.Write( 'OverlapApplied', obj.OverlapApplied);
        ret.Write( 'HamiltoniansDecimated', obj.HamiltoniansDecimated);
        ret.Write( 'MagneticFieldApplied', obj.MagneticFieldApplied); 
        ret.Write( 'GaugeTransformationApplied', obj.GaugeTransformationApplied);
        ret.Write( 'M', obj.M); 

        
        
        
    end   

%% Reset
%> @brief Resets all elements in the object.
    function Reset( obj )
        
        Reset@CreateLeadHamiltonians( obj );

        
        obj.modusmtx      = [];
        obj.modusmtx_left = [];
        obj.expk          = [];
        obj.csoportseb    = [];

        obj.expk_p           = [];
        obj.expk_m           = [];
        obj.vcsop_p       = [];
        obj.vcsop_m       = [];
        obj.modusmtx_p    = [];
        obj.modusmtx_p_left = [];
        obj.d_modusmtx_m  = [];
        obj.modusmtx_m    = [];
        obj.modusmtx_m_left = [];
        obj.d_modusmtx_p  = [];
        obj.sort_tolerance = 1e-6;       
        
             
    end

%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write(obj, MemberName, input)       
        
        if strcmp(MemberName, 'vcsop_p')
            obj.vcsop_p = input;
        elseif strcmp(MemberName, 'vcsop_m')
            obj.vcsop_m = input;
        elseif strcmp(MemberName, 'modusmtx_p')
            obj.modusmtx_p = input;
        elseif strcmp(MemberName, 'modusmtx_m')
            obj.modusmtx_m = input;
        elseif strcmp(MemberName, 'expk_p')
            obj.expk_p = input;
        elseif strcmp(MemberName, 'expk_m')
            obj.expk_m = input;  
        else
            Write@CreateLeadHamiltonians( obj, MemberName, input );
        end
        
        
    end
%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        
        if strcmp(MemberName, 'vcsop_p')
            ret = obj.vcsop_p;
        elseif strcmp(MemberName, 'vcsop_m')
            ret = obj.vcsop_m;
        elseif strcmp(MemberName, 'modusmtx_p')
            ret = obj.modusmtx_p;
        elseif strcmp(MemberName, 'modusmtx_m')
            ret = obj.modusmtx_m;
        elseif strcmp(MemberName, 'expk_p')
            ret = obj.expk_p;     
        elseif strcmp(MemberName, 'expk_m')
            ret = obj.expk_m;               
        elseif strcmp(MemberName, 'sort_tolerance')
            ret = obj.sort_tolerance;
        else            
            ret = Read@CreateLeadHamiltonians( obj, MemberName );
        end
    end
%% Clear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        
        if strcmp(MemberName, 'vcsop_p')
            obj.vcsop_p = [];
        elseif strcmp(MemberName, 'vcsop_m')
            obj.vcsop_m = []; 
        elseif strcmp(MemberName, 'modusmtx_p')
            obj.modusmtx_p = [];
        elseif strcmp(MemberName, 'modusmtx_m')
            obj.modusmtx_m = []; 
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
        else
            Clear@CreateLeadHamiltonians( obj, MemberName );
        end
        
    end

end %methods public



methods (Access=private)

%% Clear
%> @brief Renormalizes the wave functions with the overlap integrals (Eq (9) in PRB 78 035407
    function NormalizeModes(obj) %NEW
        usingDualModes = obj.Opt.usingDualModes;
            
        modusmtx_adj = obj.modusmtx';
        if obj.OverlapApplied
            for idx = 1:length(obj.expk)
                expk = obj.expk(idx);
                if ~isempty(obj.q) && ~obj.HamiltoniansDecimated
                    S0 = obj.S0 + obj.S1_transverse*diag(exp(-1i*obj.q)) + obj.S1_transverse'*diag(exp(1i*obj.q)); 
                else
                    S0 = obj.S0;
                end
                S = S0 + obj.S1*expk + obj.S1'*(1/expk);
                
                l_n = modusmtx_adj(idx,:)*S*obj.modusmtx(:,idx);
                obj.modusmtx(:,idx) = obj.modusmtx(:,idx)/sqrt(l_n);
                    
                % normalize the left sided eigenvectors
                if ~usingDualModes
                    l_n = obj.modusmtx_left(idx,:)*S*obj.modusmtx(:,idx);
                    obj.modusmtx_left(idx,:) = obj.modusmtx_left(idx,:)/(l_n);
                end
            end
        else
            %no overlap integrals are present
            normfactors = diag( modusmtx_adj*obj.modusmtx );
            obj.modusmtx = obj.modusmtx*diag(1./sqrt(normfactors));
            if ~usingDualModes
                normfactors = diag( obj.modusmtx_left*obj.modusmtx ) ;
                obj.modusmtx_left = diag(1./(normfactors))*obj.modusmtx_left;
            end
        end
    end                
        



end % methods private


    
    
    
end
