%%   Differential conductance and IV curve - based on EQuUs v4.8
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
% Calculates the differential conductance and the IV curve through a square lattice. Example taken from Eur. Phys. J. B 53, 537-549 (2006).
function Differential_conductance( filenum )


if ~exist('filenum', 'var')
    filenum = 1;
end

filename = mfilename('fullpath');
[directory, fncname] = fileparts( filename );

outfilename = [fncname,'_',num2str( filenum )];
outputdir   = [];
setOutputDir();

height = 130;
width = 121;

% Temperature in K
T = 10;


% calculated conductance (and number of open channels) as a function of B
diff_conductance = [];
bias_T0 = [];
current_T0 = [];
bias = [];
current = [];


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
        Edb = 300;
        mu_left = 1e-5;
        mu_right = 0.025;
        
        island = structures('hole');
        island.center.x = width;
        island.center.y = height/2;
        island.radius = 30;
        
        cRibbon = Ribbon('width', width, 'height', height, 'EF', 0, 'filenameIn', fullfile( directory, 'Basic_Input.xml'), 'filenameOut', fullfile( outputdir, [outfilename, '.xml']) );
        cRibbon.rCC = 1;
        
        hScatterPotential = @(coordinates)(ScatterPotential(coordinates, island));
        display('Calculating the IV curve for zero temperature.')
        cIV = IV( 'mu_left', mu_left, 'mu_right', mu_right, 'T', 0, 'TwoTerminal', cRibbon, 'scatterPotential', hScatterPotential, 'gfininvfromHamiltonian', true);
		[current_T0, bias_T0, diff_conductance] = cIV.IVcalc('Edb', Edb, 'tuned_contact', 'right');
        
        display('Calculating the IV curve for non-zero temperature.')
        cIV = IV( 'mu_left', mu_left, 'mu_right', mu_right, 'T', T, 'TwoTerminal', cRibbon, 'scatterPotential', hScatterPotential, 'gfininvfromHamiltonian', true);
		[current, bias] = cIV.IVcalc('Edb', Edb, 'tuned_contact', 'right');        
                
    end
    



%% sets output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = fullfile( directory, resultsdir );        
    end




    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductance
               
        x_lim = [0 25];
        y_lim = [0 16];
        
        
        Position = [0.08 0.28 0.33 0.33];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                   
                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
       plot(bias_T0*1000, real(diff_conductance), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);  
        
               
         % Create xlabel
        xlabel('$$E\; [meV]$$','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond, 'Interpreter', 'latex');

        % Create ylabel
        ylabel('$$G \left[\frac{2e^2}{h}\right]$$','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond, 'Interpreter', 'latex');
        
        %% IV curve 
        
        Position = [0.52 0.28 0.33 0.33];
        axes_current = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                        'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        plot(bias_T0*1000, real(current_T0)/max(bias_T0), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_current);  
        plot(bias*1000, real(current)/max(bias), 'Linewidth', 2, 'color', [1 0 0], 'Parent', axes_current);  
       
        legend_labels = {'$$T=0\;K$$', ['$$T= ', num2str(T), '\;K$$'] };
        
        fig_legend = legend(axes_current, legend_labels);
        set(fig_legend, 'FontSize', fontsize*0.7, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthWest', 'Interpreter', 'latex')
        
               
        % Create xlabel
        xlabel('$$U\; [meV]$$','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_current, 'Interpreter', 'latex');

        % Create ylabel
        ylabel('$$I \left[\frac{2U_{0}e^2}{h}\right]$$','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_current, 'Interpreter', 'latex');

        
        print('-depsc2', fullfile(outputdir,[outfilename, '.eps']))
        print('-dpdf', fullfile(outputdir,[outfilename, '.pdf']))
        close(figure1)
        
        
    end


end
