%    Energy dependent conductivity through a wide (4 micron) graphene ribbon calculated by the adatpiveQ interface - based on EQuUs v4.8
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
function Graphene_Conductivity( filenum )

 
if ~exist('filenum', 'var')
    filenum = 1;
end

filename = mfilename('fullpath');
[~, fncname] = fileparts( filename );

outfilename = [fncname,'_',num2str( filenum )];
outputdir   = [];

height = 2400;
total_width = 19200;
width_uc = 2; %width of the unit cell
aspect_ratio = (total_width*3/2)/((height)*sqrt(3));
disp(['aspect ratio W/L = ', num2str(aspect_ratio, '%10.2f')]);

EF = 0; %In EV
EF_vec = [1e-5:1e-3:0.04];
DeltaEF_index = 5;
EF_vec_fromH = EF_vec(1:DeltaEF_index:end);
Energy = 0;

% Conductivity calculated by the fast way
Conductivity = zeros(size(EF_vec));

% conductivity calculated from the full Hamiltonian
Conductivity_fromH = zeros(size(EF_vec_fromH));



% for graphene infinite mass boundary
% zigzag edged ribbon, q in units of 1/(3r_{cc})
W = (total_width/2*2 + (total_width/2-1) ); %in units of r_CC
m = 0:1:total_width/width_uc;
interpq = transpose( (m+0.5)*pi/(W/3) ); % for graphene infinite mass boundary


    
setOutputDir()
filename = mfilename('fullpath');
[root, fncname] = fileparts( filename );
inputXML = fullfile( root, 'Basic_Input_zigzag_leads.xml');
outputXML = [outputdir, filesep, outfilename, '.xml'];
[ Opt, param ] = parseInput( inputXML );


calcConductivity();

%% Calculate the conductivity for the selected energy points
    function calcConductivity
        
        %----------------------
        % handling of nested functions and variable spaces is buggy. 
        width_uc;
        height;
        Energy;
        EF;
        inputXML;
        outputXML;
        %-------------------
        
        for idx = 1:length(EF_vec)
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductivity for the given Fermi energy')    
            
            EF = EF_vec(idx);
            TransportFast = @(qvec)(CalculateTransporSpecq( qvec, width_uc, height, Energy, EF, inputXML, outputXML, false ) );
            adaptive_moments = adaptiveQ( Opt, interpq );            
            adaptive_moments.runAdaptiveIterations( TransportFast );
            sumCq = sum(adaptive_moments.Read('interpy'))/aspect_ratio;
            Conductivity(idx) = sumCq;
            
            if mod(idx-1,DeltaEF_index) == 0
                disp(' ')
                disp(' ')
                disp(' ----------------------------------')
                disp('Calculating Conductivity for the given Fermi energy from the full Hamiltonian')
                TransportSlow = @(qvec)(CalculateTransporSpecq( qvec, width_uc, height, Energy, EF, inputXML, outputXML, true) );
                adaptive_moments = adaptiveQ( Opt, interpq );
                adaptive_moments.runAdaptiveIterations( TransportSlow );
                sumCq = sum(adaptive_moments.Read('interpy'))/aspect_ratio;
                Conductivity_fromH( floor(idx/DeltaEF_index)+1 ) = sumCq;
            end
            
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
            
            disp([ num2str(idx/length(EF_vec)*100),' % calculated of the conductance.'])
            save( fullfile(outputdir, [outfilename, '.mat']), 'Conductivity', 'Conductivity_fromH', 'Energy', 'EF_vec', 'height', 'total_width');
        end
        
        
        

    end











%% sets the output directory
    function setOutputDir()
        resultsdir = fullfile(pwd, 'results');
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end

%% Plot function
    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductivity    
        
        
        theoretical_limit = 2/pi; %Theoretical minimal conductivity
        
        %y_lim = [0 2];
        
        indexek = logical(Conductivity);
        indexek_fromH = logical(Conductivity_fromH);
        x_lim = [0 max(EF_vec(indexek))*1000];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        legend_labels = [];
        
        if ~isempty(Conductivity(indexek))
            numerics = plot(EF_vec(indexek)*1000, Conductivity(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);  
            legend_labels{end+1} = 'conductivity fast way';
        end
        if ~isempty(Conductivity_fromH(indexek_fromH))
            numerics2 = plot(EF_vec_fromH(indexek_fromH)*1000, Conductivity_fromH(indexek_fromH), 'LineStyle', 'v', 'MarkerSize',6, 'color', [0 0 1], 'Parent', axes_cond);   
            legend_labels{end+1} =  'conductivity slow way';
        end
        
         plot( [0 max(EF_vec(indexek))*1000], ones(1,2)*theoretical_limit, 'r',  'Parent', axes_cond);
        
               
        % Create xlabel
        xlabel('E_F [meV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);

        % Create ylabel
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        
        if length( legend_labels) == 2
            fig_legend = legend([numerics, numerics2], legend_labels);
            set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')
        end
        
        print('-depsc2', fullfile(outputdir, [fncname, '_', num2str(filenum),'.eps']))
        print('-dpdf', fullfile(outputdir, [fncname, '_', num2str(filenum),'.pdf']))
        close(figure1)
        
    end




end


