%   Transport through an SSH chain including a topological band inversion -  Based on EQuUs v4.7
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.

%%
function SSH_transport( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

filename = mfilename('fullpath');
[~, fncname] = fileparts( filename );

outfilename = [fncname,'_',num2str( filenum )];
outputdir   = [];
setOutputDir()


EF = 0;
width = 1;
height = 30;
silent = 0;



Emin = 1e-3;
Emax = 0.15;
Edb = 100;
Evec = Emin:(Emax-Emin)/(Edb-1):Emax;
Conductance = [];
DeltaC = [];
nyitott_csatornak = [];

inputXML1 = 'Basic_Input_SSH_1.xml';
inputXML2 = 'Basic_Input_SSH_2.xml';


Ribbons = cell(2,1);
combineribbons = [];


Transport();
Egeszabra();



%% CalculateTransport
function Transport()
    
    
     Conductance = NaN(size(Evec));
     DeltaC = NaN(size(Evec));
     nyitott_csatornak = NaN(size(Evec));
        
     
    createRibbons();
    combineribbons = CombineRibbons('Ribbons', Ribbons);
    
    
   
    
    for idx = 1:length(Evec)
        try
            [Conductance_tmp, ny_tmp, DeltaC_tmp] = combineribbons.Transport( Evec(idx) );
            Conductance(idx) = Conductance_tmp;
            DeltaC(idx) = DeltaC_tmp;
            nyitott_csatornak(idx) = ny_tmp(1);
            Egeszabra()
        catch
            display( ['Calculating conductance failed at E = ', num2str(Evec(idx))]);
        end
        
        
        save( [outputdir,'/',outfilename], 'EF', 'width', 'nyitott_csatornak', 'DeltaC', 'height', 'Evec', 'Conductance' );
        disp([ num2str(idx/length(Evec)*100),' % calculated of the conductance.'])
        disp( ' ' )
        
    end     
        
end

%% sets the output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end


%% createRibbons
% Fills ribbon list
function createRibbons()
	Ribbons{1} = Ribbon('E', Evec(1), 'EF', EF, 'width', width, 'height', height, 'silent', silent, 'filenameIn', inputXML1, 'filenameOut', fullfile( outputdir, [outfilename,'_1.xml']) );
	Ribbons{2} = Ribbon('E', Evec(1), 'EF', EF, 'width', width, 'height', height, 'silent', silent, 'filenameIn', inputXML2, 'filenameOut', fullfile( outputdir, [outfilename,'_2.xml']) );
end





%% Egeszabra
    function Egeszabra()
        
         figure1 = figure();
        fontsize = 9;
        
        %% Conductivity    
        
        %x_lim = [min(coordinates.x(:,1)) max(coordinates.x(:,1)) ];
        %x_lim = [0 rightside];
        
        
        %y_lim = [0 2];
        
        indexek = logical( (abs(DeltaC) <= 0.5) & ~isnan(DeltaC) );
        x_lim = [0 max(Evec)];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(Evec(indexek), Conductance(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);       
        %plot(Widthvec(indexek), nyitott_csatornak(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);
        try
            plot(Evec(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
        
               
          
        
         % Create xlabel
        xlabel('E [eV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);

        % Create ylabel
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        
        

        fig_legend = legend(axes_cond, {'conductance', 'unitarity error'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2',  [outputdir,'/',outfilename,'.eps'])
        close(figure1)
        
        
    end




end
