%   Spectral density function in superconductor-graphene-superconductor junctions. -  Based on EQuUs v4.7
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.

%%
function SpectralDensity( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

rCC = 1.42e-10;

toCalculate = 1;
Continue    = 0;
outfilename = ['SpectralDensity_',num2str( filenum )];
outputdir   = [];
setOutputDir();


inputXML = 'Basic_Input_zigzag_leads.xml';
outputXML = [outputdir,'/',outfilename, '.xml'];


deltaPhi = 1.4043; %the phase difference between the superconducting contacts


height = 1016;
width = 2;
EF = 0.08; %In EV
eta = 1e-6; % the imaginary part of the energy

surfacePotential = 0.1; %in eV

[ Opt, param ] = parseInput( inputXML );

Delta = [param.Leads{1}.pair_potential, param.Leads{2}.pair_potential ];
[Evec, qvec] = setVectors();

ribbon_hamiltonian    = [];

Rho = NaN(size(qvec)); 
density_of_states = zeros( length(Evec), length(qvec));

if Continue && toCalculate
  HandleForLoad = LoadFromFile( [outputdir,'/',outfilename, '.mat'] );
  density_of_states = HandleForLoad.LoadVariable('density_of_states', 'NoEmpty', 'On');
  Evec = HandleForLoad.LoadVariable('Evec', 'NoEmpty', 'On');
  qvec = HandleForLoad.LoadVariable('qvec', 'NoEmpty', 'On');
  width = HandleForLoad.LoadVariable('width', 'NoEmpty', 'On');
  height = HandleForLoad.LoadVariable('height', 'NoEmpty', 'On');
  EF     = HandleForLoad.LoadVariable('EF', 'NoEmpty', 'On');
  HandleForLoad.ClearLoadedVariables(); 
end

CalculateSpectralDensity();





%% Calculates transport
    function CalculateSpectralDensity()
        
    
        if ~toCalculate 
             disp('Loading Calculated data')
            HandleForLoad = LoadFromFile( [outputdir,'/',outfilename, '.mat'] );
            density_of_states = HandleForLoad.LoadVariable('density_of_states', 'NoEmpty', 'On');
            Evec = HandleForLoad.LoadVariable('Evec', 'NoEmpty', 'On');
            qvec = HandleForLoad.LoadVariable('qvec', 'NoEmpty', 'On');
            width = HandleForLoad.LoadVariable('width', 'NoEmpty', 'On');
            height = HandleForLoad.LoadVariable('height', 'NoEmpty', 'On');
            EF     = HandleForLoad.LoadVariable('EF', 'NoEmpty', 'On');
            HandleForLoad.ClearLoadedVariables(); 
            
            setOutputDir();
            return
        end
        
        indexes = logical( density_of_states(1,:) );
        index2calc = 1:length(indexes);
        index2calc = index2calc( ~indexes );
        
        
        for idx = 1:length(index2calc)
            q = qvec(index2calc(idx));
            
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp(['Calculating Greens function for the transverse momentum q=', num2str(q)]) 
                
            
            Rho = zeros( length(Evec), 1);
            
            ribbon_hamiltonian = Ribbon('width', width, 'height', height, 'E', 0, 'EF', EF, 'filenameIn', inputXML, 'filenameOut', outputXML, 'q', q, ...
                    'phi', [0 deltaPhi]);
                
            for jdx=1:length(Evec)
                Energy = Evec(jdx);
                
                ribbon_hamiltonian.setEnergy( Energy+1i*eta );
                
                
                [Atot, G] = ribbon_hamiltonian.CalcSpectralFunction( Energy+1i*eta, 'constant_channels',false, ...
                'FiniteGreensFunctionFromHamiltonian', true, 'PotInScatter', @SurfacePot, ...
                'decimateDyson', false);
                            
                Rho(jdx) = Atot;                   
                
            end
            
            density_of_states(:,index2calc(idx)) = Rho;
            Egeszabra( )
            save( [outputdir,'/',outfilename, '.mat'], 'qvec', 'Evec', 'width', 'height', 'EF', 'density_of_states' );
            disp([ num2str(index2calc(idx)/length(qvec)*100),' % calculated of the density of states.'])
        end
        
        
                
    end


%% SetVectors
% sets the energy and transverse quantum number data points
    function [Evec, qvec] = setVectors()
        Evec = (0:max(abs(Delta))/70:1.1*max(abs(Delta)));
        vargamma = param.scatter.vargamma;
        qmax = (abs(EF)+max(abs(Evec)))/(vargamma/2)*1.7;
        qvec = (-qmax:qmax/70:qmax); % in units of (3r_CC)^-1

    end

%% ScatterPotetial
% potential through the scattering region (transversally translational
% invariant)
    function ret = SurfacePot( coordinates )
        if isempty(surfacePotential)
            ret = 0;
            return;
        end
        
        %lead_doping = [param.Leads{1}.epsilon, param.Leads{2}.epsilon] - EF;
        lattice_const = norm(coordinates.a);
        
        lambda = 15*lattice_const; %transition length of the pn potential in units of rCC
        y1         = 0.15*height*lattice_const;
        y2         = height*lattice_const - y1;
        
        %x = coordinates.x;
        y = coordinates.y;        
        
        
         ret = surfacePotential*( tanh((y-y1)/lambda) - tanh((y-y2)/lambda) )/2;
    end
    
%% sets the output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end


function Egeszabra( )
 
        
        indexes = logical( density_of_states(1,:));
        
        if length(qvec(indexes)) < 2
            return
        end
        
        figure1 = figure('Colormap',...
    [0.0416666679084301 0 0;0.0833333358168602 0 0;0.125 0 0;0.16666667163372 0 0;0.20833332836628 0 0;0.25 0 0;0.291666656732559 0 0;0.333333343267441 0 0;0.375 0 0;0.416666656732559 0 0;0.458333343267441 0 0;0.5 0 0;0.541666686534882 0 0;0.583333313465118 0 0;0.625 0 0;0.666666686534882 0 0;0.708333313465118 0 0;0.75 0 0;0.791666686534882 0 0;0.833333313465118 0 0;0.875 0 0;0.916666686534882 0 0;0.958333313465118 0 0;1 0 0;1 0.0416666679084301 0;1 0.0833333358168602 0;1 0.125 0;1 0.16666667163372 0;1 0.20833332836628 0;1 0.25 0;1 0.291666656732559 0;1 0.333333343267441 0;1 0.375 0;1 0.416666656732559 0;1 0.458333343267441 0;1 0.5 0;1 0.541666686534882 0;1 0.583333313465118 0;1 0.625 0;1 0.666666686534882 0;1 0.708333313465118 0;1 0.75 0;1 0.791666686534882 0;1 0.833333313465118 0;1 0.875 0;1 0.916666686534882 0;1 0.958333313465118 0;1 1 0;1 1 0.0625;1 1 0.125;1 1 0.1875;1 1 0.25;1 1 0.3125;1 1 0.375;1 1 0.4375;1 1 0.5;1 1 0.5625;1 1 0.625;1 1 0.6875;1 1 0.75;1 1 0.8125;1 1 0.875;1 1 0.9375;1 1 1]);

        
        fontsize = 12;
        colbarlimits = [min(min(log(density_of_states))) max(max(log(density_of_states)))];
        
        x_lim = [min(qvec) max(qvec)];
        y_lim = [min(Evec) max(Evec)]/min(abs(Delta));
        
        
        %y_lim = [0 1];
        Position = [0.08 0.68 0.33 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...           
                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'CLim', colbarlimits, ...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        [X,Y] = meshgrid( qvec(indexes), Evec );
        levelnum = 50;
        density_of_states2plot = log(density_of_states(:,indexes));
        db=1;
        contour(X(1:db:end,1:db:end), Y(1:db:end,1:db:end)/min(abs(Delta)), density_of_states2plot(1:db:end,1:db:end), levelnum ,'LineStyle','none','Fill','on',...   'LevelList',[0.2391 0.2866 0.3342 0.3817 0.4293 0.4769 0.5244 0.572 0.6195 0.6671 0.7147 0.7622 0.8098 0.8573 0.9049 0.9524],...
           'Parent', axes_cond);
        
         % Create xlabel
        xlabel('q [1/(3r_{CC})]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        ylabel('E [\Delta]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);



     

%% potential plot
        coordinates = structures('coordinates');
        coordinates.y = 0:0.01:height*sqrt(3);
        coordinates.a = [0, sqrt(3)];
		pot = SurfacePot( coordinates );

		lead_shift = param.Leads{1}.epsilon;
		mu = EF - lead_shift - pot;
        
		x_lim = [ min(coordinates.y-fix(height/4)*coordinates.a(2)) max(coordinates.y+fix(height/4)*coordinates.a(2))]*rCC*1e10;
		y_lim = [ min(mu) max(mu)];
        Position = [0.52 0.68 0.33 0.3];
        axes_pot = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...   
			   'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        

		plot( coordinates.y*rCC*1e10, mu, 'LineWidth', 2, 'parent', axes_pot);
        plot( [min(coordinates.y)*rCC*1e10, min(coordinates.y)*rCC*1e10], y_lim, 'LineStyle', '--', 'color', [0 0 0], 'parent', axes_pot )
        plot( [max(coordinates.y)*rCC*1e10, max(coordinates.y)*rCC*1e10], y_lim, 'LineStyle', '--', 'color', [0 0 0], 'parent', axes_pot )
         
   
        


		xlabel('x [A]', 'FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_pot);
		ylabel('doping level [ev]' ,'FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_pot);
        
            
                

        print('-depsc2', [outputdir,'/',outfilename,'.eps'])
        print('-dpdf', [outputdir,'/',outfilename,'.pdf'])
        
            close(figure1)
        
        
    end

   



end
