%   Transport through graphene ribbon in magnetic field -  Based on EQuUs v4.7
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%%
function magnetic_ribbon( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

filename = mfilename('fullpath');
[directory, fncname] = fileparts( filename );

outfilename = [fncname,'_',num2str( filenum )];
outputdir   = [];
setOutputDir();

height = 300;
width = 200;


Emin = 0.001; %In EV
Emax = 0.6;
Enum = 100;
Evec = Emin:(Emax-Emin)/(Enum+1):Emax;


B = 40e-0; %in Tesla
homegac = [];


% calculated conductance (and number of open channels) as a function of B
Conductance = zeros(1,length(Evec)); 
nyitott_csatornak = zeros(1,length(Evec));
DeltaC      = zeros(1,length(Evec));
Difference  = zeros(1,length(Evec));

ribbon = [];


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
        for idx = 1:length(Evec)  
            Energy = Evec(idx);     
            ribbon = Ribbon('width', width, 'height', height, 'filenameIn', fullfile( pwd, 'Basic_Input_zigzag_leads.xml'), 'filenameOut', fullfile( outputdir, [outfilename, '.xml']) );
            
            rCC = ribbon.rCC*1e-10; %Angstrom
            phi0 = ribbon.h/ribbon.qe; % magnetic flux quantum
            eta_B = 2*pi/phi0*(rCC)^2*B;
            if B>1e-8
                homegac = 2.97*sqrt(9/2*eta_B);
            end

       
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
        
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductance for the given magentic field')  

            
            tic
            [Conductance_tmp,nyitott_csatornak_num,DeltaC_tmp] = Transport( Energy, B, false );
            [Conductance_tmp_selfEnergy] = Transport( Energy, B, true );
            toc
            
            Conductance(idx) = Conductance_tmp;
            nyitott_csatornak(idx) = nyitott_csatornak_num(1);
            DeltaC(idx) = DeltaC_tmp;
            Difference(idx) = Conductance_tmp - Conductance_tmp_selfEnergy;
            
            
            
            save( fullfile(outputdir,[outfilename, '.mat']), 'Conductance', 'nyitott_csatornak', 'Evec', 'DeltaC', 'height', 'width', 'homegac' );
            disp([ num2str(idx/length(Evec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
        end
                
    end
    
%% Calculates the conductance
    function [Conductance,nyitott_csatornak,DeltaC] = Transport( Energy, B, selfEnergy )
    
    	CreateHandlesForMagneticField( B )    	
    	ribbon.setEnergy( Energy )
        
        ribbon.CalcFiniteGreensFunction( 'gauge_trans', true );        
        
        
        Dysonfunc = @()ribbon.CustomDysonFunc( 'constant_channels', false, 'SelfEnergy', selfEnergy );
        ribbon.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        [S,nyitott_csatornak] = ribbon.FL_handles.SmatrixCalc();
        
        norma = norm(S*S'-eye(sum(nyitott_csatornak)));
        
        if norma >= 1e-3 || isnan(norma)
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if nyitott_csatornak(1) ~= nyitott_csatornak(2)
           disp( 'openchannels do not match' )
           nyitott_csatornak
        end
        
        try
        	conductance = ribbon.FL_handles.Conduktance();
        catch
            Conductance = NaN;
            DeltaC = NaN;
            return
        end
        
        	conductance = abs([conductance(1,:), conductance(2,:)]);
        	DeltaC = std(conductance);
        
        	C = mean(conductance);
        	Conductance = C;
        
        	disp( ['E = ', num2str(Energy), ' conductance = ', num2str(C)])   
    
    end
    
    
%% this creates handles to apply magnetic filed in the ribbon Hamiltonians 
    function CreateHandlesForMagneticField( B )        
        hLandauy = createVectorPotential( B );        
        ribbon.setHandlesForMagneticField('scatter', hLandauy, 'lead', hLandauy );          
    end
    

%% creates handle for vector potential
    function hLandauy  = createVectorPotential( B )
    
    	rCC = ribbon.rCC*1e-10; %Angstrom
		phi0 = ribbon.h/ribbon.qe;		
        eta_B = 2*pi/phi0*(rCC)^2*B;
                        
        hLandauy = @(x,y)([zeros(size(x,1),1),eta_B*x]);
        
    end



%% sets output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = fullfile( directory, resultsdir );        
    end




    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductance
               
        
        indexek = logical( (abs(DeltaC) <= 0.5) & (Evec~=0) & ~isnan(DeltaC) );
        x_lim = [min(Evec(indexek)) max(Evec(indexek))];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(Evec(indexek), Conductance(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);  
        try
            plot(Evec(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
            plot(Evec(indexek), Difference(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
        
        if ~isempty(homegac)
            nmax = round( (Emax/homegac)^2 );
            nvec = 1:nmax;
            for idx = 1:length(nvec)
                plot(sqrt([nvec(idx), nvec(idx)])*homegac, [0 max(Conductance(indexek))], 'Linewidth', 0.5, 'color', [0 0 1], 'Parent', axes_cond);
            end
        end
        
               
          
        
         % Create xlabel
        xlabel('E [ev]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);

        % Create ylabel
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        
        

        fig_legend = legend(axes_cond, {'conductance', 'unitarity error', '"Surface Green Func - Self energy"'});
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', fullfile(outputdir,[outfilename, '.eps']))
        print('-dpdf', fullfile(outputdir,[outfilename, '.pdf']))
        close(figure1)
        
        
    end




end
