%%    Eotvos Quantum Transport Utilities - XMLinput
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    This function is the input filter to generate running parameters from
%    XML file.
%%
function [Opt, param] = XMLinput( filename )


Opt      = structures('Opt');
param    = structures('param');

parameter_names = { 'Decimation', 'Decimation_block', 'magnetic_field', 'Lattice_Type', ...
    'Simple_Green_Function', 'Linear_Regression_in_B', 'BdG', 'Silent', 'usingDualModes', 'debug', 'workers', 'custom_Hamiltonians' };
hamiltonian_parameters = { 'epsilon', 'vargamma', 'SOintrinsic', 'deltaAB', 'SO_rashba_intrinsic', 'Overlap_in_Scatter', 'pair_potential', 'vargamma1', 'vargamma3' };

setDefaults()
parseXML( filename )

% Old format of lattice types
if Opt.Lattice_Type == 0
    Opt.Lattice_Type = 'Square';
elseif Opt.Lattice_Type == 1
    Opt.Lattice_Type = 'Graphene';
elseif Opt.Lattice_Type == 2
    Opt.Lattice_Type = 'Silicene';   
end

    %% setDefaults
    % This sets default values to the parameter structures
    function setDefaults()
       
        Opt = structures('Opt');
        Opt.Decimation = 4;
        Opt.Decimation_block = 151;
        Opt.NofLeads = 0;
        Opt.magnetic_field = 0;
        Opt.Lattice_Type = []; %New default value
        Opt.Simple_Green_Function = 0;
        Opt.Linear_Regression_in_B = 1;
        Opt.BdG       = 0;
        Opt.Silent    = 0;
        Opt.usingDualModes = 1;
        Opt.debug = 0;
        Opt.workers = [];
        Opt.custom_Hamiltonians = [];
        
        param = structures('param');
        param.scatter = structures('scatter_param');
        param.scatter.epsilon  = 0;
        param.scatter.vargamma = 0;
        param.scatter.shape = structures('shape');
        param.scatter.Overlap_in_Scatter = 0;
        
        param.Leads = cell(0);
        
    end
    
    %% parseXML
    % This opens the XML file and fills up the parameter structures
    function parseXML(filename)

		cCommonFunctions = CommonFunctions();
		if cCommonFunctions.isOctave()
			parser = javaObject('org.apache.xerces.parsers.DOMParser');
 			parser.parse(filename); 
 			tree = parser.getDocument;
		else        
	        tree = xmlread(filename); 
		end

        traverseNodes( tree );
        
    end


    %% traverseNodes
    % This parses over the attributes of the tree node
    function traverseNodes( theNode )
        % Recurse over node children.
        
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end


        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmpi(char(theChild.getNodeName), 'parameters' )
                traverseNodes( theChild )
            elseif strcmpi(char(theChild.getNodeName), 'computing_parameters' )
                setComputingParameters( theChild )
            elseif strcmpi(char(theChild.getNodeName), 'Scatter_region' )
                setScatterParameters( theChild )
            elseif strcmpi(char(theChild.getNodeName), 'Lead_parameters' )
                setLeadParameters( theChild )
            end
            
           
            
            %children(count) = makeStructFromNode(theChild);
        end
    end

    %% setComputingParameters
    % This sets computing parameters in the structures
    function setComputingParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if ismember(char(theChild.getNodeName), parameter_names )
                attributes = parseAttributes(theChild);
                setOptValue( attributes, char(theChild.getNodeName) )
            end
        end
        
    end


    %% setScatterParameter
    % This sets parameters for the scattering region
    function setScatterParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if ismember(char(theChild.getNodeName), parameter_names )
                attributes = parseAttributes(theChild);
                setOptValue( attributes, char(theChild.getNodeName) )
            elseif strcmpi(char(theChild.getNodeName), 'shape' )
                createShape( theChild )                      
            else 
                param.scatter = setParamValue( param.scatter, theChild );
            end
        end
        
    end

    %% createShap
    % This creates shape for scattering region
    function createShape( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        shape = [];
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmpi(char(theChild.getNodeName), 'width' )
                attributes = parseAttributes( theChild );
                shape.width = str2double( attributes.value );
            elseif strcmpi(char(theChild.getNodeName), 'height' )
                attributes = parseAttributes( theChild );
                shape.height = str2double( attributes.value );            
            end  
        end
       
        param.scatter.shape = shape;
    end


    %% setLeadParameters
    % This sets parameters for Leads
    function setLeadParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmpi(char(theChild.getNodeName), 'lead' )
                createLeadNode( theChild );
                
            end
        end
        
    end

    %% createLeadNode
    % This fills up data from one lead node
    function createLeadNode( theNode )
        lead_attributes = parseAttributes( theNode );
        num = str2double(lead_attributes.num);
        
        try 
            params = param.Leads{num}; 
        catch err
            params = structures('lead_param'); 
            params.Overlap_in_Leads = 0;
        end
            
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);            
            params = setParamValue( params, theChild );
        end
        
        
        param.Leads{num}      = params;
        
    end


    %% setOptValue
    % This sets a value for a field in the Opt structure 
    function setOptValue( attributes, attribname )
        
        if ( isfield( attributes, 'value') )
            if ~isnan( str2double( attributes.('value') ) )
                Opt.( attribname ) = str2double( attributes.('value') );
            else
                Opt.( attribname ) = attributes.('value');
            end
        else
            display(['bad attribute: ', attribname ])
        end
        
    end

    %% setParamValue
    % This sets a value for a field in the Opt structure 
    function param_structure = setParamValue( param_structure, node )
        
        if strcmpi(char(node.getNodeName), 'epsilon' )
                attributes = parseAttributes( node );
                param_structure.epsilon = str2double(attributes.value);
        elseif strcmpi(char(node.getNodeName), 'vargamma' )
                attributes = parseAttributes( node );
                param_structure.vargamma = str2double(attributes.value);
        elseif strcmpi(char(node.getNodeName), 'vargamma_sc' )
                attributes = parseAttributes( node );
                param_structure.vargamma_sc = str2double(attributes.value);                
        elseif strcmpi(char(node.getNodeName), 'vargamma1' )
                attributes = parseAttributes( node );
                param_structure.vargamma1 = str2double(attributes.value); 
        elseif strcmpi(char(node.getNodeName), 'vargamma3' )
                attributes = parseAttributes( node );
                param_structure.vargamma3 = str2double(attributes.value);                 
        elseif strcmpi(char(node.getNodeName), 'SOintrinsic' )
                attributes = parseAttributes( node );
                param_structure.SOintrinsic = str2double(attributes.value); 
        elseif strcmpi(char(node.getNodeName), 'deltaAB' )
                attributes = parseAttributes( node );
                param_structure.deltaAB = str2double(attributes.value);
        elseif strcmpi(char(node.getNodeName), 'SO_rashba_intrinsic' )
                attributes = parseAttributes( node );
                param_structure.SO_rashba_intrinsic = str2double(attributes.value); 
        elseif strcmpi(char(node.getNodeName), 'pair_potential' )
                attributes = parseAttributes( node );
                if isfield( attributes, 'value' )
                    param_structure.pair_potential = str2double(attributes.value); 
                else
                    param_structure.pair_potential = str2double(attributes.abs)*exp(1i*str2double(attributes.phase));
                end
        elseif strcmpi(char(node.getNodeName), 'Overlap_in_Leads' )
                attributes = parseAttributes( node );
                param_structure.Overlap_in_Leads = str2double(attributes.value);  
        elseif strcmpi(char(node.getNodeName), 'Filename' )
                attributes = parseAttributes( node );
                param_structure.Filename = attributes.value;                  
        elseif strcmpi(char(node.getNodeName), 'Atom' )
                if isempty( param_structure.Atom )
                    param_structure.Atom = structures('Atom');
                end
                
                atom_idx = length( param_structure.Atom ) + 1;      
            
                if node.hasChildNodes
                    childNodes = node.getChildNodes;
                    numChildNodes = childNodes.getLength;
                    
                    for count = 1:numChildNodes
                        theChild = childNodes.item(count-1);
                        
                        if strcmpi(char(theChild.getNodeName), 'PL' )
                            attributes = parseAttributes( theChild );
                            param_structure.Atom(atom_idx).PL = str2double(attributes.value);  
                        elseif strcmpi(char(theChild.getNodeName), 'Id' )
                            attributes = parseAttributes( theChild );
                            param_structure.Atom(atom_idx).Id = str2double(attributes.value);                             
                        end
            
                    end                    
                                       
                end
        elseif strcmpi(char(node.getNodeName), 'Lead_Orientation' ) || strcmpi(char(node.getNodeName), 'orientation' )
                 attributes = parseAttributes( node );
                 param_structure.Lead_Orientation = str2double(attributes.value);
        elseif strcmpi(char(node.getNodeName), 'M' )
                 attributes = parseAttributes( node );
                 param_structure.M = str2double(attributes.value);
        elseif strcmpi(char(node.getNodeName), 'End_Type' )
                 attributes = parseAttributes( node );
                 param_structure.End_Type = attributes.value;   
        end
        
    end



    %% parseAttributes
    % This function parses the attributes of the XML node
    % ----- Local function PARSEATTRIBUTES -----
    function attributes = parseAttributes(theNode)
    % Create attributes structure.

    attributes = [];
    if theNode.hasAttributes
        theAttributes = theNode.getAttributes;
        numAttributes = theAttributes.getLength;
        attributes = struct();
    else
        return
    end

    for count = 1:numAttributes
        attrib = theAttributes.item(count-1);
        attributes.( char(attrib.getName) ) = char(attrib.getValue);
    end


    end

end
