%%    Eotvos Quantum Transport Utilities - Surface_Green_function
%    Copyright (C) 2009-2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to calculate the Greend functions and self energies of a translational invariant lead
%%
classdef InterfaceRegion < handle & CommonFunctions & CreateLeadHamiltonians %NEW
    
    
properties ( Access = protected )
	%> Coupling Hamiltonian between the two sides
    Hcoupling	
	%> Adjungate of the Coupling Hamiltonian between the two sides
    Hcouplingadj	    
	%> Overlap integrals between the two sides
    Scoupling	    
    %> Coordinates of the sites at the second side.
    coordinates2
end





methods ( Access = public )
%% constructorof the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = InterfaceRegion(Opt, param, varargin)    
        obj = obj@CreateLeadHamiltonians( Opt, param, varargin{:} ); 

        if strcmpi(class(obj), 'InterfaceRegion')       
            obj.Reset(); 
        end       
    end
    
    
%% Transform2BdG
%> @brief Transforms the Hamiltonians and the overlap matrices into the BdG model.
    function Transform2BdG( obj )
    
        if isempty(obj.Opt.BdG) || ~obj.Opt.BdG
            return
        end
        
        if ~isempty( obj.coordinates.BdG_u )
            % already transformed into BdG model
            return
        end
        
        Transform2BdG@CreateLeadHamiltonians( obj );
        
        if isempty( obj.coordinates2 )        
            fnames = fieldnames( obj.coordinates2 );
            for idx = 1:length(fnames)
                fname = fnames{idx};
                if strcmp(fname, 'a') || strcmp(fname, 'b')
                    continue
                end                    
                obj.coordinates2.(fname) = [ obj.coordinates2.(fname); obj.coordinates2.(fname) ];
            end
            obj.coordinates2.BdG_u = [ true(size(obj.H0,1),1); false(size(obj.H0,1),1)];
        end
                
                  
        pair_potential = obj.params.pair_potential;
        
        % transforming the coupling Hamiltonians and overlaps        
        if isempty(obj.Scoupling)
            Scoupling = sparse([],[],[], size(obj.Hcoupling,1), size(obj.Hcoupling,2));
        else
            Scoupling = obj.Scoupling;
        end                
        
        obj.Hcoupling = [obj.Hcoupling, Scoupling*pair_potential; Scoupling*conj(pair_potential), -conj(obj.Hcoupling)];
        obj.Hcouplingadj = [obj.Hcouplingadj, Scoupling'*pair_potential; Scoupling'*conj(pair_potential), -conj(obj.Hcouplingadj)];
        
        if ~isempty( obj.Scoupling )
            obj.Scoupling = [obj.Scoupling, sparse([], [], [], size(obj.Scoupling,1), size(obj.Scoupling,2)); sparse([], [], [], size(obj.Scoupling,1), size(obj.Scoupling,2)), obj.Scoupling];
        end
             
        
    end
   

%% CreateClone
%> @brief Creates a clone of the present Surface_Green_function object.
%> @return Returns with the cloned object.
    function cInterfaceRegion = CreateClone( obj )
        
        
        cInterfaceRegion = InterfaceRegion(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', obj.Hanyadik_Lead,...
                                            'Lead_Orientation', obj.Lead_Orientation, ...
                                            'q', obj.q);                    
                                        
        cInterfaceRegion.Write( 'H0', obj.H0); 
        cInterfaceRegion.Write( 'H1', obj.H1);
        cInterfaceRegion.Write( 'H1adj', obj.H1adj);
        cInterfaceRegion.Write( 'Hcoupling', obj.Hcoupling);
        cInterfaceRegion.Write( 'Hcouplingadj', obj.Hcouplingadj);
        cInterfaceRegion.Write( 'S0', obj.S0); 
        cInterfaceRegion.Write( 'S1', obj.S1);
        cInterfaceRegion.Write( 'S1adj', obj.S1adj);
        cInterfaceRegion.Write( 'Scoupling', obj.Scoupling);
        cInterfaceRegion.Write( 'H1_transverse', obj.H1_transverse);
        cInterfaceRegion.Write( 'fazis_mtx_H0', obj.fazis_mtx_H0); 
        cInterfaceRegion.Write( 'fazis_mtx_H1', obj.fazis_mtx_H1);        
        cInterfaceRegion.Write( 'fazis_mtx_H1t', obj.fazis_mtx_H1t);
        cInterfaceRegion.Write( 'H00', obj.H00);
        cInterfaceRegion.Write( 'kulso_szabfokok', obj.kulso_szabfokok);
		cInterfaceRegion.Write( 'coordinates', obj.coordinates);
        cInterfaceRegion.Write( 'params', obj.params);
        cInterfaceRegion.Write( 'HamiltoniansCreated', obj.HamiltoniansCreated);
        cInterfaceRegion.Write( 'OverlapApplied', obj.OverlapApplied);
        cInterfaceRegion.Write( 'HamiltoniansDecimated', obj.HamiltoniansDecimated);
        cInterfaceRegion.Write( 'MagneticFieldApplied', obj.MagneticFieldApplied); 
        cInterfaceRegion.Write( 'GaugeTransformationApplied', obj.GaugeTransformationApplied);
        cInterfaceRegion.Write( 'M', obj.M); 

        
        
        
    end   

%% Reset
%> @brief Resets all elements in the object.
    function Reset( obj )
        
        Reset@CreateLeadHamiltonians( obj );       


        obj.Hcoupling      = [];
        obj.Hcouplingadj   = [];
        obj.Scoupling      = [];
        obj.coordinates2   = [];
             
    end

%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write(obj, MemberName, input)       
        
        
        if strcmp(MemberName, 'Hcoupling')
            obj.Hcoupling = input;
        elseif strcmp(MemberName, 'Hcouplingadj')
            obj.Hcouplingadj = input;             
        elseif strcmp(MemberName, 'Scoupling')
            obj.Scoupling = input;            
        elseif strcmp(MemberName, 'coordinates2')
            obj.coordinates2 = input;                      
        else
            Write@CreateLeadHamiltonians( obj, MemberName, input );
        end
        
        
    end
%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        
        if strcmp(MemberName, 'Hcoupling')
            ret = obj.Hcoupling;
        elseif strcmp(MemberName, 'Hcouplingadj')
            ret = obj.Hcouplingadj;              
        elseif strcmp(MemberName, 'Scoupling')
            ret = obj.Scoupling;            
        elseif strcmp(MemberName, 'coordinates2')
            ret = obj.coordinates2;                    
        else            
            ret = Read@CreateLeadHamiltonians( obj, MemberName );
        end
    end
%% Clear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        if strcmp(MemberName, 'Hcoupling')
            obj.Hcoupling = [];
        elseif strcmp(MemberName, 'Hcouplingadj')
            obj.Hcouplingadj = [];                    
        elseif strcmp(MemberName, 'Scoupling')
            obj.Scoupling = [];            
        elseif strcmp(MemberName, 'coordinates2')
            obj.coordinates2 = [];               
        else
            Clear@CreateLeadHamiltonians( obj, MemberName );
        end
        
    end

end

%------------------------------------------------------------------
end % Global End
