!#include "fintrf.h"
!======================================================================
!     Wrapper routine to calculate the Schur complement of a complex sparse matrix via the PARDISO libraries.
!     This file is part of the EQuUs package. (for details see: http://eqt.elte.hu/equus/home)
!     Copyright (C) 2016 Peter Rakyta, Ph.D.
!
!     This program is free software: you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation, either version 3 of the License, or
!     (at your option) any later version.
! 
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program.  If not, see http://www.gnu.org/licenses/.
!     
!======================================================================
!     Gateway routine
include 'mkl_pardiso.f90'
MODULE getPartialInv


!****************************************************************      
  CONTAINS
!****************************************************************


! DESCRIPTION:
!> @file getPartialInv.f90
!> @brief Get the partial inverse of double valued real matrices
!! 
!> @param[in] sizeA The size of the matrix to be inverted.
!> @param[in] nonzerosA The number of nonzero elements in the matrix to be inverted.
!> @param[in] a The nonzero elements in the matrix to be inverted.
!> @param[in] ia The row indexes of the elements in the matrix to be inverted.
!> @param[in] ja The column indexes of the elements in the matrix to be inverted.
!> @param[in] sizeInv The dimension of the partial inverse to be calculated.
!> @param[out] invA The preallocated array for the partial inverse
! 
  subroutine dgetPartialInv(sizeA, nonzerosA, a, ia, ja, sizeInv, invA)

    use mkl_pardiso

    IMPLICIT NONE

!     Internal solver memory pointer 
    TYPE(MKL_PARDISO_HANDLE), pointer :: PT(:) => null()
    integer*4, pointer :: idum(:) => null(), pia(:) => null(), pja(:) => null()
    real(KIND=8), pointer :: ddum(:) => null(), pa(:) => null(), pB(:) => null(), px(:) => null()
    INTEGER*4 maxfct, mnum, mtype, phase, nrhs, error, msglvl, i, nnz, nequations
    INTEGER*4 iparm(64)


!     The matrix is given in the CSR format A = (a, ia, ja)
      INTEGER*4 sizeA, nonzerosA !matrix A is of size sizeA x sizeA
      INTEGER*4, TARGET ::  ia(sizeA+1), ja(nonzerosA) 
      REAL*8, TARGET :: a(nonzerosA)

      INTEGER*4, ALLOCATABLE :: perm(:)

      INTEGER*4 sizeInv !size of the inverse matrix
      REAL*8 :: invA(sizeInv,sizeInv) 
      
      INTEGER*4 idx, jdx



      DATA nrhs /1/, maxfct /1/, mnum /1/
      allocate( PT(64), idum(0), ddum(0) )
  
      do i = 1, 64
        PT(i)%DUMMY = 0
      end do
      pa => a
      pia => ia
      pja => ja
! initialize error flag
      error = 0

#ifdef DEBUG
      write(0,*)
      write(0,*) 'The input matrix: (getPartialInv)'
      idx = 0
      do jdx = 1, nonzerosA
        if (jdx >= ia(idx+1)) then
          idx=idx+1
        end if

        if (jdx < ia(idx+1)) then  
          write(0,*) 'row', idx, 'col', ja(jdx), 'element of a = ', a(jdx)
        end if
      end do
      write(0,*) 'The size of the Schur complement to be calculated:', sizeInv
#endif

!
!  .. Setup Pardiso control parameters und initialize the solvers     
!     internal adress pointers. This is only necessary for the FIRST   
!     call of the PARDISO solver.                                     
!       
!     choose the matrix type for pardiso factorization
      mtype     = 1  ! structurally symmetric real matrix

!  .. PARDISO license check and initialize solver
      call pardisoinit(pt, mtype, iparm)


!     Allocating the right hand side
      allocate( pB(sizeA) )
      allocate( px(sizeA) )

!     Reordering and Factorization, This step also allocates
!     all memory that is necessary for the factorization
#ifdef DEBUG      
      msglvl    = 1      ! with statistical information
      iparm(27) = 1      ! perform matrix check
#else
      msglvl    = 0      ! without statistical information
      iparm(27) = 0      ! perform matrix check
#endif
      iparm(6) = 0  ! The solution is written into x  
      iparm(28) = 0 ! double real/complex

      phase     = 12  ! reordering and factorization  
      CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, pa, pia, pja, &
                   idum, nrhs, iparm, msglvl, pB, px, error) 

#ifdef DEBUG
      write(0,*) 'Reordering and factorization completed ... '
      IF (error .NE. 0) THEN
         write(0,*) 'The following ERROR was detected: ', error
        STOP
      ENDIF 
#endif

!     Set parameters to calculate only the last sizeInv components of the solution
      iparm(31) = 3
      phase = 33 ! solver
      allocate( perm(sizeA) )
      perm = 0
      do idx = 1, sizeinv
        perm(sizeA-sizeinv+idx) = 1
      end do

      do idx = 1, sizeinv   
!       Creating the right hand side     
        pB = 0.d0
        px = 0.d0
        pB(sizeA-sizeinv+idx) = 1.d0
!       Solver        
        CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, pa, pia, pja, &
                   perm, nrhs, iparm, msglvl, pB, px, error)    
        do jdx = 1,sizeinv
          invA(idx,jdx) = px(sizeA-sizeinv+jdx)
        end do
      end do   


      deallocate(pB, px)

!  
!   Termination and release of memory
      phase     = -1           ! release internal memory
      CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, ddum, &
                   idum, idum, &
                   idum, nrhs, iparm, msglvl, ddum, ddum, error) 
      deallocate(idum, ddum, perm, PT)


  end



! DESCRIPTION:
!> @file getPartialInv.f90
!> @brief Get the partial inverse of a complex valued real matrices
!! 
!> @param[in] sizeA The size of the matrix to be inverted.
!> @param[in] nonzerosA The number of nonzero elements in the matrix to be inverted.
!> @param[in] a The nonzero elements in the matrix to be inverted.
!> @param[in] ia The row indexes of the elements in the matrix to be inverted.
!> @param[in] ja The column indexes of the elements in the matrix to be inverted.
!> @param[in] sizeInv The dimension of the partial inverse to be calculated.
!> @param[out] real_invA The preallocated array for the real part of the partial inverse
!> @param[out] imag_invA The preallocated array for the imaginary part of the partial inverse
! 
  subroutine zgetPartialInv(sizeA, nonzerosA, a, ia, ja, sizeInv, real_invA, imag_invA)

    use mkl_pardiso

    IMPLICIT NONE

!     Internal solver memory pointer 
    TYPE(MKL_PARDISO_HANDLE), pointer :: PT(:) => null()
    integer*4, pointer :: idum(:) => null(), pia(:) => null(), pja(:) => null()
    complex(KIND=8), pointer :: ddum(:) => null(), pa(:) => null(), pB(:) => null(), px(:) => null()
    INTEGER*4 maxfct, mnum, mtype, phase, nrhs, error, msglvl, i, nnz, nequations
    INTEGER*4 iparm(64)


!     The matrix is given in the CSR format A = (a, ia, ja)
      INTEGER*4 sizeA, nonzerosA !matrix A is of size sizeA x sizeA
      INTEGER*4, TARGET ::  ia(sizeA+1), ja(nonzerosA) 
      COMPLEX*16, TARGET :: a(nonzerosA)

      INTEGER*4, ALLOCATABLE :: perm(:)

      INTEGER*4 sizeInv !size of the inverse matrix
      REAL*8 :: real_invA(sizeInv,sizeInv) 
      REAL*8 :: imag_invA(sizeInv,sizeInv) 
      
      INTEGER*4 idx, jdx



      DATA nrhs /1/, maxfct /1/, mnum /1/
      allocate( PT(64), idum(0), ddum(0) )
  
      do i = 1, 64
        PT(i)%DUMMY = 0
      end do
      pa => a
      pia => ia
      pja => ja
! initialize error flag
      error = 0


#ifdef DEBUG
      write(0,*)
      write(0,*) 'The input matrix: (getPartialInv)'
      idx = 0
      do jdx = 1, nonzerosA
        if (jdx >= ia(idx+1)) then
          idx=idx+1
        end if

        if (jdx < ia(idx+1)) then  
          write(0,*) 'row', idx, 'col', ja(jdx), 'element of a = ', a(jdx)
        end if
      end do
      write(0,*) 'The size of the Schur complement to be calculated:', sizeInv
#endif

!
!  .. Setup Pardiso control parameters und initialize the solvers     
!     internal adress pointers. This is only necessary for the FIRST   
!     call of the PARDISO solver.                                     
!       
!     choose the matrix type for pardiso factorization
      mtype     = 3  ! structurally symmetric complex matrix

!  .. PARDISO license check and initialize solver
      call pardisoinit(pt, mtype, iparm)


!     Allocate the right hand side
      allocate( pB(sizeA) )
      allocate( px(sizeA) )

!     Reordering and Factorization, This step also allocates
!     all memory that is necessary for the factorization
#ifdef DEBUG      
      msglvl    = 1      ! with statistical information
      iparm(27) = 1      ! perform matrix check
#else
      msglvl    = 0      ! without statistical information
      iparm(27) = 0      ! perform matrix check
#endif
      iparm(6) = 0  ! The solution is written into x  
      iparm(28) = 0 ! double real/complex

      phase     = 12  ! reordering and factorization  
      CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, pa, pia, pja, &
                   idum, nrhs, iparm, msglvl, pB, px, error) 



#ifdef DEBUG
      write(0,*) 'Reordering and factorization completed ... '
      IF (error .NE. 0) THEN
         write(0,*) 'The following ERROR was detected: ', error
        STOP
      ENDIF 
#endif

!     Set parameters to calculate only the last sizeInv components of the solution
      iparm(31) = 3
      phase = 33 ! solver
      allocate( perm(sizeA) )
      perm = 0
      do idx = 1, sizeinv
        perm(sizeA-sizeinv+idx) = 1
      end do

      do idx = 1, sizeinv  
!       Creating the right hand side      
        pB = (0.d0, 0.d0)
        px = (0.d0, 0.d0)
        pB(sizeA-sizeinv+idx) = (1.d0, 0.d0)
!       Solver        
        CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, pa, pia, pja, &
                   perm, nrhs, iparm, msglvl, pB, px, error)    
        do jdx = 1,sizeinv
          real_invA(idx,jdx) = REAL(px(sizeA-sizeinv+jdx))
          imag_invA(idx,jdx) = AIMAG(px(sizeA-sizeinv+jdx))
        end do
      end do   


      deallocate(pB, px)


!  
!   Termination and release of memory
      phase     = -1           ! release internal memory
      CALL pardiso (pt, maxfct, mnum, mtype, phase, sizeA, ddum, &
                   idum, idum, &
                   idum, nrhs, iparm, msglvl, ddum, ddum, error) 
      deallocate(idum, ddum, perm, PT)


  end








end module


