/*======================================================================
     Gateway routine to call the zgetpartialinv subroutine from the mkl_EQuUs package.
     Copyright (C) 2016 Peter Rakyta, Ph.D.

     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program.  If not, see http://www.gnu.org/licenses/.
     
====================================================================== */
#include "mex.h"
#ifdef __cplusplus
extern "C" 
#endif

typedef struct dcomplex { 
	double real, imag; 
} comp;

#ifdef __INTEL_COMPILER
void getpartialinv_mp_zgetpartialinv_(int *sizeA, int *nonzerosA, comp *a, int *ia, int *ja, int *sizeInv, double *real_invA, double *imag_invA);
#else
void __getpartialinv_MOD_zgetpartialinv(int *sizeA, int *nonzerosA, comp *a, int *ia, int *ja, int *sizeInv, double *real_invA, double *imag_invA);
#endif

/** Gateway routine to call the zgetpartialinv subroutine from the mkl_EQuUs package. (see http://www.mathworks.com/help/matlab/matlab_external/standalone-example.html#zmw57dd0e18558).
 \param nlhs	Number of output (left-side) arguments, or the size of the plhs array.
 \param plhs	Array of output arguments.
 \param nrhs	Number of input (right-side) arguments, or the size of the prhs array.
 \param prhs	Array of input arguments.
*/
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
/*     Array information: */
      mwSize nonzerosA; /*number of nonzeros elements in A*/
      mwSize sizeAm;
      mwSize sizeAn; /*size of the matrix A (rows, and cols)  */
      mwSize sizeInv; /*Size of the Schur complement to be calculated*/
      int sizeAn_fort, nonzerosA_fort, sizeInv_fort; /*fortran compatible matrix dimensions*/

      mwSize *ia, *ja;
      int *ia_fort, *ja_fort; /* fortran compatible indices */
      comp* a;
      double *real, *imag;


/*-----------------------------------------------------------------------*/
/*     Check for proper number of arguments. */
      if(nrhs != 2) {
         mexErrMsgIdAndTxt ("MATLAB:timestwo:nInput", "Two inputs required.");
      }
      else if (nlhs > 1) {
         mexErrMsgIdAndTxt ("MATLAB:timestwo:nOutput", "Too many output arguments.");
      }

/*     Validate inputs */
      
/*     Check that the input is a double.*/
      if ((mxIsDouble(prhs[0]) == 0)) {
         mexErrMsgIdAndTxt ("MATLAB:timestwo:NonDouble", "Input must be a double.");
      }
      
/*     Check if inputs are complex.*/
      if ((mxIsComplex(prhs[0]) == 0) ) {
         mexErrMsgIdAndTxt ("MATLAB:convec:NonReal", "Inputs must be complex.");
      }

/*     Get the size of the input array. (MATLAB) */
      nonzerosA = mxGetNzmax(prhs[0]);
      sizeAm = mxGetM(prhs[0]);
      sizeAn = mxGetN(prhs[0]);

      ia = mxGetJc(prhs[0]);
      ja = mxGetIr(prhs[0]);
      real = mxGetPr(prhs[0]);
      imag = mxGetPi(prhs[0]);

      ia_fort = (int*)mxMalloc( (sizeAm+1)*sizeof(int) );
      ja_fort = (int*)mxMalloc( nonzerosA*sizeof(int) );
#ifdef matlab
      a = (comp*)mxMalloc( nonzerosA*sizeof(comp) );
#else
      a = (comp*)real;
#endif

      mwSize idx; 
      for( idx=1; idx<=sizeAm+1; idx++ ) {
        ia_fort[idx-1] = (int)ia[idx-1] + 1;
      }

      for( idx=1; idx<=nonzerosA; idx++ ) {
        ja_fort[idx-1] = (int)ja[idx-1] + 1;
#ifdef matlab
        a[idx-1].real = real[idx-1];
        a[idx-1].imag = imag[idx-1];
#endif
      }

/*     getting the size of the partial inverse */
      sizeInv = *mxGetPr(prhs[1]);


      plhs[0] = mxCreateDoubleMatrix(sizeInv, sizeInv, mxCOMPLEX); /* create output matrix */
      /* populate the real and imaginary part of the created array */
      real = mxGetPr(plhs[0]);
      imag = mxGetPi(plhs[0]);

      sizeAn_fort = (int)sizeAn;
      nonzerosA_fort = (int)nonzerosA;
      sizeInv_fort = (int)sizeInv;
#ifdef __INTEL_COMPILER
      getpartialinv_mp_zgetpartialinv_(&sizeAn_fort, &nonzerosA_fort, a, ia_fort, ja_fort, &sizeInv_fort, real, imag);
#else
      __getpartialinv_MOD_zgetpartialinv(&sizeAn_fort, &nonzerosA_fort, a, ia_fort, ja_fort, &sizeInv_fort, real, imag);
#endif

      mxFree(ia_fort);
      mxFree(ja_fort);
      mxFree(a);

}


