%%    Eotvos Quantum Transport Utilities - Peierls
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an object reasponsible for the Peierls and gauge transformations.
%%
classdef Peierls < handle & Messages

    properties ( Access = protected )
        %> The Peierls phases in the Hamiltonian can be multiplied by this parameter. TODO remove this
        parameter %obsolete
        %> String containing the name of the built-in gauge ('LandauX', 'LandauY').
        gauge
        %> The strength of the magnetic field for the built-in vector potentials.
        eta
        %> The function handle of the vector potential.
        Vectorpotential
    end



methods ( Access = public )

%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = Peierls(Opt, varargin)
        obj = obj@Messages( Opt );
        
        obj.parameter = [];
        obj.gauge     = []; % 'LandauX', 'LandauY' gauges or [] for load from input files
        obj.eta       = 0;  % eta_B = 2*pi/phi0*(rCC)^2*B;

        obj.inputparsing( varargin{:} );

        obj.Vectorpotential = [];

        obj.inputparsing( varargin{:} );
        
    end

%% PeierlsTransformLeads
%> @brief Algorithm to perform Peierls transformation in the Hamiltonians of the leads.
%> @param Surface_tmp An instance of class CreateLeadHamiltonians (or a derived class) 
function PeierlsTransformLeads( obj, Surface_tmp, varargin ) %TODO delete varargin, check input classes

    if Surface_tmp.Read( 'MagneticFieldApplied' )
		obj.display('Magnetic field already applied in the Hamiltonians');
		return
    end
    
    if isempty(obj.Vectorpotential)
        obj.display('Vectorpotential is not set.');
        return
    end
		

    if ~Surface_tmp.Read( 'HamiltoniansCreated' ) || Surface_tmp.Read( 'HamiltoniansDecimated' ) 
       err = MException('Peierls:PeierlsTransformLeads', 'Hamiltonians in Surface_tmp are not created, or they are decimated. Thus, unable to perform Peierls transformation.');
       save('Error_Peierls_PeierlsTransformLeads.mat');
       throw(err); 
    end       
    
    H0 = Surface_tmp.Read( 'H0' );
    Surface_tmp.Clear( 'H0' );
    H1 = Surface_tmp.Read( 'H1' );
    Surface_tmp.Clear( 'H1' );
    H1_transverse = Surface_tmp.Read( 'H1_transverse' );
    Surface_tmp.Clear( 'H1_transverse' );
    
    if obj.Opt.BdG
        H0_size = size(H0,1)/2;
        mtx_pair_potential = H0( 1:H0_size, H0_size+1:2*H0_size);
        H0                 = H0( 1:H0_size, 1:H0_size);
        H1                 = H1( 1:H0_size, 1:H0_size);
        if ~isempty(H1_transverse)
            H1_transverse      = H1_transverse( 1:H0_size, 1:H0_size);
        end
        
        if norm(mtx_pair_potential,1) >= 1e-6
            error( 'The peierls transformation with nonzero B is forbidden in the superconducting phase. Set the pair potential to zero, or use gauge transformation instead.')
            mtx_pair_potential = mtx_pair_potential*0;
        end
    end
    
    
    coordinates = Surface_tmp.Read( 'coordinates' );
       
    
    %Peierls in the unit cell
    [sor_idx,oszlop_idx,elements] = find(H0);
        
    kezdopont = [coordinates.x( oszlop_idx ), coordinates.y( oszlop_idx )];
    vegpont     = [coordinates.x( sor_idx ), coordinates.y( sor_idx )];
    fazis          = obj.Peierls_transzform(kezdopont,vegpont);
        
    fazis_mtx_H0 = sparse(sor_idx,oszlop_idx,fazis,size(H0,1), size(H0,2) );
    elements        = exp(1i*fazis).*elements;
    H0                  = sparse(sor_idx,oszlop_idx,elements,size(H0,1), size(H0,2) );
        
    Surface_tmp.Write( 'fazis_mtx_H0', fazis_mtx_H0);
    
    
    %Peierls in the coupling of unit cells
    [sor_idx,oszlop_idx,elements] = find(H1);
    orientation = Surface_tmp.Read('Lead_Orientation');
    Hanyadik_Lead = Surface_tmp.Read('Hanyadik_Lead' );
    if orientation == 1 && ~isempty( Hanyadik_Lead )
        kezdopont = [coordinates.x( oszlop_idx ) , coordinates.y( oszlop_idx )];
        vegpont     = [coordinates.x( sor_idx ) - coordinates.a(1), coordinates.y( sor_idx ) - coordinates.a(2)];
    else
        kezdopont = [coordinates.x( oszlop_idx ) + coordinates.a(1), coordinates.y( oszlop_idx ) + coordinates.a(2)];
        vegpont     = [coordinates.x( sor_idx ), coordinates.y( sor_idx )];
    end
    fazis          = obj.Peierls_transzform(kezdopont,vegpont);
        
    fazis_mtx_H1 = sparse(sor_idx,oszlop_idx,fazis,size(H1,1), size(H1,2) );
    elements        = exp(1i*fazis).*elements;
    H1                  = sparse(sor_idx,oszlop_idx,elements,size(H1,1), size(H1,2) );
        
    Surface_tmp.Write( 'fazis_mtx_H1', fazis_mtx_H1); 
        
        
    %Peierls in the transverse coupling of unit cells
    if ~isempty(H1_transverse)
        [sor_idx,oszlop_idx,elements] = find(H1_transverse);
        kezdopont = [coordinates.x( oszlop_idx ) + coordinates.b(1), coordinates.y( oszlop_idx ) + coordinates.b(2)];
        vegpont     = [coordinates.x( sor_idx ), coordinates.y( sor_idx )];
        fazis          = obj.Peierls_transzform(kezdopont,vegpont);
        
        fazis_mtx_H1t = sparse(sor_idx,oszlop_idx,fazis,size(H1_transverse,1), size(H1_transverse,2) );
        elements        = exp(1i*fazis).*elements;
        H1_transverse = sparse(sor_idx,oszlop_idx,elements,size(H1_transverse,1), size(H1_transverse,2) );       
        
        Surface_tmp.Write( 'fazis_mtx_H1t', fazis_mtx_H1t); 
    end
        
   
    
    if obj.Opt.BdG
        H0 = [H0, mtx_pair_potential; mtx_pair_potential', -conj(H0)];
        H1 = [H1, zeros(size(H1)); zeros(size(H1')), -conj(H1)];
        H1_transverse = [H1_transverse, zeros(size(H1_transverse)); zeros(size(H1_transverse')), -conj(H1_transverse)];
    end   
    
    
     Surface_tmp.Write( 'H0', H0 );
     Surface_tmp.Write( 'H1', H1 );
     Surface_tmp.Write( 'H1adj', H1' );
     Surface_tmp.Write( 'H1_transverse', H1_transverse );
     Surface_tmp.Write( 'MagneticFieldApplied', 1 );
     %Surface_tmp.Write( 'q', q);
    
    
end

%% PeierlsTransform
%> @brief Algorithm to perform Peierls transformation in Hamiltonian stored in the Way2Hamiltonian object. 
%> @param Way2Hamiltonian An instance of class CreateHamiltonians (or a derived class).
function PeierlsTransform( obj, Way2Hamiltonian ) %TODO check input classes

	if Way2Hamiltonian.Read( 'MagneticFieldApplied' )
		obj.display('Magnetic field already applied in the Hamiltonians', 1);
		return
    end
    
    if isempty(obj.Vectorpotential)
        obj.display('Vectorpotential is not set.');
        return
    end
    
    if ~Way2Hamiltonian.Read( 'HamiltoniansCreated' ) || Way2Hamiltonian.Read( 'HamiltoniansDecimated' )
       err = MException('Peierls:PeierlsTransform', 'Hamiltonians in Surface_tmp nor created, or they are decimated. Thus, unable to perform Peierls transformation.');
       save('Error_Peierls_PeierlsTransform.mat');
       throw(err); 
    end    
    
    NameOfH = 'Hscatter';
    NameOfH_transverse = 'Hscatter_transverse';
    Hamiltoni = Way2Hamiltonian.Read( NameOfH );
    Hamiltoni_transverse = Way2Hamiltonian.Read( NameOfH_transverse );
    Way2Hamiltonian.Clear( NameOfH );
    Way2Hamiltonian.Clear( NameOfH_transverse );
    coordinates =  Way2Hamiltonian.Read( 'coordinates' );
    
    if obj.Opt.BdG
        mtx_pair_potential = Hamiltoni( ~coordinates.BdG_u, coordinates.BdG_u );
        Hamiltoni                 = Hamiltoni( coordinates.BdG_u, coordinates.BdG_u );
        if ~isempty(Hamiltoni_transverse)
            Hamiltoni_transverse      = Hamiltoni_transverse( coordinates.BdG_u, coordinates.BdG_u );
        end
        coordinates.x             = coordinates.x( coordinates.BdG_u );
        coordinates.y             = coordinates.y( coordinates.BdG_u );
        
        if norm(mtx_pair_potential,1) >= 1e-6
            error( 'The peierls transformation with nonzero B is forbidden in the superconducting phase. Set the pair potential to zero, or use gauge transformation instead.')
            mtx_pair_potential = mtx_pair_potential*0;
        end
    end

               
    %Peierls in the scattering region
    [sor_idx,oszlop_idx,elements] = find(Hamiltoni);
    kezdopont = [coordinates.x( oszlop_idx ), coordinates.y( oszlop_idx )];
    vegpont     = [coordinates.x( sor_idx ), coordinates.y( sor_idx )];
    fazis          = obj.Peierls_transzform(kezdopont,vegpont);
        
    fazis_mtx = sparse(sor_idx,oszlop_idx,fazis,size(Hamiltoni,1), size(Hamiltoni,2) );
    elements        = exp(1i*fazis).*elements;
    Hamiltoni = sparse(sor_idx,oszlop_idx,elements,size(Hamiltoni,1), size(Hamiltoni,2) );               
        
    Way2Hamiltonian.Write( 'fazis_mtx', fazis_mtx);
        
        
    %Peierls in the transverse coupling of the scattering region
    if ~isempty(Hamiltoni_transverse)
        [sor_idx,oszlop_idx,elements] = find(Hamiltoni_transverse);
        kezdopont = [coordinates.x( oszlop_idx ) + coordinates.b(1), coordinates.y( oszlop_idx ) + coordinates.b(2)];
        vegpont     = [coordinates.x( sor_idx ), coordinates.y( sor_idx )];
        fazis          = obj.Peierls_transzform(kezdopont,vegpont);
        
        fazis_mtx_t = sparse(sor_idx,oszlop_idx,fazis,size(Hamiltoni_transverse,1), size(Hamiltoni_transverse,2) );
        elements        = exp(1i*fazis).*elements;
        Hamiltoni_transverse = sparse(sor_idx,oszlop_idx,elements,size(Hamiltoni_transverse,1), size(Hamiltoni_transverse,2) );  
        
        Way2Hamiltonian.Write( 'fazis_mtx_t', fazis_mtx_t);
    end
            
    
    if obj.Opt.BdG
        Hamiltoni_tmp = sparse([],[],[], size(Hamiltoni,1)*2, size(Hamiltoni,2)*2 );
        Hamiltoni_transverse_tmp = sparse([],[],[], size(Hamiltoni_transverse,1)*2, size(Hamiltoni_transverse,2)*2 );
        Hamiltoni_tmp( coordinates.BdG_u, coordinates.BdG_u ) = Hamiltoni;
        Hamiltoni_tmp( ~coordinates.BdG_u, ~coordinates.BdG_u ) = -conj(Hamiltoni);
        Hamiltoni_tmp( ~coordinates.BdG_u, coordinates.BdG_u ) = mtx_pair_potential;
        Hamiltoni_tmp( coordinates.BdG_u, ~coordinates.BdG_u ) = mtx_pair_potential';
        
        if ~isempty(Hamiltoni_transverse)
            Hamiltoni_transverse_tmp( coordinates.BdG_u, coordinates.BdG_u ) = Hamiltoni_transverse;
            Hamiltoni_transverse_tmp( ~coordinates.BdG_u, ~coordinates.BdG_u ) = -conj(Hamiltoni_transverse);
        end
        
        Hamiltoni            = Hamiltoni_tmp;
        Hamiltoni_transverse = Hamiltoni_transverse_tmp;
    end   
    
    Way2Hamiltonian.Write( NameOfH, Hamiltoni );
    Way2Hamiltonian.Write( NameOfH_transverse, Hamiltoni_transverse );
    Way2Hamiltonian.Write( 'MagneticFieldApplied', 1 );
    %Way2Hamiltonian.Write( 'q', q);
end

%% gaugeTransformation
%> @brief Algorithm to perform gauge transformation on Green's function and/or on Hamiltonian.
%> @param mtx The matrix of the Green's function or the Hamiltonian.
%> @param coordinates An instance of structure coordinates representing the site coordinates
%> @param gauge_field Function handle of the gauge field.
    function ret = gaugeTransformation( obj, mtx, coordinates, gauge_field ) 
        if isempty( gauge_field )
            ret = mtx;
            return
        end    
        % for Landau_y = Landau_x + grad( gauge_field )
        %gauge_field = @(x,y)( eta*x.*y);
        
        mtx_size = size(mtx,1);
        if length( coordinates.x ) ~= mtx_size
            warning( 'The coordinates and matrix elements do not correspond to each other.' )
            return
        end
        
        % "u" and "v" like components must be transformed differently in BdG theory
        if isfield( coordinates, 'BdG_u' ) && ~isempty(coordinates.BdG_u)
            fact = -(-1).^coordinates.BdG_u;
        else
            fact = ones( size(coordinates.x) );
        end
        
                
        if issparse( mtx )
            Umtx = sparse( 1:mtx_size, 1:mtx_size, exp(1i*fact.*gauge_field( coordinates.x, coordinates.y )), mtx_size, mtx_size);
        else
            Umtx = diag( exp(1i*fact.*gauge_field( coordinates.x, coordinates.y )) );
        end
        
        ret = Umtx * mtx * Umtx';        
        
    end

%% gaugeTransformationOnLead
%> @brief Algorithm to perform gauge transformation in the Hmiltonians of a lead.
%> @param Surface_tmp An instance of class CreateLeadHamiltonians (or a derived class) 
%> @param gauge_field Function handle of the gauge field.
    function gaugeTransformationOnLead( obj, Surface_tmp, gauge_field ) %TODO check input classes
        
        if isempty( gauge_field )
            return
        end         
        
        if Surface_tmp.Read( 'GaugeTransformationApplied' )
            obj.display('Gauge transformation already applied in the Hamiltonians');
            return
        end
        
        H0 = Surface_tmp.Read('H0');
        H1 = Surface_tmp.Read('H1');
        H1adj = Surface_tmp.Read('H1adj');
        orientation = Surface_tmp.Read('Lead_Orientation');
        Hanyadik_Lead = Surface_tmp.Read('Hanyadik_Lead' );
        coordinates = Surface_tmp.Read( 'coordinates' );
        coordinates_tmp = structures( 'coordinates');
        if orientation == 1 && ~isempty( Hanyadik_Lead )
            coordinates_tmp.x = [coordinates.x-coordinates.a(1); coordinates.x];
            coordinates_tmp.y = [coordinates.y-coordinates.a(2); coordinates.y];
        else
            coordinates_tmp.x = [coordinates.x; coordinates.x+coordinates.a(1)];
            coordinates_tmp.y = [coordinates.y; coordinates.y+coordinates.a(2)];
        end
            
            
        
        H0size = size(H0, 1);
        
        if isfield( coordinates, 'BdG_u' )
            coordinates_tmp.BdG_u = [coordinates.BdG_u; coordinates.BdG_u];
        end
        
        H_tmp = [H0,H1;H1adj,H0];%          
        H_tmp = obj.gaugeTransformation( H_tmp, coordinates_tmp, gauge_field );
        
        
        H0 = H_tmp(1:H0size,1:H0size);
        H1 = H_tmp(1:H0size,H0size+1:end);  
        H1adj = H_tmp(H0size+1:end,1:H0size);
        
        Surface_tmp.Write( 'H0', H0);
        Surface_tmp.Write( 'H1', H1);    
        Surface_tmp.Write( 'H1adj', H1adj);
        
        % gauge transformation of the transverse H1
        H1_transverse = Surface_tmp.Read('H1_transverse');
        if ~isempty(H1_transverse) && ~Surface_tmp.Read('HamiltoniansDecimated')
            coordinates_tmp.x = [coordinates.x; coordinates.x+coordinates.b(1)];
            coordinates_tmp.y = [coordinates.y; coordinates.y+coordinates.b(2)];
            
            H_tmp = [sparse([],[],[],size(H1_transverse,2), size(H1_transverse,1)), H1_transverse; ...
                H1_transverse', sparse([],[],[],size(H1_transverse,1), size(H1_transverse,2))];     
            
            H_tmp = obj.gaugeTransformation( H_tmp, coordinates_tmp, gauge_field );
            H1_transverse = H_tmp( 1:size(H1_transverse,2), size(H1_transverse,1)+1:end );
            Surface_tmp.Write( 'H1_transverse', H1_transverse);
        end
        
        
        %gauge transformation of the transverse momentum
%         q = Surface_tmp.Read('q');
%         identity_mtx = sparse(1:size(H1_transverse,1),1:size(H1_transverse,1),1, size(H1_transverse,1), size(H1_transverse,2) );
%         if ~isempty(q) && ~Surface_tmp.Read('HamiltoniansDecimated')
%             coordinates_tmp.x = [coordinates.x+coordinates.b(1); coordinates.x];
%             coordinates_tmp.y = [coordinates.y+coordinates.b(2); coordinates.y];
%             
%             H_tmp = [sparse([],[],[],size(identity_mtx,2), size(identity_mtx,1)), identity_mtx; ...
%                 identity_mtx', sparse([],[],[],size(identity_mtx,1), size(identity_mtx,2))];     
%             
%             H_tmp = gaugeTransformation( H_tmp, coordinates_tmp, gauge_field );
%             identity_mtx = H_tmp( 1:size(H1_transverse,2), size(H1_transverse,1)+1:end );
%             q = q - angle( diag(identity_mtx) );
%             %Surface_tmp.Write( 'q', q);
%         end
        
        
        Surface_tmp.Write( 'GaugeTransformationApplied', true);
        
        
    end

%% CreateClone
%> @brief Creates a clone of the present object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )        
        ret = Peierls(obj.Opt, ...
            'parameter', obj.parameter, ...
            'Vectorpotential', obj.Vectorpotential );             
        
    end   
    
%% setVectorPotential
%> @brief Use to set the handle for the vector potential.
%> @param vectorpot The function handle of the vector potential
function setVectorPotential( obj, vectorpot )
	obj.Vectorpotential = vectorpot;
end  


end % methods public


methods ( Access = protected )

%% Peierls_transzform
%> @brief Calculates the Peierls phase along the bond.
%> @param kezdopont Coordinates ([x,y]) of the starting point. 
%> @param vegpont Coordinates ([x,y]) of the end point. 
%> @return Returns with the peierls phase
function fazis = Peierls_transzform(obj, kezdopont,vegpont)

if obj.Opt.Linear_Regression_in_B
    A1        = obj.Vectorpotential(kezdopont(:,1),kezdopont(:,2)); 
    A2        = obj.Vectorpotential(vegpont(:,1),vegpont(:,2));  
    fazis     = 0.5*(A1(:,1)+A2(:,1)).*(vegpont(:,1)-kezdopont(:,1)) + 0.5*(A1(:,2)+A2(:,2)).*(vegpont(:,2)-kezdopont(:,2));
    return
end
    
% the following is not vectorized, and is obsolete !!!! TODO
fazis = zeros( size(kezdopont,1), 1);
for idx = 1:length(fazis)
    if ~(kezdopont(1) == vegpont(1))
        dx        = (vegpont(idx,1)-kezdopont(idx,1))/100;
        int_array = kezdopont(idx,1):dx:vegpont(idx,1);
        Ind       = max(size(int_array));
        iidx       = 1;
        A         = zeros(1,Ind);
        while iidx <= Ind
            x      = int_array(iidx);
            A_temp = obj.Vectorpotential(x, y_temp(x,kezdopont(idx,:),vegpont(idx,:))); 
            A(iidx) = A_temp(1);
            iidx    = iidx + 1;
        end
        fazis(idx)     = trapz(int_array,A);
    end
end

if obj.Opt.my_own_potential
    fazis = zeros( size(kezdopont,1), 1);
    for idx = 1:length(fazis)
        if ~(kezdopont(idx,2) == vegpont(idx,2) )
            dy        = (vegpont(idx,2)-kezdopont(idx,2))/100;
            int_array = kezdopont(idx,2):dy:vegpont(idx,2);
            Ind       = max(size(int_array));
            iidx       = 1;
            A         = zeros(1,Ind);
            while iidx <= Ind
                y      = int_array(iidx);
                A_temp = obj.Vectorpotential(x_temp(y,kezdopont(idx,:),vegpont(idx,:)), y);
                A(iidx) = A_temp(2);
                iidx    = iidx + 1;
            end
            fazis(idx)     = fazis(idx) + trapz(int_array,A);
        end
    end
end

    function x = x_temp(y,kezdopont,vegpont)
        x = vegpont(1) + (kezdopont(1)-vegpont(1))/(vegpont(2)-kezdopont(2))*(vegpont(2)-y);
    end
    function y = y_temp(x,kezdopont,vegpont)
        y = vegpont(2) + (kezdopont(2)-vegpont(2))/(vegpont(1)-kezdopont(1))*(vegpont(1)-x);
    end
end

%% CreateHandleForVectorpotential
%> @brief Creates function handles for in-built Vector potentials
%> @return Returns A function handle
function HandleForA = CreateHandleForVectorpotential( obj )    
        
    
    if ~isempty( obj.gauge )  
        if strcmp( obj.gauge, 'LandauX' )
            HandleForA = @(x,y)([-obj.eta*y;0]);
        elseif strcmp( obj.gauge, 'LandauY' )
            HandleForA = @(x,y)([0;obj.eta*x]);
        else
            HandleForA = [];
        end
    else
        HandleForA = [];
    end

end


end % methods protected



methods ( Access = private )


%% inputparsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
function inputparsing( obj, varargin)
    p_validating = inputParser;
    p_validating.addParamValue('parameter', 1);
    p_validating.addParamValue('Vectorpotential', []);
    p_validating.parse(varargin{:});
        
    obj.parameter       = p_validating.Results.parameter;
    obj.Vectorpotential = p_validating.Results.Vectorpotential;
    obj.gauge           = [];
    obj.eta             = 0;
    
    
    if isfield( obj.Opt, 'eta' )
        obj.eta   = obj.Opt.eta;
    else
        obj.eta = 0;
    end    
    
    
    if isempty( obj.Vectorpotential )
        if isfield( obj.Opt, 'gauge' )
            obj.gauge     = obj.Opt.gauge;
        else
            obj.gauge = [];
        end
        obj.Vectorpotential = obj.CreateHandleForVectorpotential();
    end
        

end

end % methods private

end %Global end


    


