%%   Eotvos Quantum Transport Utilities - Square_Lead_Hamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%   Class to create the Hamiltonian of one unit cell in a ribbon made of square lattice structure, including the SSH model.
%%
classdef Square_Lead_Hamiltonians


methods (Static = true)
%% SquareLattice_Lead_Hamiltonians
%> @brief Creates Hamiltonians H_0 and H_1 for square lattice structure
%> @param lead_param An instance of structure lead_param containing the physical parameters of the ribbon
%> @param M Number of sites in the cross section of the lead.
%> @param varargin Optional parameters, see the web documentation for details.
%> @return Returns with the created Hamiltonians, for details see the web documentation.
    function [H0, H1, H1_transverse, coordinates] = SquareLattice_Lead_Hamiltonians( lead_param, M, varargin)
    
        p = inputParser;
        p.addParamValue('q', []);
        
        p.parse(varargin{:});    
        q = p.Results.q;
    
        epsilon  = lead_param.epsilon;
        vargamma = lead_param.vargamma;
            
        H0 = sparse(1:M,1:M,epsilon,M,M) - sparse(1:M-1,2:M,vargamma,M,M) - sparse(2:M,1:M-1,vargamma,M,M);
        H1 = sparse(1:M,1:M,-vargamma,M,M);
            
        coordinates = structures('coordinates');
        coordinates.x = transpose(1:M);
        coordinates.y = zeros(M,1);
        coordinates.a = [0; 1];
            
        if M == 1
            H0 = full(H0);
            H1 = full(H1);
        end
            
        if isempty(q) 
            H1_transverse = [];
            return
        end
            
        coordinates.b = [M,0];
        H1_transverse = sparse(M,1,-vargamma,M,M);   
    
    
    end

%% SSH_Lead_Hamiltonians
%> @brief Creates Hamiltonians H_0 and H_1 of the SSH model.
%> @param lead_param An instance of structure lead_param containing the physical parameters of the ribbon
%> @param varargin Optional parameters, see the web documentation for details.
%> @return Returns with the created Hamiltonians, for details see the web documentation.
    function [H0, H1, H1_transverse, coordinates] = SSH_Lead_Hamiltonians(lead_param, varargin)
    
        p = inputParser;
        p.addParamValue('q', []);
        
        p.parse(varargin{:});    
        q = p.Results.q;
    
        epsilon  = lead_param.epsilon;
        vargamma1 = lead_param.vargamma1;
        vargamma3 = lead_param.vargamma3;
        deltaAB = lead_param.deltaAB; %NEW
        if isempty( deltaAB )
            deltaAB = 0;
        end
    
        H0 = zeros(2,2);
        H1 = zeros(2);
    
        H0(1,2) = -vargamma1;
        H0(2,1) = -vargamma1;
        H0(1,1) = deltaAB/2 + epsilon;
        H0(2,2) = -deltaAB/2 + epsilon;
    
        H1(2,1) = -vargamma3;
    
        H1_transverse = [];
                
        coordinates = structures('coordinates');
        coordinates.x = transpose(1:2);
        coordinates.y = zeros(2,1);
        coordinates.a = [0; 1];       
    
        if ~isempty(q)
            error('Transverse computations are not allowed in the SSH model')
        end
    
    
    end

%% Lieb_Lead_Hamiltonians
%> @brief Creates Hamiltonians H_0 and H_1 for Lieb lattice structure
%> @param lead_param An instance of structure lead_param containing the physical parameters of the ribbon
%> @param M Number of sites in the cross section of the lead.
%> @param varargin Optional parameters, see the web documentation for details.
%> @return Returns with the created Hamiltonians, for details see the web documentation.
    function [H0, H1, H1_transverse, coordinates] = Lieb_Lead_Hamiltonians( lead_param, M, varargin)
    
        p = inputParser;
        p.addParamValue('q', []);
        
        p.parse(varargin{:});    
        q = p.Results.q;

        epsilon  = lead_param.epsilon;
        vargamma = lead_param.vargamma;
        N = 3*M;
           

        h0=toeplitz([epsilon,-vargamma,0]);
	h1=-vargamma*[0 0 0; 1 0 0; 0 0 0];
	h2=-vargamma*[0 0 0; 0 0 0; 0 1 0];

	H0=sparse(zeros(N));
	H1=sparse(zeros(N));

	for l=1:3;
          for m=1:3;
            H0 = H0 + sparse(l:3:N,m:3:N,h0(l,m),N,N) + sparse(l:3:N-3,m+3:3:N,h1(l,m),N,N) + sparse(l+3:3:N,m:3:N-3,h1(m,l),N,N);
            H1 = H1 + sparse(l:3:N,m:3:N,h2(l,m),N,N);
          end
        end

%TODO            
        coordinates = structures('coordinates');
        coordinates.x = transpose(1:3*M);
        coordinates.y = zeros(3*M,1);
        coordinates.a = [0; 1];
%
            
        if M == 1
            H0 = full(H0);
            H1 = full(H1);
        end
            
        if isempty(q) 
            H1_transverse = [];
            return
        end
            
        coordinates.b = [M,0];
        H1_transverse = sparse(M,1,-vargamma,3*M,3*M);   
    
    
    end

end % methods static


end
