%%    Eotvos Quantum Transport Utilities - Surface_Green_function
%    Copyright (C) 2009-2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to calculate the Greend functions and self energies of a translational invariant lead
%%
classdef Surface_Green_function < handle & CommonFunctions & EigenProblemLead
    
    
properties ( Access = protected )
	%>function handle to calculate the self energy
    SelfEnergyCalc
	%>Function handle to calculate the infinite Greens function.
    GinfCalc    
	%>The normalization matrix in the Greens function.
    Normamtx
	%>The inverse of the normalization matrix.
    invNormamtx
	%>The matrix of the surface Greens function of the semi-infinite ribbon.
    gsurf
	%>The inverse of the Greens function.
    gsurfinv
	%>The matrix of the Greens function of the infinite ribbon.
    ginf
	%>The matrix of the surface Greens function of a finite ribbon.
    gfin
	%>The inverse of the surface Greens function of a finite ribbon.
    gfininv
	%>The self-energy of the semi-infinite ribbon.
    Sigma
end





methods ( Access = public )
%% constructorof the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = Surface_Green_function(Opt, param, varargin)    
        obj = obj@EigenProblemLead( Opt, param, varargin{:} ); 

        if strcmpi(class(obj), 'Surface_Green_function')       
            obj.Reset(); 
        end       
    end

%% InfGreenFunction
%> @brief Calculates the Greens function of the infinite lead between slabs z1 and z2. The calculated Greens function is stored within the current object. 
%> @param z1 The index of the first slab.
%> @param z2 The index of the second slab.
%> @param varargin Optional parameters, see the web documenattion for details.
%> @return Return with the  Green function of the infinite lead between the given slabs.
    function ret = InfGreenFunction(obj, z1,z2, varargin)
        obj.szetvalaszto();
        
        p = inputParser;
        if z1 >= z2
            p.addParamValue('z1points', 1:size(obj.modusmtx_p,1));
            p.addParamValue('z2points', 1:size(obj.modusmtx_p,2));
        else
            p.addParamValue('z1points', 1:size(obj.modusmtx_m,1));
            p.addParamValue('z2points', 1:size(obj.modusmtx_m,2));            
        end
        p.parse(varargin{:});

        z1points = p.Results.z1points;
        z2points = p.Results.z2points;
        
        obj.ginf = obj.GinfCalc(z1, z2, z1points, z2points);
        ret = obj.ginf;
        
%        d_modusmtx_m = [];
%        d_modusmtx_p = [];
    end
    
end

methods ( Access = protected )
%%    GinfCalcWithDualModes     
%> @brief Calculates the Green function of the infinite ribbon with dual modes
%> @param z1 The index of the first slab.
%> @param z2 The index of the second slab.
%> @param z1points Site indexes in the slab at z1.
%> @param z2points Site indexes in the slab at z2.
%> @return Return with the  Greens function of the infinite lead between the given slabs.
    function ginf = GinfCalcWithDualModes(obj, z1, z2, z1points, z2points) 
        obj.NormamtxSzamolo();
        obj.ginf = 0;
        
        if z1 >= z2
            ginf = obj.modusmtx_p(z1points,:)*diag(obj.expk_p.^(obj.Lead_Orientation*(z1-z2)))*obj.d_modusmtx_p;
        else
%             for idx = 1:M          
%                 xi_m    = modusmtx_m(z1points,idx);
%                 d_xi_m  = d_modusmtx_m(idx,:);
%                 ginf = ginf + xi_m*d_xi_m*exp(1i*k_m(idx)*Lead_Orientation*(z1-z2));  
%             end
            ginf = obj.modusmtx_m(z1points,:)*diag(obj.expk_m.^(obj.Lead_Orientation*(z1-z2)))*obj.d_modusmtx_m;
        end
        
        ginf = ginf*obj.invNormamtx(:,z2points);
    end
    %--------------------------------------------------------------------------
    
%%    GinfCalcWithLeftModes
%> @brief Calculates the Green function of the infinite ribbon with with the left-sided modes
%> @param z1 The index of the first slab.
%> @param z2 The index of the second slab.
%> @param z1points Site indexes in the slab at z1.
%> @param z2points Site indexes in the slab at z2.
%> @return Return with the  Greens function of the infinite lead between the given slabs.
    function ginf = GinfCalcWithLeftModes(obj, z1, z2, z1points, z2points) 
        
        if z1 >= z2
            ginf = obj.modusmtx_p(z1points,:)*diag(obj.expk_p.^(obj.Lead_Orientation*(z1-z2))./(1i*obj.vcsop_p*obj.Lead_Orientation))*obj.modusmtx_p_left(:,z2points);
        else
            ginf = obj.modusmtx_m(z1points,:)*diag(obj.expk_m.^(obj.Lead_Orientation*(z1-z2))./(-1i*obj.vcsop_m*obj.Lead_Orientation))*obj.modusmtx_m_left(:,z2points);
        end    
        
    end
    %--------------------------------------------------------------------------
    
end    


methods ( Access = public )
    
%% Gamma
%> @brief Calculates the effective coupling of the lead to the scattering region according to Eq (3) in Eur. Phys. J. B 53, 537-549 (2006)
%> @return Return with the  matrix of the effective coupling.
    function ret = Gamma( obj )
       if isempty( obj.Sigma )
           obj.SelfEnergy();
       end
       
       ret = 1i*( obj.Sigma - (obj.Sigma)' );
       
       
    end
%% SelfEnergy
%> @brief Calculates the self energy of the semi-infinite lead according to Eq (36) of Ref PRB 78, 035407 (2008). The calculated self energy is stored within the current object. 
%> @return Returns with the calculated self energy.
% 
    function ret = SelfEnergy( obj )
        obj.szetvalaszto();        
        
        obj.Sigma = obj.SelfEnergyCalc();
        ret = obj.Sigma;
        
    end
    
    
end


methods ( Access = protected )    

%% SelfEnergyCalcWithDualModes
%> @brief Calculates the self energy with dual modes according to Eqs (17) and (36) in PRB 78 035407
%> @return Return with the self energy.
    function Sigma = SelfEnergyCalcWithDualModes( obj ) 
    %>Eq (17) and (36) in PRB 78 035407
    %>g00 = inv(Normamtx)
        obj.NormamtxSzamolo();      
        Sigma = obj.modusmtx_m*diag(obj.expk_m.^(-obj.Lead_Orientation))*obj.d_modusmtx_m;
        if obj.Lead_Orientation == 1
            Sigma = obj.H1adj*Sigma;
        else
            Sigma =  obj.H1*Sigma;
        end
    end
    %--------------------------------------------------------------------------

%% SelfEnergyCalcWithLeftModes
%> @brief Calculates the self energy with the left-sided modes according to Eqs (20), (17) and (36) in PRB 78 035407
%> @return Return with the self energy.
    function Sigma = SelfEnergyCalcWithLeftModes( obj ) 
    %>Eq (20), (17) and (36) in PRB 78 035407
    %>g00 = inv(Normamtx)
        obj.NormamtxSzamolo(); 
        Sigma = obj.modusmtx_m*diag(obj.expk_m.^(-obj.Lead_Orientation)./(-1i*obj.vcsop_m*obj.Lead_Orientation))*obj.modusmtx_m_left;
        if obj.Lead_Orientation == 1
            Sigma = obj.H1adj*Sigma*obj.Normamtx;
        else
            Sigma = obj.H1*Sigma*obj.Normamtx;
        end                
    end
    %--------------------------------------------------------------------------  
    
end

methods ( Access = public )

%% FiniteGreenFunction
%> @brief Calculates the Green function of a finite piece of the lead between slabs z1 and z2. The calculated Greens function is stored by the current object.
%> @param z1 The index of the first slab.
%> @param z2 The index of the second slab.
%> @param varargin Optional parameters, see the web documenattion for details.
%> @return Return with the  Green function of the infinite lead between the given slabs.
    function ret = FiniteGreenFunction(obj, z1,z2, varargin)
        
        p = inputParser;
        p.addParamValue('onlygfininv', false );
        
        p.parse(varargin{:});       
        onlygfininv    = p.Results.onlygfininv;
        
        obj.szetvalaszto();
        obj.NormamtxSzamolo();        
        
        obj.gfin = zeros(4*obj.M,4*obj.M);
        
        if z1 >= z2
            points = 1:size(obj.modusmtx_p,1);
            dpoints = 1:size(obj.modusmtx_p,1);
        else
            points = 1:size(obj.modusmtx_m,1);
            dpoints = 1:size(obj.modusmtx_m,1);  
        end
        
        M = obj.M;
        gtmp = obj.GinfCalc(z1-1, z1-1, points, dpoints);
        obj.gfin(1:M,1:M)              = gtmp;
        obj.gfin(1+M:2*M, 1+M:2*M)     = gtmp;
        obj.gfin(1+2*M:3*M, 1+2*M:3*M) = gtmp;              
        obj.gfin(1+3*M:4*M, 1+3*M:4*M) = gtmp;
        
        gtmp = obj.GinfCalc(z1-1, z1, points, dpoints);
        obj.gfin(1:M,1+M:2*M) = gtmp;
        obj.gfin(1+2*M:3*M,1+3*M:4*M) = gtmp;
        
        gtmp = obj.GinfCalc(z1, z1-1, points, dpoints);
        obj.gfin(1+M:2*M,1:M) = gtmp;
        obj.gfin(1+3*M:4*M, 1+2*M:3*M) = gtmp;    
        
        gtmp = obj.GinfCalc(z1-1, z2, points, dpoints);
        obj.gfin(1:M,1+2*M:3*M) = gtmp; 
        
        gtmp = obj.GinfCalc(z2, z1-1, points, dpoints);
        obj.gfin(1+2*M:3*M,1:M) = gtmp; 
                
        gtmp = obj.GinfCalc(z1-1, z2+1, points, dpoints);
        obj.gfin(1:M,1+3*M:4*M) = gtmp; 
                
        gtmp = obj.GinfCalc(z2+1, z1-1, points, dpoints);
        obj.gfin(1+3*M:4*M,1:M) = gtmp;   
        
        gtmp = obj.GinfCalc(z1, z2, points, dpoints);
        obj.gfin(1+M:2*M,1+2*M:3*M) = gtmp; 
        
        gtmp = obj.GinfCalc(z2, z1, points, dpoints);
        obj.gfin(1+2*M:3*M,1+M:2*M) = gtmp; 
        
        gtmp = obj.GinfCalc(z1, z2+1, points, dpoints);
        obj.gfin(1+M:2*M,1+3*M:4*M) = gtmp; 
        
        gtmp = obj.GinfCalc(z2+1, z1, points, dpoints);
        obj.gfin(1+3*M:4*M,1+M:2*M) = gtmp;   
        
        rcond_gfin = rcond(obj.gfin);
        if isnan(rcond_gfin) || abs( rcond_gfin ) < 1e-15
            obj.display( 'Regularizing gfin by SVD',1);
            obj.gfininv = obj.inv_SVD( obj.gfin );
        else
            obj.gfininv = inv(obj.gfin);
        end       
        obj.gfininv = obj.gfininv( 1+M:3*M, 1+M:3*M );
        
        if onlygfininv
            ret = [];
        else
            rcond_gfinin = rcond(obj.gfininv);
            if isnan(rcond_gfinin ) || abs( rcond_gfinin ) < 1e-15
                obj.display( 'Regularizing gfininv by SVD',1);
                obj.gfin = obj.inv_SVD( obj.gfininv );
            else
                obj.gfin = inv(obj.gfininv);
            end       
            obj.gfininv = [];
            ret = obj.gfin;
        end
        
    end

%% SurfaceGreenFunction
%> @brief Calculates the surface Green function of a semi-infinite lead. The calculated Green function is stored by the current object.
%> @param varargin Optional parameters, see the web documenattion for details.
    function SurfaceGreenFunction( obj, varargin ) 

    	p = inputParser;
    	p.addParamValue('OnlyInverse', true ); %NEW
    	p.addParamValue('CalcInverse', true ); %NEW
    	p.parse(varargin{:});
    	OnlyInverse = p.Results.OnlyInverse;     
		CalcInverse = p.Results.CalcInverse;     
            
        obj.szetvalaszto();
        obj.NormamtxSzamolo(); 
        if obj.Opt.usingDualModes
            obj.GsurfSzamoloWithDualModes();
        else
            obj.GsurfSzamoloWithLeftModes();
        end 

		if ~CalcInverse
			return
		end

        rcond_gsurf = rcond(obj.gsurf);
        if isnan(rcond_gsurf) || abs( rcond_gsurf ) < 1e-15
            obj.display( 'Regularizing gsurf by SVD',1);
            obj.gsurfinv = obj.inv_SVD( obj.gsurf );
        else
            obj.gsurfinv = inv(obj.gsurf);
        end  

		if  OnlyInverse            
        	obj.gsurf = [];
		end      
        
    end
    
end

methods ( Access = protected )
    
%> @brief Calculates the surface Green function with dual modes. The calculated Green function is stored by the current object.
    function GsurfSzamoloWithDualModes( obj )  
            
            %test for the self energy
%              obj.SelfEnergy();
%              gsurf2 = -inv((obj.H0) + obj.Sigma);
        
            
        gsurf_m = obj.modusmtx_m*diag(obj.expk_m.^(-obj.Lead_Orientation))*obj.d_modusmtx_m;
        gsurf_p = obj.modusmtx_p*diag(obj.expk_p.^(obj.Lead_Orientation))*obj.d_modusmtx_p;
             

        %gsurf = (eye(M) - gsurf_p*gsurf_m)*inv(Normamtx);
        obj.gsurf = (eye(obj.M) - gsurf_m*gsurf_p)*obj.invNormamtx;

            
            
    end
    %-------------------------------------------------------------------------- 
        
        
%> @brief Calculates the surface Green function with the left-sided modes according to Eq (33) in PRB 78, 035407 (2008). The calculated Green function is stored by the current object.
    function GsurfSzamoloWithLeftModes( obj )  
            
        points = 1:obj.M;
        dpoints = 1:obj.M;
            
    %>right SGF Eq (33) in PRB 78, 035407 (2008)
    %>g00 = inv(Normamtx)
%         g01 = GinfCalc(0, 1, points, dpoints);
%         g10 = GinfCalc(1, 0, points, dpoints);
%         gsurfR = invNormamtx - g10*Normamtx*g01;
            
    %>left SGF Eq (33) in PRB 78, 035407 (2008)
    %>g00 = inv(Normamtx)
        g0m1 = obj.GinfCalc(0, -1, points, dpoints);
        gm10 = obj.GinfCalc(-1, 0, points, dpoints);
        obj.gsurf = obj.invNormamtx - gm10*obj.Normamtx*g0m1;
                       
        
    end
    %--------------------------------------------------------------------------  
    
end

methods ( Access = public )
     
%%    NormamtxSzamolo
%> @brief The normalization matrix to calculate the Green function. The calculated normalization matrix is stored by the current object.
    function NormamtxSzamolo( obj )
            
            if ~isempty(obj.Normamtx) && ~isempty(obj.invNormamtx)
               return 
            elseif obj.Opt.usingDualModes
                obj.NormamtxSzamoloWithDualModes();
            else
                obj.NormamtxSzamoloWithLeftModes();
            end       
    end
    
end
                        

methods ( Access = protected )
%% NormamtxSzamoloWithDualModes
%> @brief Calculates the normalization matrix to calculate the Green function using the dual modes. The calculated normalization matrix is stored by the current object.
    function NormamtxSzamoloWithDualModes(obj)
            
        obj.calcDualModes();  
        obj.Normamtx = 0;
        obj.invNormamtx = 0;
            
            
        obj.Normamtx = (obj.modusmtx_p*diag(obj.expk_p.^(-1))*obj.d_modusmtx_p - ...
                    obj.modusmtx_m*diag(obj.expk_m.^(-1))*obj.d_modusmtx_m)*obj.Lead_Orientation;        
                
        
        obj.Normamtx = obj.H1adj*obj.Normamtx;
        
        try
            rcond_Normamtx = rcond(obj.Normamtx);
            if isnan(rcond_Normamtx) || abs( rcond_Normamtx ) < 1e-15
                obj.display( 'Regularizing Normamtx by SVD',1);
                obj.invNormamtx = obj.inv_SVD( obj.Normamtx );
            else
                obj.invNormamtx = obj.Normamtx\(eye(size(obj.Normamtx)));%inv(Normamtx);
            end                
        catch errCause
            err = MException('Surface_Green_function:NormamtxSzamoloWithDualModes', 'The inversion of the Normamtx failed');
            err = addCause(err, errCause);
            save('Error_Surface_Green_function_NormamtxSzamoloWithDualModes.mat');
            throw(err); 
        end
    end
    %---------------------------------------------------------------
%% NormamtxSzamoloWithLeftModes
%> @brief Calculates the normalization matrix to calculate the Green function using the left-sided modes. The calculated normalization matrix is stored by the current object.
    function NormamtxSzamoloWithLeftModes(obj)
           
        obj.invNormamtx = obj.InfGreenFunction(0,0);
        rcond_invNormamtx = rcond(obj.invNormamtx);
        if isnan(rcond_invNormamtx) || abs( rcond_invNormamtx ) < 1e-15
            obj.display( 'Regularizing invNormamtx by SVD',1);
            obj.Normamtx = obj.inv_SVD( obj.invNormamtx );
        else
            obj.Normamtx    = obj.invNormamtx\(eye(size(obj.invNormamtx)));
        end                
           
            
    end

%% calcDualModes
%> @brief Calculates the dual modes. The dual modes is stored by the current object. 
    function  calcDualModes( obj )
        if isempty(obj.d_modusmtx_m) 
            obj.display('Calculating dual modes for m')
            if obj.Opt.usingDualModes
           		obj.d_modusmtx_m = obj.CalcDualModesEigenvec( obj.modusmtx_m ); %a dualis modusok
            else
				obj.d_modusmtx_m = obj.CalcDualModesEigenvecWithLeftModes( obj.modusmtx_m_left, -obj.vcsop_m*obj.Lead_Orientation ); %a dualis modusok
            end
       end
       
       if isempty(obj.d_modusmtx_p) 
            obj.display('Calculating dual modes for p')
			if obj.Opt.usingDualModes
           		obj.d_modusmtx_p = obj.CalcDualModesEigenvec( obj.modusmtx_p ); %a dualis modusok
			else
				obj.d_modusmtx_p = obj.CalcDualModesEigenvecWithLeftModes( obj.modusmtx_p_left, obj.vcsop_p*obj.Lead_Orientation ); %a dualis modusok
			end
       end

%  	obj.display('modusmtx inverse',1);     
%     obj.display(num2str(norm( obj.modusmtx_m*obj.d_modusmtx_m - eye(size(obj.modusmtx_m)) )), 1);
%     obj.display(num2str(norm( obj.modusmtx_p*obj.d_modusmtx_p - eye(size(obj.modusmtx_p)) )), 1);   
    end


%% CalcDualModesEigenvec
%> @brief Calculates the dual modes by inverting the right-sided eigenvectors. The dual modes is stored by the current object.
%> @param modusmtx The right-sided eigenvectors. 
%> @return Returns with the calculated dual modes.
    function d_modes = CalcDualModesEigenvec( obj, modusmtx )  
            
        try
            rcond_modusmtx = rcond(modusmtx);
            if isnan(rcond_modusmtx ) || abs( rcond_modusmtx ) < 1e-15
                obj.display( 'Regularizing modusmtx by SVD',1);
                d_modes = obj.inv_SVD( modusmtx );
            else
                d_modes = (modusmtx)\eye(size(modusmtx)); % a dualis modusok
            end                
        catch errCause
            err = MException('Surface_Green_function:CalcDualModesEigenvec', 'The inversion of the modusmtx failed');
            err = addCause(err, errCause);
            save('Error_Surface_Green_function_CalcDualModesEigenvec.mat');
            throw(err); 
        end
            
    end

%% CalcDualModesEigenvecWithLeftModes
%> @brief Calculates the dual modes using the left-sided eigenvectors. The dual modes is stored by the current object.
%> @param modusmtx_left The left-sided eigenvectors. 
%> @param vcsop The group velocities.
%> @return Returns with the calculated dual modes
    function d_modes = CalcDualModesEigenvecWithLeftModes( obj, modusmtx_left, vcsop ) 
    %>Eq (20) in  PRB 78, 035407 (2008)
		d_modes = diag(1./(1i*vcsop))*modusmtx_left*obj.Normamtx;
    end    
    
end
   
methods ( Access = public )

%% TrukkosSajatertekek
%> @brief Calculates the wave numbers corresponding to the propagating states at given energy. The calculated wave numbers are stored within the object. The normalization matrix is set to empty.
%> @param E The energy value used in the calculations.
    function TrukkosSajatertekek(obj, E)
        TrukkosSajatertekek@EigenProblemLead(obj, E);
        obj.Normamtx       = [];
        obj.invNormamtx    = [];        
    end
    
%% ShiftLead
%   Shifts the on-site energies in the leads by a given energy. The normalization matrix is set to empty.
%> @param Energy The enrgy value.
    function ShiftLead( obj, Energy )        
        ShiftLead@EigenProblemLead( obj, Energy );
        obj.Normamtx = [];
    end

%% AddPotential
%> @brief Adds on-site potential to the Hamiltonian H0.
%> @param V The potential calculated on the sites. The normalization matrix is set to empty.
    function AddPotential( obj, V )        

        AddPotential@EigenProblemLead( obj, V );
        obj.Normamtx = [];
    end

%% CreateClone
%> @brief Creates a clone of the present Surface_Green_function object.
%> @return Returns with the cloned object.
    function Surface_tmp = CreateClone( obj )
        
        
        Surface_tmp = Surface_Green_function(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', obj.Hanyadik_Lead,...
                                            'Lead_Orientation', obj.Lead_Orientation, ...
                                            'q', obj.q);                    
                                        
        Surface_tmp.Write( 'H0', obj.H0); 
        Surface_tmp.Write( 'H1', obj.H1);
        Surface_tmp.Write( 'H1adj', obj.H1adj);
        Surface_tmp.Write( 'H1_transverse', obj.H1_transverse);
        Surface_tmp.Write( 'S0', obj.S0); 
        Surface_tmp.Write( 'S1', obj.S1);
        Surface_tmp.Write( 'S1_transverse', obj.S1_transverse);        
        Surface_tmp.Write( 'fazis_mtx_H0', obj.fazis_mtx_H0); 
        Surface_tmp.Write( 'fazis_mtx_H1', obj.fazis_mtx_H1);        
        Surface_tmp.Write( 'fazis_mtx_H1t', obj.fazis_mtx_H1t);
        Surface_tmp.Write( 'H00', obj.H00);
        Surface_tmp.Write( 'kulso_szabfokok', obj.kulso_szabfokok);
		Surface_tmp.Write( 'coordinates', obj.coordinates);
        Surface_tmp.Write( 'gsurf', obj.gsurf);
        Surface_tmp.Write( 'ginf', obj.ginf);
        Surface_tmp.Write( 'gfin', obj.gfin);
        Surface_tmp.Write( 'Normamtx', obj.Normamtx);
        Surface_tmp.Write( 'vcsop_p', obj.vcsop_p);
        Surface_tmp.Write( 'vcsop_m', obj.vcsop_m);
        Surface_tmp.Write( 'modusmtx_p', obj.modusmtx_p);
        Surface_tmp.Write( 'modusmtx_m', obj.modusmtx_m);
        Surface_tmp.Write( 'expk_p', obj.expk_p);
        Surface_tmp.Write( 'expk_m', obj.expk_m);
        Surface_tmp.Write( 'params', obj.params);
        Surface_tmp.Write( 'HamiltoniansCreated', obj.HamiltoniansCreated);
        Surface_tmp.Write( 'OverlapApplied', obj.OverlapApplied);
        Surface_tmp.Write( 'HamiltoniansDecimated', obj.HamiltoniansDecimated);
        Surface_tmp.Write( 'MagneticFieldApplied', obj.MagneticFieldApplied); 
        Surface_tmp.Write( 'GaugeTransformationApplied', obj.GaugeTransformationApplied);
        Surface_tmp.Write( 'M', obj.M); 

        
        
        
    end   

%% Reset
%> @brief Resets all elements in the object.
    function Reset( obj )
        
        Reset@EigenProblemLead( obj );
        


        obj.Normamtx      = [];
        obj.invNormamtx   = [];
        obj.gsurf         = [];
        obj.gsurfinv      = [];
        obj.ginf          = [];
        obj.gfin          = [];
        obj.gfininv       = [];
        obj.Sigma        = [];
        
        
        if obj.Opt.usingDualModes
            obj.display( 'Using dual modes in the calculations.');
            obj.GinfCalc = @(z1, z2, z1points, z2points)(obj.GinfCalcWithDualModes(z1, z2, z1points, z2points));
            obj.SelfEnergyCalc = @()(obj.SelfEnergyCalcWithDualModes);
        else
            obj.display( 'Using left-sided modes in the calculations.');
            obj.GinfCalc = @(z1, z2, z1points, z2points)(obj.GinfCalcWithLeftModes(z1, z2, z1points, z2points));
            obj.SelfEnergyCalc = @()(obj.SelfEnergyCalcWithLeftModes);
        end
             
    end

%% Write
%> @brief Sets the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @param input The value to be set.
    function Write(obj, MemberName, input)       
        
        
        if strcmp(MemberName, 'gsurf')
            obj.gsurf = input;
        elseif strcmp(MemberName, 'gsurfinv')
            obj.gsurfinv = input;
        elseif strcmp(MemberName, 'ginf')
            obj.ginf = input;
        elseif strcmp(MemberName, 'gfin')
            obj.gfin = input;            
        elseif strcmp(MemberName, 'Normamtx')
            obj.Normamtx = input;
        elseif strcmp(MemberName, 'Sigma')
            obj.Sigma = input;     
        elseif strcmp(MemberName, 'd_modusmtx_p')
            obj.d_modusmtx_p = input;
        elseif strcmp(MemberName, 'd_modusmtx_m')
            obj.d_modusmtx_m = input;                        
        else
            Write@EigenProblemLead( obj, MemberName, input );
        end
        
        
    end
%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be set.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        
        if strcmp(MemberName, 'gsurf')
            ret = obj.gsurf;
        elseif strcmp(MemberName, 'gsurfinv')
            ret = obj.gsurfinv;            
        elseif strcmp(MemberName, 'ginf')
            ret = obj.ginf; 
        elseif strcmp(MemberName, 'gfin')
            ret = obj.gfin;   
        elseif strcmp(MemberName, 'gfininv')
            ret = obj.gfininv;               
        elseif strcmp(MemberName, 'Normamtx')
            ret = obj.Normamtx;         
        elseif strcmp(MemberName, 'sort_tolerance')
            ret = obj.sort_tolerance;
        elseif strcmp(MemberName, 'Sigma')
            ret = obj.Sigma; 
        elseif strcmp(MemberName, 'd_modusmtx_p')
            obj.calcDualModes()
            ret = obj.d_modusmtx_p;
        elseif strcmp(MemberName, 'd_modusmtx_m')
            obj.calcDualModes()
            ret = obj.d_modusmtx_m;             
        else            
            ret = Read@EigenProblemLead( obj, MemberName );
        end
    end
%% Clear
%> @brief Clears the value of an attribute in the interface.
%> @param MemberName The name of the attribute to be cleared.
    function Clear(obj, MemberName)
        if strcmp(MemberName, 'Normamtx')
            obj.Normamtx = [];
        elseif strcmp(MemberName, 'gsurf')
            obj.gsurf = [];
        elseif strcmp(MemberName, 'ginf')
            obj.ginf = [];
        elseif strcmp(MemberName, 'gfin')
            obj.gfin = [];   
        elseif strcmp(MemberName, 'd_modusmtx_p')
            obj.d_modusmtx_p = [];
        elseif strcmp(MemberName, 'd_modusmtx_m')
            obj.d_modusmtx_m = [];                
        else
            Clear@EigenProblemLead( obj, MemberName );
        end
        
    end

end

%------------------------------------------------------------------
end % Global End
