%%    Eotvos Quantum Transport Utilities - CreateHamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    Returns an interface to evaluate the Dyson equation and to calculate the scattering matrix.
%%
classdef Transport_Interface < handle & Messages


    properties ( Access = protected )
        %> The energy to be used in the calculations.
        E
        %> An instance of the structure param
        param
        %> The calculated Greens function.
        G
        %> The calculated scattering matrix.
        Smatrix
        %> Numbers of the open channels in the leads.
        nyitott_csatornak
        %> The matrix of the calculated conductance.
        Conductance_Matrix
        %> A function handle of the Dyson equation.
        Dysonfunc
        %> A function handle of the Dyson equation.
        CreateH
        %> A function handle of the Dyson equation.
        PeierlsTransform
        %> An array of classes Surface_Green_Function
        Surface
    end




methods

%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param param An instance of the structure param.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
function obj = Transport_Interface(E, Opt, param, varargin)
    obj = obj@Messages( Opt );
    obj.E = E;
    obj.param = param;
    obj.G = [];
    obj.Smatrix = [];
    obj.nyitott_csatornak  = [];
    obj.Conductance_Matrix = [];
    obj.Dysonfunc = [];

    obj.CreateH = [];
    obj.PeierlsTransform = [];
    obj.Surface = [];
    
    
    obj.InputParsing( varargin{:} );
    
end

%% ScatterCalc
%> @brief Calculates the effective (decimated) Hamiltonian of the scattering region. (Obsolete)
function ScatterCalc( obj )
        
    obj.display('Creating Hamiltonian for Scattering region')
    if isempty(obj.CreateH)
        obj.CreateH = CreateHamiltonians(obj.Opt, obj.param);
    end
    obj.CreateH.CreateScatterH();

    if obj.Opt.magnetic_field == 1
        obj.display('Applying magnetic field in scattering region')
        if isempty( obj.PeierlsTransform )
            obj.PeierlsTransform = Peierls(obj.Opt, 'param', obj.param);
        end
        obj.PeierlsTransform.PeierlsTransform(obj.CreateH, 'Hscatter');
    else
        obj.PeierlsTransform = [];
    end

    if ~isempty(obj.param.scatter.Overlap_in_Scatter) && obj.param.scatter.Overlap_in_Scatter && ~(isempty(obj.Opt.type_of_scanning) ) && ~obj.Opt.Just_Create_Hamiltonians
        obj.display('Applying overlap integrals in scattering region')
        obj.ApplyOverlapInScatter( obj.CreateH, obj.E )
    end

    % Deciamtion
    if obj.Opt.Decimation >= 1  && ~obj.Opt.Decimate_only_Dyson && ~obj.Opt.Just_Create_Hamiltonians
        ido = cputime();
        obj.display('Applying decimation procedure in scattering region')
        Decimation_Procedure = Decimation(obj.Opt); 
        Decimation_Procedure.DecimationFunc(obj.E, obj.CreateH, 'Hscatter', 'kulso_szabfokok');
        obj.display(['A decimalasnal eltelt ido:', num2str(cputime()-ido)]);
    else
        obj.display('No decimation calculation is performed in the scattering region')
    end

end

%% LeadCalc
%> @brief Invokes the function SurfaceGreenFunctionCalculator for each lead.
%> @param varargin Optional parameters, see the documenattion for details.
function surf_ret = LeadCalc( obj, varargin )
        
    p = inputParser;
    p.addParamValue('shiftLeads', zeros(length(obj.param.Leads),1));
    p.addParamValue('potential', []);
    p.addParamValue('coordinates_shift', zeros(length(obj.param.Leads),1) );
    p.addParamValue('gauge_field', [] );
    p.addParamValue('createCore', 0);
    p.addParamValue('SelfEnergy',false); % set true to calculate the self energy of the semi-infinite lead
    p.addParamValue('SurfaceGreensFunction', true );% set true to calculate the surface Greens function of the semi-infinite lead
	p.addParamValue('leads', 1:length(obj.param.Leads) );% list of leads to be calculated
    p.addParamValue('leadmodel', []); %function handle for an individual physical model for the contacts
    p.addParamValue('CustomHamiltonian', []);    % NEW  
    p.addParamValue('q', [] ) %transverse momentum
    p.parse(varargin{:});
    
    shiftLeads             = p.Results.shiftLeads;
    potential              = p.Results.potential;
    coordinates_shift      = p.Results.coordinates_shift; % shifts coordinates by "coordinates_shift" times of a lattice vector.
    gauge_field            = p.Results.gauge_field; % if a valid function handle is given, it is used to perform gauge transformation on Lead Hamiltonians
    createCore             = p.Results.createCore;
    SelfEnergy             = p.Results.SelfEnergy;
    SurfaceGreensFunction  = p.Results.SurfaceGreensFunction;
	leads                  = p.Results.leads;
    q                      = p.Results.q;
    CustomHamiltonian      = p.Results.CustomHamiltonian;
    leadmodel              = p.Results.leadmodel;
        
	if isempty(obj.Surface)
	    obj.Surface = cell(length(obj.param.Leads),1);
	end
    obj.display(' ')
    obj.display('Creating surface_Greens_function interfaces:')
    obj.display(' ')
    idx = 1;
    while idx <= length(leads)
        obj.display('------------------------------')
        obj.display(['Lead: ',num2str(leads(idx))]) 
        obj.Surface{leads(idx)} = obj.SurfaceGreenFunctionCalculator(leads(idx), 'shiftLead', shiftLeads(leads(idx)), 'potential', potential, 'coordinates_shift', coordinates_shift(leads(idx)),...
            'gauge_field', gauge_field, 'createCore', createCore, ...
            'SelfEnergy', SelfEnergy, 'SurfaceGreensFunction', SurfaceGreensFunction, 'q', q, 'leadmodel', leadmodel, 'CustomHamiltonian', CustomHamiltonian);
        obj.display(' ')
        idx = idx + 1;
    end

    surf_ret = obj.Surface;

end

%% DysonEq
%> @brief Invokes the function handle of the Dyson equation. 
%> @param varargin Optional parameters, see the documenattion for details.
function Gret = DysonEq( obj, varargin )

    obj.display('------------------------------')

    p = inputParser;
    p.addParamValue('CustomDyson', []);        
    p.parse(varargin{:});

    if ~isempty( p.Results.CustomDyson )
        obj.Dysonfunc = p.Results.CustomDyson;
    end
    
    if ~isempty( obj.Dysonfunc )
        obj.G = obj.Dysonfunc();
    else
        obj.G = [];
    end
    Gret = obj.G;

end


%% Conductance2
%> @brief Conductance calculated by Eq (19) in PRB 73 085414
    function C = Conductance2( obj )
        
               
        Gamma = cell(size(obj.Surface));
        for idx = 1:length(obj.Surface)
            Gamma{idx} = obj.Surface{idx}.Gamma();
        end  
        
        Gamma_left = [Gamma{1}, zeros( size(Gamma{1},1), size(Gamma{2},2));
                      zeros( size(Gamma{2},1), size(Gamma{1},2)+size(Gamma{2},2)) ];
                  
        Gamma_right = [zeros( size(Gamma{1},1), size(Gamma{1},2)+size(Gamma{2},2));
                       zeros(size(Gamma{2},1), size(Gamma{1},2)), Gamma{2}]; 
                   
        for idx = 1:length(obj.Surface)
            Gamma{idx} = [];
        end
        
        % Eq (19) PRB 73 085414
        C =  trace( Gamma_left*obj.G'*Gamma_right*obj.G ) ;
        
        
%***********************************
%         Sigma = cell(size(obj.Surface));
%         for idx = 1:length(obj.Surface)
%             Sigma{idx} = obj.Surface{idx}.Read('Sigma');
%         end  
%         
%         Sigma_left = [Sigma{1}, zeros( size(Sigma{1},1), size(Sigma{2},2));
%                       zeros( size(Sigma{2},1), size(Sigma{1},2)+size(Sigma{2},2)) ];
%                   
%         Sigma_right = [zeros( size(Sigma{1},1), size(Sigma{1},2)+size(Sigma{2},2));
%                        zeros(size(Sigma{2},1), size(Sigma{1},2)), Sigma{2}]; 
%                    
%         for idx = 1:length(obj.Surface)
%             Sigma{idx} = [];
%         end
%         
%         Eq (22) Eur. Phys. J. B 53, 537-549 (2006)
%         current2 = 2*real( trace(-1i*obj.G*Gamma_right*obj.G'*Sigma_left') );
%         
%         current-current2
%*********************************
        

    end

%% Conduktance
%> @brief Calculates the conductance matrix from the scattering matrix
function C = Conduktance( obj )
    obj.display('calculating conductance matrix')

    obj.nyitott_csatornak = obj.nyitott_csatornak;
    LeadsNumber        = length(obj.nyitott_csatornak);
    C = zeros(LeadsNumber, LeadsNumber);
    
    soridx            = 1;
    while soridx <= LeadsNumber
        oszlopidx = 1;
        while oszlopidx <= LeadsNumber      
            s                   = obj.Smatrix((sum(obj.nyitott_csatornak(1:soridx-1))+1:sum(obj.nyitott_csatornak(1:soridx))), (sum(obj.nyitott_csatornak(1:oszlopidx-1))+1:sum(obj.nyitott_csatornak(1:oszlopidx))));
            if soridx == oszlopidx
                C(soridx,oszlopidx) = obj.nyitott_csatornak(soridx) - trace(s'*s);
            else
                C(soridx,oszlopidx) = -trace(s'*s);      
            end
            oszlopidx           = oszlopidx +1;
        end    
        soridx = soridx +1;
    end
    
    obj.Conductance_Matrix = C;


end

%% SmatrixCal
%> @brief Calculates the scattering matrix
function [S, M] = SmatrixCalc( obj )
    obj.display('calculating S-matrix')
    M = obj.GetM();

    
    Normamtx = cell(length(M),1);
    modusmtx_m  = cell(length(M),1);
    modusmtx_p  = cell(length(M),1);
    d_modusmtx_m  = cell(length(M),1);
    gyokvcsop_p = cell(length(M),1);
    gyokvcsop_m = cell(length(M),1);
    expk_p = cell(length(M),1);
    expk_m = cell(length(M),1);
    sort_tolerance = cell(length(M),1);
    evanescent_indexes_p = cell(length(M),1);
    evanescent_indexes_m = cell(length(M),1);
    
    for idx = 1:length(M)
        Normamtx{idx} = obj.Surface{idx}.Read('Normamtx');
        modusmtx_p{idx} = obj.Surface{idx}.Read('modusmtx_p');
        modusmtx_m{idx} = obj.Surface{idx}.Read('modusmtx_m');
        d_modusmtx_m{idx} = obj.Surface{idx}.Read('d_modusmtx_m');
        vcsop_p = obj.Surface{idx}.Read('vcsop_p');
        expk_p{idx}  = obj.Surface{idx}.Read('expk_p');
        gyokvcsop_p{idx} = sqrt(abs(vcsop_p));
        vcsop_m = obj.Surface{idx}.Read('vcsop_m');
        expk_m{idx} = obj.Surface{idx}.Read('expk_m');
        gyokvcsop_m{idx} = sqrt(abs(vcsop_m));
        sort_tolerance{idx} = obj.Surface{idx}.Read('sort_tolerance');
    end
    
    % indexing out evanescent modes
    for idx = 1:length(M)
        tolerance = sort_tolerance{idx};
        evanescent_indexes_p{idx} = logical( abs( abs(expk_p{idx}) - 1 ) > tolerance );
        evanescent_indexes_m{idx} = logical( abs( abs(expk_m{idx}) - 1 ) > tolerance );
    end   

    NormamtxAll    = zeros(sum(M), sum(M));
    Modusmtx_pAll  = zeros(sum(M), sum(M));
    Modusmtx_mAll  = zeros(sum(M), sum(M));
    d_Modusmtx_mAll = zeros(sum(M), sum(M));
    Gyokvcsop_pAll = zeros(sum(M), 1);
    Gyokvcsop_mAll = zeros(sum(M), 1);
    evanescent_indexes_pAll = false(sum(M), 1);
    evanescent_indexes_mAll = false(sum(M), 1);

    for idx = 1:length(M)
        NormamtxAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = Normamtx{idx};
        Modusmtx_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = modusmtx_p{idx};
        Modusmtx_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = modusmtx_m{idx};
        d_Modusmtx_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = d_modusmtx_m{idx};
        Gyokvcsop_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = gyokvcsop_p{idx};
        Gyokvcsop_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = gyokvcsop_m{idx};
        evanescent_indexes_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = evanescent_indexes_p{idx};
        evanescent_indexes_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = evanescent_indexes_m{idx};
    end
    
    GN0 = obj.G(1:sum(M),1:sum(M))*NormamtxAll;
    S   = (d_Modusmtx_mAll*(GN0-eye(sum(M))) )*Modusmtx_pAll;

    %sorting out evanescent modes
    S(:,evanescent_indexes_pAll) = [];
    for idx=1:length(M)
        M(idx) = length(find(~evanescent_indexes_p{idx}));
    end    
    S(evanescent_indexes_mAll,:) = []; 
    
    S = diag( Gyokvcsop_mAll(~evanescent_indexes_mAll) )*S*diag(Gyokvcsop_pAll(~evanescent_indexes_pAll).^(-1));
    
    
        
    
 obj.nyitott_csatornak = M;
 obj.Smatrix = S;

end

%% SurfaceGreenFunctionCalculator
%> @brief Calculates the surface Green's function or the self energy of a Lead.
%> @param idx the identification number of the lead in the attribute Surface.
%> @param varargin Optional parameters, see the documenattion for details.
function Surface_tmp = SurfaceGreenFunctionCalculator( obj, idx, varargin)
    
    p = inputParser;
    p.addParamValue('createCore', 0);
    p.addParamValue('Just_Create_Hamiltonians', 0);
    p.addParamValue('shiftLead', 0);
    p.addParamValue('coordinates_shift', 0);
    p.addParamValue('potential',[]);
    p.addParamValue('Surface_tmp',[]);
    p.addParamValue('gauge_field', [] );% gauge field for performing gauge transformation
    p.addParamValue('SelfEnergy',false); % set true to calculate the self energy of the semi-infinite lead
    p.addParamValue('SurfaceGreensFunction', true );% set true to calculate the surface Greens function of the semi-infinite lead
    p.addParamValue('leadmodel', []); %function handle for an individual physical model for the contacts
    p.addParamValue('CustomHamiltonian', []); 
    p.addParamValue('q', [] ) %transverse momentum
    p.parse(varargin{:});
    createCore            = p.Results.createCore;
    Just_Create_Hamiltonians = p.Results.Just_Create_Hamiltonians;
    shiftLead             = p.Results.shiftLead;
    coordinates_shift     = p.Results.coordinates_shift;
    potential             = p.Results.potential;
    Surface_tmp           = p.Results.Surface_tmp;
    gauge_field           = p.Results.gauge_field; % gauge field for performing gauge transformation
    SelfEnergy            = p.Results.SelfEnergy;
    SurfaceGreensFunction = p.Results.SurfaceGreensFunction;
    q                     = p.Results.q;
    CustomHamiltonian     = p.Results.CustomHamiltonian;
    leadmodel             = p.Results.leadmodel;
    
    
    
    if isempty(idx)
        Surface_tmp = Surface_Green_function(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', idx, ...
                                            'q', q);
                                        
    elseif ~isempty( Surface_tmp )
        % do nothing if Surface_tmp is given
        Surface_tmp.Reset();
                                        
    elseif ~isempty(obj.param.Leads{idx}.Lead_Orientation)     
        Surface_tmp = Surface_Green_function(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', idx,....
                                            'Lead_Orientation', obj.param.Leads{idx}.Lead_Orientation, ...
                                            'q', q);
    else
        Surface_tmp = Surface_Green_function(obj.Opt, obj.param,...
                                            'Hanyadik_Lead', idx, ...
                                            'q', q);
    end
    
    
    if createCore
        return
    end
                                        
    if obj.Opt.Simple_Green_Function
        Surface_tmp.SurfaceGreen_simple(obj.E);       
    elseif ~isempty(leadmodel)
        Surface_tmp = leadmodel( idx, obj.E, 'createCore', createCore, ...
                            'Just_Create_Hamiltonians', Just_Create_Hamiltonians, ...
                            'shiftLead', shiftLead, ...
                            'coordinates_shift', coordinates_shift, ...
                            'potential', potential, ...
                            'Surface_tmp', Surface_tmp, ...
                            'gauge_field', gauge_field, ...
                            'SelfEnergy', SelfEnergy, ...
                            'SurfaceGreensFunction', SurfaceGreensFunction, ...
                            'q', q);
    else
    
        % creating Hailtonians
        obj.display(['Creating Hamiltonians for lead'])
        Surface_tmp.CreateHamiltonians( 'CustomHamiltonian', CustomHamiltonian );
        Surface_tmp.ShiftCoordinates( coordinates_shift );
        if obj.Opt.magnetic_field == 1
            obj.display(['Applying magnetic field in lead'])
            
            % In superconducting lead one must not include nonzero magnetic
            % field.
            % Hamiltonians in transverse computations must remain
            % traslational invariant. In transverse computations a gauge
            % tranformation is performed on each lead-sample interface to
            % cancel the vector potential
            if ~Surface_tmp.isSuperconducting() %&& isempty( Surface_tmp.Read('q') )
                obj.PeierlsTransform.PeierlsTransformLeads(Surface_tmp);
            elseif ~isempty( gauge_field ) %&& isempty( Surface_tmp.Read('q') )
            	obj.PeierlsTransform.gaugeTransformationOnLead( Surface_tmp, gauge_field );
            end
        end
        
        if abs(shiftLead) > 1e-6
            Surface_tmp.ShiftLead( shiftLead );
        end

        if ~isempty(potential) && isempty(q) %In transverse computations no transverse potential can be applied
            coordinates = Surface_tmp.Read('coordinates');
       		if nargin( potential ) == 1
           		potential2apply = potential( coordinates );
		    elseif nargin( potential ) == 2
    	        potential2apply = potential( coordinates, obj.param );
    	    else
    	        error('EQuUs:Ribbon:ApplyTransversePotential', 'To many input arguments in function handle PotInScatter');
    	    end

           	if isfield(coordinates, 'BdG_u')
				if size( potential2apply, 1) == 1 || size( potential2apply, 2) == 1
	                potential2apply(~coordinates.BdG_u) = -potential2apply(~coordinates.BdG_u);
				else
					potential2apply(~coordinates.BdG_u, ~coordinates.BdG_u) = -conj(potential2apply(~coordinates.BdG_u, ~coordinates.BdG_u));
				end
           	end
           	Surface_tmp.AddPotential( potential2apply );
        end    
        
        if Just_Create_Hamiltonians
            return;
        end
                
        
        %Perform decimation if necessary
        obj.display(['Decimating the lead Hamiltonians'])
        if obj.Opt.Decimation == 0
             decimation = 4;
        else
            decimation = obj.Opt.Decimation;
        end
        Decimation_Procedure_Lead = Decimation(obj.Opt, 'ForcedDecimation', decimation);
        Decimation_Procedure_Lead.DecimationFunc(obj.E, Surface_tmp, 'Hamiltonian2Dec', 'kulso_szabfokok');  
        clear Decimation_Procedure_Lead;
        
        % Trukkos sajatertek
        obj.display(['Eigenvalues for lead'])
        Surface_tmp.TrukkosSajatertekek(obj.E);
        
        %Csportsebessegek
        obj.display(['Group velocities for lead'])
        Surface_tmp.Group_Velocity();
               
        
        %Feluleti Green fuggveny
        if SurfaceGreensFunction
            Surface_tmp.SurfaceGreenFunction();
        end
        
        %SelfEnergy
        if SelfEnergy
            Surface_tmp.SelfEnergy();
        end
        
    end
    
    
end

%% setEnergy
%> @brief Sets the energy to be used in the calculations
%> @param Energy The energy value
    function setEnergy( obj, Energy )
       obj.E = Energy; 
    end

%% Read
%> @brief Query for the value of an attribute in the interface.
%> @param MemberName The name of the attribute.
%> @return Returns with the value of the attribute.
    function ret = Read(obj, MemberName)
        if strcmp(MemberName, 'param')
            ret = obj.param;
        elseif strcmp(MemberName, 'Opt')
            ret = obj.Opt;            
         elseif strcmp(MemberName, 'Surface')
            ret = obj.Surface;  
         elseif strcmp(MemberName, 'E')
            ret = obj.E;             
         elseif strcmp(MemberName, 'M')
            ret = obj.GetM();    
         elseif strcmp(MemberName, 'G')
            ret = obj.G;             
        else
            disp(['No member to read with name ',MemberName]);
        end
    end

%--------------------------------------------------------------------------
%     function Write(MemberName, value)
%         if strcmp(MemberName, 'param')
%             param = value;
%             InitializeParameters();
%         elseif strcmp(MemberName, 'Opt')
%             Opt = value;  
%             InitializeParameters();
%         else
%             disp(['No member to write with name ',MemberName]);
%         end
%     end

%% replaceLead
%> @brief Adds/replaces a lead to/in the system.
%> @param Surface_tmp An instance of class Surface_GReen_Function
%> @param idx Identification number of the lead.
    function replaceLead( obj, Surface_tmp, idx )
        obj.Surface{ idx } = Surface_tmp;   
        obj.Opt.NofLeads = length(obj.Surface);
    end

%% LeadSave
%> @brief Saves the Hamiltonians of each lead
    function LeadSave( obj )        
       for idx = 1:length(obj.Surface)
          obj.Surface{idx}.SaveLeads();
       end 
    end

%% CreateClone
%> @brief Creates a clone of the present Surface_Green_function object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )
        
        if isempty( obj.CreateH )
            CreateH_loc = [];
        else
            CreateH_loc = obj.CreateH.CreateClone();
        end
        
        if isempty( obj.PeierlsTransform )
            PeierlsTransform_loc = [];
        else
            PeierlsTransform_loc = obj.PeierlsTransform.CreateClone();
        end
        
        ret = Transport_Interface(obj.E, obj.Opt, obj.param, ...
            'CreateH', CreateH_loc, ...
            'PeierlsTransform', PeierlsTransform_loc );
        
        for idx = 1:length(obj.Surface)
            ret.replaceLead( obj.Surface{idx}.CreateClone(), idx );
        end      
        
    end   

%% GetM
%> @brief Gets the width of the leads
%> @return Returns with an array of the lead widths;
    function M = GetM( obj )
        M = zeros(length(obj.Surface),1);
        for idx = 1:length(obj.Surface)
           M(idx) = obj.Surface{idx}.Read( 'M' );            
        end  
        
%         if ~isempty(M)
%             param.Leads.M = M;
%         end
    end

end % method public


methods ( Access = protected )

%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin )
        
        p = inputParser;
        p.addParamValue('CreateH', []);
        p.addParamValue('PeierlsTransform', []);
        
        p.parse(varargin{:});

        obj.PeierlsTransform = p.Results.PeierlsTransform;
        obj.CreateH = p.Results.CreateH;

    end

end % methods protected

end
