%%    Eotvos Quantum Transport Utilities - IV
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   An object to calculate the I-V curve for a two terminal setup, based on the non-equilibrium Greens function technique Eur. Phys. J. B 53, 537-549 (2006)
%%
classdef IV < handle & CommonFunctions & Messages & FermiDirac %NEW
  
    
    properties (Access = protected)
        %> the chemical potential in the left lead
        mu_left
        %> the chemical potential in the right lead
        mu_right
        %> an instance of class TwoTerminal (or a derived class)
        cTwoTerminal
        %> logical value. Set true (default) for solve the Dyson equation with the self energies, or false otherwise.
        useSelfEnergy
        %> Function handle pot = f(coordinates) for the additional potential to be applied in the scattering region
        scatterPotential   
        %> Height of the contact potential
        ContactPotInterface
        %> logical value. Set true for calculate the Greens function of the scattering region from Hamiltonian, or false (default) otherwise.
        gfininvfromHamiltonian
    end
    
    
methods (Access=public)
        
%% Contructor of the class
%> @brief Constructor of the class.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = IV( varargin )        

        obj.InputParsing( varargin{:});
        
        
    end


    
%% CurrentCalc   
%> @brief Calculates the current-voltage relation as a function of the bias.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return The calculated current and voltage bias arrays and the differential conductance.
    function [current, bias, DifferentialConductance_vec] = IVcalc( obj, varargin )

		p = inputParser;
        p.addParamValue('Edb', 50, @isscalar);
        p.addParamValue('tuned_contact', 'right', @ischar); %which contact is tuned during the calculations: right, left, symmetric
        
        p.parse(varargin{:});
       
        Edb        = p.Results.Edb;  
        tuned_contact = p.Results.tuned_contact;
        
        Evec = obj.CreateEnergyArray('Edb', Edb, 'tuned_contact', tuned_contact);
        
        cTwoTerminal_loc = obj.cTwoTerminal;          
        
        
        
        DifferentialConductance_vec = zeros( size(Evec) );
        
        poolmanager = Parallel( obj.Opt );
        poolmanager.openPool(); 
        
        parfor idx = 1:length(Evec)
            Energy = Evec(idx);              	
            DifferentialConductance_vec(idx) = obj.DifferentialConductanceT0( Energy, cTwoTerminal_loc);
        end
        
        poolmanager.closePool();
        
        if strcmpi( tuned_contact, 'right') || strcmpi( tuned_contact, 'left')
            current = zeros( size(Evec) );
            for idx = 2:length(Evec)
                current(idx) = obj.currentcalc( DifferentialConductance_vec(1:idx), Evec(1:idx), tuned_contact );
            end            
            bias = Evec - Evec(1);
            indexes = (obj.mu_left <= Evec & Evec <= obj.mu_right) | (obj.mu_left >= Evec & Evec >= obj.mu_right);
            bias = bias(indexes);
            current = current(indexes);
            DifferentialConductance_vec = DifferentialConductance_vec(indexes);            
                        
        elseif strcmpi( tuned_contact, 'symmetric' )
            Evec_length = floor(length( Evec )/2);
            current = zeros( Evec_length, 1 );
            for idx = 2:Evec_length
                current(idx) = obj.currentcalc( DifferentialConductance_vec(Evec_length-idx+1: Evec_length+idx-1), Evec(Evec_length-idx+1: Evec_length+idx-1), tuned_contact );
            end
            bias = Evec(Evec_length:end) - Evec(Evec_length);                     
            indexes = 1;
        end 
        
        
		


    end
    
        
    
    
end % public methods



methods (Access=protected)

%> @brief Calculates the current by integrating the differential conductance in a bias window
%> @param differentialConductance Array of differential conductance
%> @param Evec Array of energy values
%> @return Returns with the calculated current
	function current = currentcalc( obj, differentialConductance, Evec, tuned_contact )

		deltaEvec = [0, diff(Evec)];
        
        % defining the Fermi functions for the individual leads
        if strcmpi( tuned_contact, 'right') 
            Fermi_left  = @(E)(obj.Fermi( E-obj.mu_left ));
            Fermi_right = @(E)(obj.Fermi( E-Evec(end) ));
        elseif strcmpi( tuned_contact, 'left') 
            Fermi_left  = @(E)(obj.Fermi( E-Evec(end) ));
            Fermi_right = @(E)(obj.Fermi( E-obj.mu_right ));
        elseif strcmpi( tuned_contact, 'symmetric') 
            Fermi_left  = @(E)(obj.Fermi( E-Evec(end) ));
            Fermi_right = @(E)(obj.Fermi( E-Evec(1) ));            
        else
            current = [];
            return
        end
        
        % including temperature
        differentialConductance = differentialConductance.*(Fermi_right(Evec) - Fermi_left(Evec));
        
        if mod(length( Evec ),2) == 0
            current = obj.SimphsonInt( differentialConductance(1:end-1).*deltaEvec(1:end-1) );
            current = current + (differentialConductance(end-1) + differentialConductance(end) )*deltaEvec(end)/2;
        else
            current = obj.SimphsonInt( differentialConductance.*deltaEvec );            
        end 

	end
    
    
    
%% create_scatter_GreensFunction
%> @brief Calculates the surface Green operator of the scattering region.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return gfininv The inverse of the Green operator.
    function gfininv = create_scatter_GreensFunction(  obj, varargin )
        
        p = inputParser;
        p.addParamValue('onlyAtEnds', false ); 
        p.addParamValue('gauge_trans', true); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians
        p.addParamValue('TwoTerminal', obj.cTwoTerminal); %for parallel computations
        p.parse(varargin{:});       
        onlyAtEnds     = p.Results.onlyAtEnds;
        gauge_trans     = p.Results.gauge_trans;
        TwoTerminal_loc     = p.Results.TwoTerminal;       
        
        if onlyAtEnds
            if obj.gfininvfromHamiltonian
                %calculating the Greens function from the Hamiltonian
                TwoTerminal_loc.CalcFiniteGreensFunctionFromHamiltonian('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface, 'PotInScatter', obj.scatterPotential );
            else
                % Calculating the Greens function by the fast way optimized for translational invariant systems
                TwoTerminal_loc.CalcFiniteGreensFunction('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface);
            end
            [~, gfininv] = TwoTerminal_loc.GetFiniteGreensFunction();
            coordinates_scatter = TwoTerminal_loc.getCoordinates();
        else            
            z1 = 0; % TODO to use function GetíHeight...
            z2 = TwoTerminal_loc.height-1;
            z3 = round( TwoTerminal_loc.height/2 )+1;
            z3_coords = [z3-1, z3];     
            M = TwoTerminal_loc.Surface_tmp.Read('M');
            gfin = GreensFuncAtRibbonEnds_z1z2z3();
            if isnan(rcond(gfin) ) || abs( rcond(gfin) ) < 1e-16
                gfininv = obj.inv_SVD( gfin );
            else
                gfininv = inv(gfin);
            end    
            coordinates_scatter = TwoTerminal_loc.getCoordinates();
            coordinates_tmp = TwoTerminal_loc.Surface_tmp.Read( 'coordinates' );
            indexes = [1+M:3*M,4*M+1:(4+length(z3_coords))*M];
            gfininv = gfininv( indexes, indexes );
            for kdx = 1:length( z3_coords )
                coordinates_scatter.x = [coordinates_scatter.x; coordinates_tmp.x+coordinates_tmp.a(1)*z3_coords(kdx) ];
                coordinates_scatter.y = [coordinates_scatter.y; coordinates_tmp.x+coordinates_tmp.a(2)*z3_coords(kdx) ];
                if isfield( coordinates_tmp, 'BdG_u' )
                    coordinates_scatter.BdG_u = [coordinates_scatter.BdG_u; coordinates_tmp.BdG_u];
                end
            end
        end
        
        if isfield(TwoTerminal_loc, 'Ribbons')
            TwoTerminal_loc = TwoTerminal_loc.Ribbons{1}; %for the CombineRibbons interface
        end
       
        
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans && ~isempty(  TwoTerminal_loc.PeierlsTransform_Scatter )
            gfininv = TwoTerminal_loc.PeierlsTransform_Scatter.gaugeTransformation( gfininv, coordinates_scatter, TwoTerminal_loc.gauge_field );
            %TwoTerminal_loc.PeierlsTransform_Scatter.gaugeTransformationOnLead( TwoTerminal_loc.Surface_tmp, TwoTerminal_loc.gauge_field );  
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2z3()
            
            TwoTerminal_loc.CalcFiniteGreensFunction('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface);
            
            gfin = zeros(4*M+length(z3_coords)*M, 4*M+length(z3_coords)*M);
            gfin(1:4*M,1:4*M) = GreensFuncAtRibbonEnds_z1z2();
            
            
            for idx = 1:length(z3_coords) %oszlop index
                z3_indexes = 4*M+(idx-1)*M + (1:M);
                
                for jdx = 1:length(z3_coords) %sor index
                    
                    if idx == jdx
                        gtmp = gfin(1:M,1:M);
                        gfin( z3_indexes, z3_indexes) = gtmp;
                    elseif z3_coords(idx)-1==z3_coords(jdx)
                        gtmp = gfin(1:M,1+M:2*M);
                        gfin( z3_indexes-M, z3_indexes) = gtmp;                        
                    elseif z3_coords(idx)+1==z3_coords(jdx)
                        gtmp = gfin(1+M:2*M,1:M);
                        gfin( z3_indexes+M, z3_indexes) = gtmp;   
                    else
                        z3_indexes2 = 4*M+(jdx-1)*M + (1:M);
                        gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z3_coords(jdx), z3_coords(idx));
                        gfin(z3_indexes2, z3_indexes)     = gtmp;
                    end                
                end
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1-1, z3_coords(idx));
                gfin(1:M, z3_indexes)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1-1);
                gfin(z3_indexes, 1:M)     = gtmp;    
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1, z3_coords(idx));
                gfin(1+M:2*M, z3_indexes)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1);
                gfin(z3_indexes, 1+M:2*M)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2, z3_coords(idx));
                gfin(1+2*M:3*M, z3_indexes)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2);
                gfin(z3_indexes, 1+2*M:3*M)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2+1, z3_coords(idx));
                gfin(1+3*M:4*M, z3_indexes)     = gtmp;
                
                gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2+1);
                gfin(z3_indexes, 1+3*M:4*M)     = gtmp; 
                
                
            end
            
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2()
            gfin = zeros(4*M,4*M);
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1-1, z1-1);
            gfin(1:M,1:M)              = gtmp;
            gfin(1+M:2*M, 1+M:2*M)     = gtmp;
            gfin(1+2*M:3*M, 1+2*M:3*M) = gtmp;              
            gfin(1+3*M:4*M, 1+3*M:4*M) = gtmp;
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1-1, z1);
            gfin(1:M,1+M:2*M) = gtmp;
            gfin(1+2*M:3*M,1+3*M:4*M) = gtmp;
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1, z1-1);
            gfin(1+M:2*M,1:M) = gtmp;
            gfin(1+3*M:4*M, 1+2*M:3*M) = gtmp;    
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1-1, z2);
            gfin(1:M,1+2*M:3*M) = gtmp; 
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2, z1-1);
            gfin(1+2*M:3*M,1:M) = gtmp; 
                
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1-1, z2+1);
            gfin(1:M,1+3*M:4*M) = gtmp; 
                
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2+1, z1-1);
            gfin(1+3*M:4*M,1:M) = gtmp;   
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1, z2);
            gfin(1+M:2*M,1+2*M:3*M) = gtmp; 
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2, z1);
            gfin(1+2*M:3*M,1+M:2*M) = gtmp; 
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z1, z2+1);
            gfin(1+M:2*M,1+3*M:4*M) = gtmp; 
        
            gtmp = TwoTerminal_loc.Surface_tmp.InfGreenFunction(z2+1, z1);
            gfin(1+3*M:4*M,1+M:2*M) = gtmp;  
            
        end     
        
    end
    
    
%% setEnergy
%> @brief Sets the energy for the calculations
%> @param newE The value of the energy in the same units as the Hamiltonian.
%> @param varargin Optional parameters, see the documenattion for details.
    function SetEnergy( obj, newE, varargin ) 
        p = inputParser;
        p.addParamValue('TwoTerminal', obj.cTwoTerminal);
        p.parse(varargin{:});
        
        cTwoTerminal_loc = p.Results.TwoTerminal;
        cTwoTerminal_loc.setEnergy( newE ) 
    end    
    
    
    
end
    
    
methods (Access=private)

%% DifferentialConductanceT0
%> @brief Calculates the zero temperature differential conductance for a given energy
%> @param Energy The energy value (use the same units as in the Hamiltonian)
%> @param cTwoTerminal_loc An instance of class TwoTerminal or its subclass.
%> @return Returns with the calculated zero temperature differential conductance.
	function ret = DifferentialConductanceT0( obj, Energy, cTwoTerminal_loc )

        	obj.SetEnergy( Energy, 'TwoTerminal', cTwoTerminal_loc );
			cTransport_Interface = cTwoTerminal_loc.FL_handles;
        
        	gfininv = obj.create_scatter_GreensFunction('onlyAtEnds', true, 'TwoTerminal', cTwoTerminal_loc);         
        
        	Dysonfunc = @()(cTwoTerminal_loc.CustomDysonFunc( 'decimate', true, 'gfininv', gfininv, 'constant_channels', false, ...
                'SelfEnergy', obj.useSelfEnergy) );

			cTransport_Interface.DysonEq( 'CustomDyson', Dysonfunc );
       
        	
            ret = cTransport_Interface.Conductance2();

    end
    
    
    
%% CreateEnergyArray
%> @brief Creates the array of energy values for the integration
%> @param Energy The energy value (use the same units as in the Hamiltonian)
%> @param cTwoTerminal_loc An instance of class TwoTerminal or its subclass.
%> @return Returns with the calculated zero temperature differential conductance.
	function Evec = CreateEnergyArray( obj, varargin )

        p = inputParser;
        p.addParamValue('Edb', 50, @isscalar);
        p.addParamValue('tuned_contact', 'right', @ischar); %which contact is tuned during the calculations: right, left, symmetric
        
        p.parse(varargin{:});
       
        Edb        = p.Results.Edb;  
        tuned_contact = p.Results.tuned_contact;
        
        if ~isscalar( obj.T )
            error( 'IV:CreateEnergyArray', 'Temperature should be a scalar.');
        end
        
        if obj.T < obj.T_treshold
        
            if strcmpi( tuned_contact, 'right') 
                Emin = obj.mu_left;
                Emax = obj.mu_right;
                Evec = Emin:(Emax-Emin)/(Edb+1):Emax;
                return
            elseif strcmpi( tuned_contact, 'left')
                Emin = obj.mu_right;
                Emax = obj.mu_left;   
                Evec = Emin:(Emax-Emin)/(Edb+1):Emax;
                return
            elseif strcmpi( tuned_contact, 'symmetric' )
                Emin = obj.mu_left;
                Emax = obj.mu_right;  
                Emean = (Emax+Emin)/2;
                deltaEvec = (Emax-Emin)/Edb;
                Evec = [sort(Emean-deltaEvec:-deltaEvec:Emin, 'ascend'), Emean:deltaEvec:Emax];
                return
            else
                error( 'IV:CreateEnergyArray', 'Undefined contact to be tuned');
            end
            
        else
            tolerance = 1e-5;
            Fermi_left  = @(E)(obj.Fermi( E-obj.mu_left ));
            Fermi_right = @(E)(obj.Fermi( E-obj.mu_right ));
            
            
            if strcmpi( tuned_contact, 'right') || strcmpi( tuned_contact, 'left')
                if obj.mu_left < obj.mu_right
                    Emin = fzero(@(x)(Fermi_left(x)-1+tolerance), obj.mu_left);
                    Emax = fzero(@(x)(Fermi_right(x)-tolerance), obj.mu_right);
                elseif obj.mu_left > obj.mu_right
                    Emin = fzero(@(x)(Fermi_left(x)-tolerance), obj.mu_left);
                    Emax = fzero(@(x)(Fermi_right(x)-1+tolerance), obj.mu_right);    
                else
                    Evec = [];
                    return
                end
                Evec = Emin:(Emax-Emin)/(Edb+1):Emax;
                return
                
            elseif strcmpi( tuned_contact, 'symmetric' )
                if obj.mu_left < obj.mu_right
                    Emin = fzero(@(x)(Fermi_left(x)-1+tolerance), obj.mu_left);
                    Emax = fzero(@(x)(Fermi_right(x)-tolerance), obj.mu_right);
                elseif obj.mu_left > obj.mu_right
                    Emin = fzero(@(x)(Fermi_left(x)-tolerance), obj.mu_left);
                    Emax = fzero(@(x)(Fermi_right(x)-1+tolerance), obj.mu_right);    
                else
                    Evec = [];
                    return
                end
                
                Emean = (Emax+Emin)/2;
                deltaEvec = (Emax-Emin)/Edb;
                Evec = [sort(Emean-deltaEvec:-deltaEvec:Emin, 'ascend'), Emean:deltaEvec:Emax];
            else
                error( 'IV:CreateEnergyArray', 'Undefined contact to be tuned');
            end 
            
        end

	end    
    

       
%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin )
        
        
        p = inputParser;
        p.addParamValue('T', obj.T, @isscalar);
        p.addParamValue('mu_left', obj.mu_left, @isscalar);
        p.addParamValue('mu_right', obj.mu_right, @isscalar);
        p.addParamValue('TwoTerminal', obj.cTwoTerminal);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('scatterPotential', []);
        p.addParamValue('gfininvfromHamiltonian', false);   
        p.addParamValue('useSelfEnergy', true);    
        
        p.parse(varargin{:});
       
        obj.T        = p.Results.T;    
        obj.mu_left  = p.Results.mu_left;
        obj.mu_right = p.Results.mu_right;
        obj.cTwoTerminal = p.Results.TwoTerminal;
        obj.scatterPotential    = p.Results.scatterPotential;
        obj.gfininvfromHamiltonian = p.Results.gfininvfromHamiltonian;       
        obj.useSelfEnergy = p.Results.useSelfEnergy;
        obj.ContactPotInterface = p.Results.ContactPotInterface;
        
        % validate class obj.cTwoTerminal
        if isempty( obj.cTwoTerminal )
            error('EQuUs:Utils:IV:InputParsing', 'Element cTwoTerminal is empty.')
        end
        if ~strcmp( class(obj.cTwoTerminal), 'TwoTerminal' )        
            supClasses = superclasses(obj.cTwoTerminal);
            if sum( strcmp( supClasses, 'TwoTerminal' ) ) == 0
               error('EQuUs:Utils:IV:InputParsing', 'Element cTwoTerminal or its superclass is not of type TwoTerminal.')
            end
        end
        
        obj.setTemperature(obj.T); 
               

    end        
        
        
end % protected methods    
    
end
