%%    Eotvos Quantum Transport Utilities - Ribbon
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for calculations on a ribbon of finite size.
%%
classdef Ribbon < handle & TwoTerminal 
    
    properties ( Access = public )
        %> An instance of "Surface_Greens_Function" object describing the unit cell of the scattering region
        Surface_tmp
        %> function handle for the transverse potential
        transversepotential
        %> width of the scattering region (number of the atomic sites in the cross section)        
        width
        %> height (length) of the scattering region (number of unit cells)        
        height
        %> the shift of the coordinates of the sites (two component vector)
        shift      
        %> atomic distance 
        rCC
    end
    
    
methods ( Access = public )
%% Contructor of the class
%> @brief Constructor of the class.
%> @param Opt An instance of the structure Opt.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = Ribbon( varargin )             
        obj = obj@TwoTerminal();     
        
        
        obj.param = [];        
        obj.Surface_tmp         = [];
        obj.transversepotential = [];      
        obj.width               = [];    
        obj.height              = [];
        obj.shift               = 0;      
        obj.rCC                 = 1.42; %In Angstrom
        
    if ~strcmpi( class(obj), 'Ribbon') 
        % The constructor is called from within a subclass
        return
    end        
        
        obj.InputParsing(varargin{:})                  

        
        if nargin == 0
            %the method was most probably called from a constructor of a
            %subclass
            return
        end
        
        obj.display('Creating ribbon object')
        obj.createShape();
        obj.setParameters();
        obj.setFermiEnergy();
        
        createOutput( obj.filenameOut, obj.Opt, obj.param );
        
        obj.CreateHandles();
        %> create Hamiltonians and coordinates
        obj.CreateRibbon('justHamiltonians', true);
        
    end
    
%% Transport
%> @brief Calculates the transport through the two terminal setup on two dimensional lattices. Use for development pupose only.
%> @param Energy The energy value.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return Conductivity The calculated conductivity.
%> @return aspect_ratio The aspect ratio W/L of the junction.
%> @return Conductance The conductance tensor
%> @return ny Array of the open channel in the leads.
%> @return DeltaC Error of the unitarity.
%> @return S The scattering matrix.
    function [Conductivity,aspect_ratio,Conductance,ny,DeltaC,S] = Transport(obj, Energy, varargin)
        
        p = inputParser;
        p.addParamValue('constant_channels', false);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('FiniteGreensFunctionFromHamiltonian', false);
        p.addParamValue('decimateDyson', true);
        p.addParamValue('PotInScatter', [])
        p.addParamValue('selfEnergy', false)
		p.addParamValue('Smatrix', true) %logcal value to use the S-matrix method to calculate the conductance or not.
        p.parse(varargin{:});
        constant_channels     = p.Results.constant_channels;
        ContactPotInterface   = p.Results.ContactPotInterface;
        FiniteGreensFunctionFromHamiltonian = p.Results.FiniteGreensFunctionFromHamiltonian;
        PotInScatter               = p.Results.PotInScatter;
        decimateDyson          = p.Results.decimateDyson;
        selfEnergy             = p.Results.selfEnergy; %NEW
		Smatrix                = p.Results.Smatrix;
        
        obj.CalcSpectralFunction(Energy, 'constant_channels', constant_channels, 'ContactPotInterface', ContactPotInterface, 'FiniteGreensFunctionFromHamiltonian', FiniteGreensFunctionFromHamiltonian, ...
            'decimateDyson', decimateDyson, 'PotInScatter', PotInScatter, 'SelfEnergy', selfEnergy);

        if strcmp(obj.param.scatter.End_Type, 'A')
            aspect_ratio = (obj.width*3/2)/((obj.height+3)*sqrt(3));
        else
            aspect_ratio = ((obj.width-0.5)*sqrt(3))/((obj.height+3)*3);
        end
        
        if Smatrix
        	[S,ny] = obj.FL_handles.SmatrixCalc();
        
        	norma = norm(S*S'-eye(sum(ny)));
            
        	if norma >= 1e-3
        	    disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        	end
        
        	if ny(1) ~= ny(2)
        	   disp( ['openchannels do not match: ',num2str(ny(1)), ' ', num2str(ny(2))] )
        	end
        

        
        	conductance = obj.FL_handles.Conduktance();
        	conductance = abs([conductance(1,:), conductance(2,:)]);
        	DeltaC = std(conductance);
        
        	C = mean(conductance);
        	Conductivity = C/aspect_ratio;
        	Conductance = C;
        
        	disp( ['aspect ratio = ', num2str(aspect_ratio), ' conductance = ', num2str(C), ' open_channels= ', num2str(ny(1)), ' conductivity = ', num2str(Conductivity)])  

		else      
			Conductance = obj.FL_handles.Conductance2();
			DeltaC = NaN;
			Conductivity = Conductance/aspect_ratio;
			ny = NaN;
			S = [];
			disp( ['aspect ratio = ', num2str(aspect_ratio), ' conductance = ', num2str(Conductance), ' conductivity = ', num2str(Conductivity)])  
		end
        
    end

%% getCoordinates
%> @brief Gets the coordinates of the central region
%> @return coordinates Coordinates of the surface sites of the central region.
%> @return coordinates_interface Coordinates of the interface region.
function [coordinates, coordinates_interface] = getCoordinates( obj )
    try
        coordinates = obj.Surface_tmp.Read('coordinates');
        fnames = fieldnames(coordinates);
        for idx = 1:length( fnames )
            fname = fnames{idx};
            if strcmp( fname, 'a' )
                continue
            elseif strcmp( fname, 'x' )
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname) + (obj.height-1)*coordinates.a(1)];
            elseif strcmp( fname, 'y' )
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname) + (obj.height-1)*coordinates.a(2)];
            else
                coordinates.(fname) = [coordinates.(fname); coordinates.(fname)];
            end
        end
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the ribbon from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end
    
    
    try
        if isempty( obj.Surface_interface )
            coordinates_interface = [];
            return
        end
        
        coordinates_interface = cell( length(obj.Surface_interface), 1);
        for idx = 1:length( obj.Surface_interface )
            coordinates_interface{idx} = obj.Surface_interface{idx}.Read('coordinates');
        end
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the interface region from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end    
        
end

%% ShiftCoordinates
%> @brief Shifts the coordinates of the sites in the ribbon by an integer multiple of the lattice vector. The coordinates of the Leads are automatically adjusted later
%> @param shift An integer value.
    function ShiftCoordinates( obj, shift )  
        obj.Surface_tmp.ShiftCoordinates( shift );
        if ~isempty( obj.Surface_interface )
            for idx = 1:length(obj.Surface_interface)
                obj.Surface_interface{idx}.ShiftCoordinates( shift );
            end
        end
        
        obj.shift = obj.shift + shift;
    end

%% CreateScatter
%> @brief Creates CreateHamiltonians object for storing and manipulate the Hamiltonian of the the scattering region. The created object is stored in attribute CreateH.
    function Surface_tmp = CreateScatter( obj ) 
        Surface_tmp = obj.Surface_tmp.CreateClone();
        Surface_tmp.Reset();
        
        Surface_tmp.CreateHamiltonians( 'toSave', 0);
        Surface_tmp.ShiftCoordinates( obj.shift ); 
        
        %applying transverse potential
        obj.ApplyTransversePotential( Surface_tmp )
       
        
       	% apply magnetic field in scatter
        if ~isempty( obj.PeierlsTransform_Scatter ) && isempty(obj.q) %for finite q vector potential must be parallel to q, and perpendicular to the unit cell vector
           	obj.display('Applying magnetic field in ribbon Hamiltonians')
           	obj.PeierlsTransform_Scatter.PeierlsTransformLeads( Surface_tmp );
        end
        createH = CreateHamiltonians(obj.Opt, obj.param, 'q', obj.q);
        createH.CreateScatterH( 'Surface_tmp', Surface_tmp );          
        
        
        obj.CreateH = createH;    
        
    end

%% setEnergy
%> @brief Sets the energy for the calculations
%> @param Energy The value of the energy in the same units as the Hamiltonian.
    function setEnergy( obj, Energy )
        
        setEnergy@TwoTerminal( obj, Energy );
        
        if ~isempty( obj.Surface_tmp )
            obj.Surface_tmp.Reset();
        end
        
        if ~isempty(obj.Surface_tmp)
            obj.CreateRibbon('justHamiltonians', true);
        end
        
    end    


%% CustomDysonFunc
%> @brief Custom Dyson function for a two terminal arrangement on a two dimensional lattice.
%> @param varargin Optional parameters, see the documenattion for details.
    function Gret = CustomDysonFunc( obj, varargin )
        
    p = inputParser;
    p.addParamValue('gfininv', []);
    p.addParamValue('constant_channels', true);
    p.addParamValue('onlyGinverz', false );
    p.addParamValue('recalculateSurface', [1 2] );
    p.addParamValue('decimate', true );
    p.addParamValue('kulso_szabfokok', []); %The list of sites to be left after the decimation procedure
    p.addParamValue('SelfEnergy', false); %set true to calculate the Dyson equation with the self energy
    p.addParamValue('keep_sites', 'lead'); %NEW identification of sites to be kept (scatter, interface, lead)
    p.parse(varargin{:});
    gfininv     = p.Results.gfininv;
    constant_channels = p.Results.constant_channels;
    onlyGinverz        = p.Results.onlyGinverz;
    recalculateSurface = p.Results.recalculateSurface;  
    decimate           = p.Results.decimate; 
    kulso_szabfokok = p.Results.kulso_szabfokok;
    SelfEnergy         = p.Results.SelfEnergy;
    keep_sites        = p.Results.keep_sites;
    
    if isempty(gfininv)
        if ~isempty( obj.Ginv )
            gfininv = obj.Ginv;
        else
            rcond_G = rcond(obj.G);
            if isnan(rcond_G) || abs( rcond_G ) < 1e-15
                gfininv = obj.inv_SVD( obj.G );
            else
                gfininv = inv(obj.G);
            end
        end       
    end

	if ~isempty(recalculateSurface)
        pot = [];
    
        %> creating interfaces for the Leads
        if constant_channels
            shiftLeads = ones(length(obj.param.Leads),1)*obj.E;
        else
            shiftLeads = ones(length(obj.param.Leads),1)*0;
        end
    
        %> creating objects for Leads
        if isempty( obj.Surface_interface )
            coordinates_shift = [-1, obj.height] + obj.shift;
        else
            coordinates_shift = [-2, obj.height+1] + obj.shift;
        end                    
        Surface = obj.FL_handles.LeadCalc('coordinates_shift', coordinates_shift, 'shiftLeads', shiftLeads, 'potential', obj.transversepotential, ...
            'SelfEnergy', SelfEnergy, 'SurfaceGreensFunction', ~SelfEnergy, 'gauge_field', obj.gauge_field, 'leads', recalculateSurface, 'q', obj.q, ...
            'leadmodel', obj.leadmodel);
	else
		Surface = obj.FL_handles.Read( 'Surface' );
	end
    

    
    %creating matrices for the Dyson Eq
    Ms = obj.FL_handles.Read('M');
    
    
    if SelfEnergy
        Ginverz = CalcGinverzwithSelfEnergy();
    else
        Ginverz = CalcGinverz();
    end
    
    if decimate
        if isempty(kulso_szabfokok)
            if strcmpi(keep_sites, 'scatter')
                kulso_szabfokok = size(Ginverz,1)-size(gfininv,1)+1:size(Ginverz,1);
            elseif strcmpi(keep_sites, 'interface')
                kulso_szabfokok = 1+sum(Ms):size(Ginverz,1)-size(gfininv,1);
            elseif strcmpi(keep_sites, 'lead')
                kulso_szabfokok = 1:sum(Ms);
            else
                kulso_szabfokok = 1:sum(Ms);
            end
        end
        Ginverz = obj.DecimationFunction( kulso_szabfokok, Ginverz);
    end
    
    if onlyGinverz
        Gret = [];
        return
    end
 	
    rcond_Ginverz = rcond(Ginverz);
    if isnan( rcond_Ginverz )
 		err = MException('Ribbon:CustomDysonFunc', 'NaN is the condition number for Ginverz');
        save('Error_Ribbon_CustomDysonFunc.mat');
        throw(err);
    end
    
    if abs( rcond_Ginverz ) < 1e-15
        obj.display( 'Regularizing Ginverz by SVD',1);
        Gret = obj.inv_SVD( Ginverz );
    else
        Gret = inv( Ginverz );
    end  
    
    
        %-------------------------------
        %> calculate the inverse Greens function
        function Ginverz = CalcGinverz()
            
            Gsurfinverz = cell(length(Ms),1);
            H1  = cell(length(Ms),1);
            H1adj = cell(length(Ms),1);
            for idx = 1:length(Ms)
                Gsurfinverz{idx} = Surface{idx}.Read('gsurfinv');%inv(gsurftemp);
                %> H1_interface = H1_lead
                if Surface{idx}.Read('Lead_Orientation') == 1
                    H1{idx}  = Surface{idx}.Read('H1'); %WS.Surface_interface{idx}.Read('H1');
                    H1adj{idx}  = Surface{idx}.Read('H1adj'); %WS.Surface_interface{idx}.Read('H1adj'); 
                elseif Surface{idx}.Read('Lead_Orientation') == -1
                    H1{idx}  = Surface{idx}.Read('H1adj'); %WS.Surface_interface{idx}.Read('H1');
                    H1adj{idx}  = Surface{idx}.Read('H1'); %WS.Surface_interface{idx}.Read('H1adj'); 
                else
                    error('Wrong Lead Orientation');
                end
            end
    
    
            H_interface = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            if ~isempty( obj.Surface_interface )        
                for idx = 1:length( obj.Surface_interface )
                    H_interface{idx} = obj.Surface_interface{idx}.Read('H0');      
                    if ~isempty( obj.q ) && ~obj.Surface_interface{idx}.Read('HamiltoniansDecimated')
                        H1_transverse = obj.Surface_interface{idx}.Read('H1_transverse');
                        H_interface{idx} = H_interface{idx} + H1_transverse*exp(-1i*obj.q) + H1_transverse'*exp(1i*obj.q); 
                    end
                    if obj.Surface_interface{idx}.Read('Lead_Orientation') == 1
                        H1_sc{idx} = obj.Surface_interface{idx}.Read('H1');
                        H1adj_sc{idx} = obj.Surface_interface{idx}.Read('H1adj');
                    elseif obj.Surface_interface{idx}.Read('Lead_Orientation') == -1
                        H1_sc{idx} = obj.Surface_interface{idx}.Read('H1adj');
                        H1adj_sc{idx} = obj.Surface_interface{idx}.Read('H1');
                    else
                        error('Wrong Lead Orientation');
                    end
                end        
            end

    
    
            %> The Dyson Eq
            GsurfinverzAll = zeros(sum(Ms), sum(Ms));
            H1_All = zeros(sum(Ms), sum(Ms));
            H1adj_All = zeros(sum(Ms), sum(Ms));
            if ~isempty( obj.Surface_interface )
                H_interface_All = zeros(sum(Ms), sum(Ms));
                H1_sc_All = zeros(sum(Ms), sum(Ms));
                H1adj_sc_All = zeros(sum(Ms), sum(Ms));
            end
    
            for idx = 1:length(Ms)
                    GsurfinverzAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = Gsurfinverz{idx};
                    Gsurfinverz{idx} = [];

                    if ~isempty( obj.Surface_interface )
                        H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
                        H1{idx} = [];
                        H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
                        H1adj{idx} = [];
                        H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                        H_interface{idx} = [];
                        H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1_sc{idx};%WS.Surface_interface{idx}.Read('H1');
                        H1adj_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};%WS.Surface_interface{idx}.Read('H1adj');
                    else
                        H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
                        H1{idx} = [];
                        H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
                        H1adj{idx} = [];                
                    end
            end
    
            if ~isempty( obj.Surface_interface )
                Ginv_interface = obj.E*eye(size(H_interface_All)) - H_interface_All;
            end
      
  
            if ~isempty(obj.Surface_interface)
                Ginverz = zeros(3*sum(Ms));
                Ginverz(1:sum(Ms),1:sum(Ms)) = GsurfinverzAll;
                Ginverz(1:sum(Ms),sum(Ms)+1:2*sum(Ms)) = -H1_All;
                Ginverz(sum(Ms)+1:2*sum(Ms), 1:sum(Ms)) = -H1adj_All;
                Ginverz(sum(Ms)+1:2*sum(Ms), sum(Ms)+1:2*sum(Ms)) = Ginv_interface;
                Ginverz(sum(Ms)+1:2*sum(Ms),2*sum(Ms)+1:3*sum(Ms)) = -H1_sc_All;
                Ginverz(2*sum(Ms)+1:3*sum(Ms), sum(Ms)+1:2*sum(Ms)) = -H1adj_sc_All;    
                Ginverz(2*sum(Ms)+1:3*sum(Ms), 2*sum(Ms)+1:3*sum(Ms)) = gfininv;
                %H1_All
                %Ginv_interface
            else
                Ginverz = [ GsurfinverzAll, -H1_All; -H1_All', gfininv ]; 
            end
            
        end
        
        
        %-------------------------------
        %> calculate the inverse Greens function with the self energy
        function Ginverz = CalcGinverzwithSelfEnergy()
            SelfEnergy = cell(length(Ms),1);
            for iidx = 1:length(Ms)
                SelfEnergy{iidx} = Surface{iidx}.Read('Sigma');%inv(gsurftemp);
            end
    
    
            H_interface = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            for idx = 1:length( Ms )
                if ~isempty( obj.Surface_interface ) 
                    H_interface{idx} = obj.Surface_interface{idx}.Read('H0');
                    if ~isempty( obj.q ) && ~obj.Surface_interface{idx}.Read('HamiltoniansDecimated')
                        H1_transverse = obj.Surface_interface{idx}.Read('H1_transverse');
                        H_interface{idx} = H_interface{idx} + H1_transverse*exp(-1i*obj.q) + H1_transverse'*exp(1i*obj.q); 
                    end
                    if obj.Surface_interface{idx}.Read('Lead_Orientation') == 1
                        H1_sc{idx} = obj.Surface_interface{idx}.Read('H1');
                        H1adj_sc{idx} = obj.Surface_interface{idx}.Read('H1adj');
                    elseif obj.Surface_interface{idx}.Read('Lead_Orientation') == -1
                        H1_sc{idx} = obj.Surface_interface{idx}.Read('H1adj');
                        H1adj_sc{idx} = obj.Surface_interface{idx}.Read('H1');
                    else
                        error('Wrong Lead Orientation');
                    end
                else
                    H_interface{idx} = Surface{idx}.Read('H0');
                    if ~isempty( obj.q ) && ~Surface.Read('HamiltoniansDecimated')
                        H1_transverse = Surface.Read('H1_transverse');
                        H_interface{idx} = H_interface{idx} + H1_transverse*exp(-1i*obj.q) + H1_transverse'*exp(1i*obj.q); 
                    end
                    if Surface{idx}.Read('Lead_Orientation') == 1
                        H1_sc{idx} = Surface{idx}.Read('H1');
                        H1adj_sc{idx} = Surface{idx}.Read('H1adj');
                    elseif Surface{idx}.Read('Lead_Orientation') == -1
                        H1_sc{idx} = Surface{idx}.Read('H1adj');
                        H1adj_sc{idx} = Surface{idx}.Read('H1');
                    else
                        error('Wrong Lead Orientation');
                    end
                end
                
        
            end  

            %> The Dyson Eq
            SelfEnergyAll = zeros(sum(Ms), sum(Ms));
            H_interface_All = zeros(sum(Ms), sum(Ms));
            H1_sc_All = zeros(sum(Ms), sum(Ms));
            H1adj_sc_All = zeros(sum(Ms), sum(Ms));
    
            for idx = 1:length(Ms)
                    SelfEnergyAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = SelfEnergy{idx};
                    SelfEnergy{idx} = [];

                    H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                    H_interface{idx} = [];
                    H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1_sc{idx};%WS.Surface_interface{idx}.Read('H1');
                    H1_sc{idx} = {};
                    H1adj_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};%WS.Surface_interface{idx}.Read('H1adj');
                    H1adj_sc{idx} = [];
            end
    
            Ginv_interface = obj.E*eye(size(H_interface_All)) - H_interface_All;
            H_interface_All = [];

            Ginverz = zeros(2*sum(Ms));
            Ginverz(1:sum(Ms),1:sum(Ms)) = Ginv_interface - SelfEnergyAll;
            Ginverz(1:sum(Ms),sum(Ms)+1:2*sum(Ms)) = -H1_sc_All;
            Ginverz(sum(Ms)+1:2*sum(Ms), 1:sum(Ms)) = -H1adj_sc_All;    
            Ginverz(sum(Ms)+1:2*sum(Ms), sum(Ms)+1:2*sum(Ms)) = gfininv;    
                
            
            
            
            
        end        
        
    end

%% CalcFiniteGreensFunction
%> @brief Calculates the Green operator of the scattering region by the fast way (see PRB 90, 125428 (2014)).
%> @param varargin Optional parameters, see the documenattion for details. 
    function CalcFiniteGreensFunction( obj, varargin )
    
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians     
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('onlyGinv', false);
        p.parse(varargin{:});
        gauge_trans     = p.Results.gauge_trans;
        ContactPotInterface  = p.Results.ContactPotInterface;
        onlyGinv        = p.Results.onlyGinv;     
        
      	obj.CreateRibbon( 'ContactPotInterface', ContactPotInterface); 

        z1 = 0;
        z2 = obj.height-1;   
        
        obj.G = [];
        obj.Ginv = [];
        
        obj.Surface_tmp.FiniteGreenFunction(z1,z2, 'onlygfininv', true);        
        obj.Ginv = obj.Surface_tmp.Read( 'gfininv' ); 
        
        %> coordinates
        coordinates_scatter = obj.getCoordinates();
           
%disp( Ginv )           
        %> gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans
            try
            	% gauge transformation on Green's function
                if ~isempty(obj.Ginv) && ~isempty(obj.PeierlsTransform_Scatter) && isempty(obj.q) && ~isempty(obj.gauge_field)
                    %> gauge transformation on the inverse Green's function
                    obj.Ginv = obj.PeierlsTransform_Scatter.gaugeTransformation( obj.Ginv, coordinates_scatter, obj.gauge_field );
                end
            catch  errCause  
                err = MException('Ribbon:CalcFiniteGreensFunction', 'Unable to perform gauge transformation');
                err = addCause(err, errCause);
                save('Error_Ribbon_CalcFiniteGreensFunction.mat')
                throw(err);
            end
        end 
        
        if ~onlyGinv
            rcond_Ginv = rcond(obj.Ginv);
            if isnan(rcond_Ginv ) || abs( rcond_Ginv ) < 1e-15
                obj.display( 'Regularizing Ginv by SVD',1);
                obj.G = obj.inv_SVD( obj.Ginv );
            else
                obj.G = inv(obj.Ginv);
            end       
            obj.Ginv = [];
        else
            obj.G = [];
        end
        
    end

%% CalcFiniteGreensFunctionFromHamiltonian
%> @brief Calculates the Green operator of the scattering region from the whole Hamiltonian.
%> @param varargin Optional parameters, see the documenattion for details. 
    function CalcFiniteGreensFunctionFromHamiltonian( obj, varargin )
        
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians     
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('onlyGinv', false);
        p.addParamValue('PotInScatter', [])
        p.parse(varargin{:});
        gauge_trans              = p.Results.gauge_trans;
        ContactPotInterface      = p.Results.ContactPotInterface;
        onlyGinv                 = p.Results.onlyGinv;
        PotInScatter             = p.Results.PotInScatter;

        Surface_tmp = obj.CreateScatter();
        Surface_tmp.ShiftCoordinates(-1);

        CreateH = obj.CreateH.CreateClone();

        Hscatter = CreateH.Read('Hscatter');
        Hscatter_transverse = CreateH.Read('Hscatter_transverse');
        coordinates = CreateH.Read('coordinates');        
                
        H0 = Surface_tmp.Read('H0');
        H1_transverse = Surface_tmp.Read('H1_transverse');
        H1 = Surface_tmp.Read('H1');
        H1adj = Surface_tmp.Read('H1adj');
        coordinates_tmp = Surface_tmp.Read('coordinates');
        kulso_szabfokok_uc = Surface_tmp.Read('kulso_szabfokok');

if size(H0,1) ~= length(kulso_szabfokok_uc)/2
        %coordinates
        names = fieldnames( coordinates);
        for idx = 1:length(names)
            if strcmp(names{idx},'a') || strcmp(names{idx},'b')
                continue
            end
            coordinates.(names{idx}) = [coordinates_tmp.(names{idx}); coordinates.(names{idx})];
        end        
        CreateH.Write('coordinates', coordinates);
        
        %Hscatter
        Hscatter = [ H0, H1, sparse( [],[],[], size(H0,1), size(Hscatter,2)-size(H1,2)) ;
            [H1adj; sparse( [],[],[], size(Hscatter,1)-size(H1adj,1), size(H0,2))], Hscatter];
        CreateH.Write('Hscatter', Hscatter);
        
        %Hscatter_transverse
        if ~isempty( Hscatter_transverse ) && ~CreateH.Read('HamiltoniansDecimated')
            Hscatter_transverse = [ H1_transverse, sparse( [],[],[], size(H1_transverse,1), size(Hscatter_transverse,2)) ;
            sparse( [],[],[], size(Hscatter_transverse,1), size(H1_transverse,2)), Hscatter_transverse];
        end
        
end

        kulso_szabfokok_uc = kulso_szabfokok_uc(1:length(kulso_szabfokok_uc)/2);
        kulso_szabfokok = [kulso_szabfokok_uc, kulso_szabfokok_uc + (obj.height-1)*size(H0,1)];
        CreateH.Write('kulso_szabfokok', kulso_szabfokok );


        if ~isempty(PotInScatter)          
            obj.ApplyPotentialInScatter( CreateH, PotInScatter)  
            Hscatter = CreateH.Read('Hscatter');
            kulso_szabfokok = CreateH.Read('kulso_szabfokok');
        end       

        
        
        %> apply magnetic field in scatter for finite q
        if ~isempty( obj.PeierlsTransform_Scatter ) && ~isempty(obj.q)
           	obj.display('Applying magnetic field in ribbon Hamiltonians')
           	obj.PeierlsTransform_Scatter.PeierlsTransform( CreateH );
        end       
        
        
        q = CreateH.Read('q');
        if ~isempty( q ) && ~CreateH.Read('HamiltoniansDecimated')
            Hscatter = Hscatter + Hscatter_transverse*diag(exp(-1i*q)) + Hscatter_transverse'*diag(exp(1i*q)); 
        end
           
                       
      
        
if size(H0,1) ~= length(kulso_szabfokok_uc)         
        kulso_szabfokok = kulso_szabfokok + size(H0,1);           
        Hscatter( kulso_szabfokok_uc, : ) = [];
        Hscatter( :, kulso_szabfokok_uc ) = [];
        kulso_szabfokok = kulso_szabfokok - length(kulso_szabfokok_uc);
end
        

        Hscatter = (sparse(1:size(Hscatter,1), 1:size(Hscatter,1), obj.E, size(Hscatter,1),size(Hscatter,1))   - Hscatter);  
        indexes = false( size(Hscatter,1), 1);
        indexes(kulso_szabfokok) = true;
        Hscatter = [ Hscatter( ~indexes, ~indexes) , Hscatter(~indexes, indexes);
                     Hscatter(indexes, ~indexes), Hscatter(indexes, indexes)];

        obj.G = obj.partialInv( Hscatter, length(kulso_szabfokok) );

        obj.CreateRibbon( 'ContactPotInterface', ContactPotInterface);
        
        %> gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans
            try
            	% gauge transformation on Green's function
                if ~isempty(obj.G) && ~isempty(obj.PeierlsTransform_Scatter) && isempty(obj.q) && ~isempty(obj.gauge_field)
                    %> gauge transformation on the inverse Green's function
                    obj.G = obj.PeierlsTransform_Scatter.gaugeTransformation( obj.G, coordinates_scatter, obj.gauge_field );
                end
            catch  errCause  
                err = MException('Ribbon:CalcFiniteGreensFunctionFromHamiltonian', 'Unable to perform gauge transformation');
                err = addCause(err, errCause);
                save('Error_Ribbon_CalcFiniteGreensFunctionFromHamiltonian.mat')
                throw(err);
            end
        end
              
        if onlyGinv
            rcond_G = rcond(obj.G);
            if isnan(rcond_G ) || abs( rcond_G ) < 1e-15
                obj.display( 'Regularizing Ginv by SVD',1);
                obj.Ginv = obj.inv_SVD( obj.G );
            else
                obj.Ginv = inv(obj.G);
            end       
            obj.G = [];
        else
            obj.Ginv = [];
        end
        
    end

%% CreateRibbon
%> @brief Creates the Hamiltonians for the ribbon shaped scattering region.
%> @param varargin Optional parameters, see the documenattion for details.
    function CreateRibbon( obj, varargin )
    
        p = inputParser;
        p.addParamValue('justHamiltonians', false); 
        p.addParamValue('ContactPotInterface', []); 
        p.addParamValue('forcedInterface', false); 
        p.parse(varargin{:});
        justHamiltonians     = p.Results.justHamiltonians;
        ContactPotInterface  = p.Results.ContactPotInterface;
        forcedInterface      = p.Results.forcedInterface;
        
        
                
        
        if ~obj.Surface_tmp.Read( 'HamiltoniansCreated' )
        	obj.display( 'Creating ribbon Hamiltonian.' )
        	obj.Surface_tmp.CreateHamiltonians( 'toSave', 0);
        	obj.Surface_tmp.ShiftCoordinates( obj.shift ); 
            
            %applying transverse potential
            obj.ApplyTransversePotential( obj.Surface_tmp )
        end
        
        %coupling between the interface and scattering region
        Surface_sc = cell( size(obj.Surface_interface) );
        for idx = 1:length( obj.Surface_interface )
            if obj.Surface_interface{idx}.Read('HamiltoniansDecimated' )
                continue
            end
            Surface_sc{idx} = obj.createSurface_sc( idx );
        end
       
        
       	% apply magnetic field in scatter
        if ~isempty( obj.PeierlsTransform_Scatter ) && ~obj.Surface_tmp.Read('MagneticFieldApplied')
           	obj.display('Applying magnetic field in ribbon Hamiltonians')
           	obj.PeierlsTransform_Scatter.PeierlsTransformLeads( obj.Surface_tmp ); 
        end
        
        if ~isempty( obj.PeierlsTransform_Leads )
            for idx = 1:length( obj.Surface_interface )
                if obj.Surface_interface{idx}.Read('HamiltoniansDecimated' )
                    continue
                end
                obj.PeierlsTransform_Leads.PeierlsTransformLeads( Surface_sc{idx} );  
            end
        end 
        
        
        %> interface region is needed only when decimation is necessary in the ribbon hamiltonians to get invertable H1
        M = obj.Surface_tmp.Read('M');
        H0_size = size( obj.Surface_tmp.Read('H0'), 1);
        if ( M == H0_size && ~obj.Surface_tmp.Read( 'HamiltoniansDecimated' ) )  && (~forcedInterface) 
            obj.Surface_interface = [];
        end       
        
        
        for idx = 1:length(obj.Surface_interface)
			obj.CreateInterface( idx );
        end
        
         
        
        %WS.Surface_tmp.CalcSpektrum('ka_min', -pi, 'ka_max', pi);
        if justHamiltonians
            return
        end

        for idx = 1:length(obj.Surface_interface)
            if ~obj.Surface_interface{idx}.Read('HamiltoniansDecimated' )
                obj.DecimateInterface( idx, 'ContactPotInterface', ContactPotInterface, 'Surface_sc', Surface_sc{idx} );
            end
        end
        
        if ~obj.Surface_tmp.Read('HamiltoniansDecimated' )
            obj.display( 'Solving Hamiltonians in the scattering region' )        
            Decimation_Procedure_Lead = Decimation(obj.Opt, 'ForcedDecimation', obj.Opt.Decimation);
            Decimation_Procedure_Lead.DecimationFunc(obj.E, obj.Surface_tmp, 'Hamiltonian2Dec', 'kulso_szabfokok'); 
        
            %> Trukkos sajatertek
            obj.display(['Eigenvalues for the scattering region'])
            obj.Surface_tmp.TrukkosSajatertekek(obj.E);
            %> group velocity
            obj.display(['Group velocity for the scattering region'])
            obj.Surface_tmp.Group_Velocity();
        end
	
    end
    

%% CreateInterface
%> @brief Creates the Hamiltonians for the interface regions between the leads and scattering center.
%> @param idx Identification number of the interface region. 
	function CreateInterface( obj, idx )

		coordinates_shift = [-1, obj.height ] + obj.shift;
        if ~obj.Surface_interface{idx}.Read( 'HamiltoniansCreated' )   
        	obj.display(['Creating Hamiltonians for interface region', num2str(idx)])
            obj.Surface_interface{idx}.Write( 'coordinates2', obj.getCoordinates() );
            obj.Surface_interface{idx}.CreateHamiltonians( 'toSave', 0);
            obj.Surface_interface{idx}.ShiftCoordinates( coordinates_shift(idx) );  
            
            %applying transverse potential
            obj.ApplyTransversePotential( obj.Surface_interface{idx} )
        end
        
        
        
        %> apply magnetic field in the interface region
        if ~isempty( obj.PeierlsTransform_Leads )      
            %> In superconducting lead one must not include nonzero magnetic
            %> field.
            %> Hamiltonians in transverse computations must remain
            %> traslational invariant. 
            if ~obj.Surface_interface{idx}.isSuperconducting() %&& isempty( WS.Surface_interface{idx}.Read('q') )
                obj.display(['Applying magnetic field in the Hamiltonians of interface region ', num2str(idx)])
               	obj.PeierlsTransform_Leads.PeierlsTransformLeads( obj.Surface_interface{idx} );  
            else %&& isempty( WS.Surface_interface{idx}.Read('q') )
                obj.display(['Applying gauge transformation in the Hamiltonians of interface region ', num2str(idx)])
               	obj.PeierlsTransform_Leads.gaugeTransformationOnLead( obj.Surface_interface{idx}, obj.gauge_field );
            end
        end  
	end



%% CreateClone
%> @brief Creates a clone of the present object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )
        
        ret = Ribbon( 'width', obj.width, ...
            'height', obj.height, ...
            'filenameIn', obj.filenameIn, ...
            'filenameOut', obj.filenameOut, ...
            'E', obj.E, ...
            'EF', obj.EF, ...
            'phi', obj.phi, ...
            'silent', obj.silent, ...
            'transversepotential', obj.transversepotential, ...
			'Opt', obj.Opt, ...
			'param', obj.param, ...
            'q', obj.q, ...
            'leadmodel', obj.leadmodel );
        
        ret.CreateH = obj.CreateH.CreateClone();
        ret.FL_handles = obj.FL_handles.CreateClone();
        ret.Surface_tmp = obj.Surface_tmp.CreateClone();
        ret.Surface_interface = cell(size(obj.Surface_interface));
        for idx = 1:length(obj.Surface_interface)
            ret.Surface_interface{idx} = obj.Surface_interface{idx}.CreateClone();
        end
        if ~isempty( obj.PeierlsTransform_Scatter )
            ret.PeierlsTransform_Scatter = obj.PeierlsTransform_Scatter.CreateClone();
        end
        
        if ~isempty( obj.PeierlsTransform_Leads )
            ret.PeierlsTransform_Leads   = obj.PeierlsTransform_Leads.CreateClone();    
        end
        ret.gauge_field              = obj.gauge_field;  % function handle for the scalar field to transform the vector potential from Landauy to Landaux
        
    end  



  

end % methods public

methods ( Access = protected )

%% setFermiEnergy
%> @brief Sets the Fermi energy on the atomic sites for the calculations (use the same units as the elements of the Hamiltonian).
    function setFermiEnergy( obj )
        if ~isempty(obj.EF)
            obj.param.scatter.epsilon = obj.param.scatter.epsilon - obj.EF;
            for idx = 1:length(obj.param.Leads)
                obj.param.Leads{idx}.epsilon = obj.param.Leads{idx}.epsilon - obj.EF;
            end
        end      
    end

%% createSurface_sc
%> @param idx The identification number of the interface region. (Integer value.)
%> @brief Creates the copuling Hamiltonians between the scattering and interface region
    function Surface_sc = createSurface_sc( obj, idx )
        Surface_sc = obj.Surface_tmp.CreateClone(); 
        Surface_sc.Reset();
        
        if ~isempty( obj.param.Leads{idx}.vargamma_sc )
            params = Surface_sc.Read( 'params' );
            params.vargamma = obj.param.Leads{idx}.vargamma_sc;
            Surface_sc.Write( 'params', params );

        end
        
        Surface_sc.CreateHamiltonians( 'toSave', 0);            
        if idx == 1
            coordinates_shift = -1 + obj.shift ;
        elseif idx == 2
            coordinates_shift = obj.height + obj.shift;
        end
        Surface_sc.ShiftCoordinates( coordinates_shift )
        Surface_sc.Write('Hanyadik_Lead', idx);
        Surface_sc.Write('Lead_Orientation', obj.Surface_interface{idx}.Read('Lead_Orientation'));
            
        %applying transverse potential
        obj.ApplyTransversePotential( Surface_sc )        
        
    end

%% ApplyTransversePotential
%> @param Surface_tmp An instance of class Surface_Green_function conatining the Hamiltonians.
%> @brief Apply the tranvesre potential in the Hamiltonians
    function ApplyTransversePotential( obj, Surface_tmp )
       if ~isempty(obj.transversepotential) && isempty(obj.q) %In transverse computations no transverse potential can be applied
			coordinates = Surface_tmp.Read('coordinates');
       		if nargin( obj.transversepotential ) == 1
            	potential2apply = obj.transversepotential( coordinates );
	        elseif nargin( obj.transversepotential ) == 2
    	        potential2apply = obj.transversepotential( coordinates, obj.param );
    	    else
    	        error('EQuUs:Ribbon:ApplyTransversePotential', 'To many input arguments in function handle PotInScatter');
    	    end

           	if isfield(coordinates, 'BdG_u')
				if size( potential2apply, 1) == 1 || size( potential2apply, 2) == 1
	                potential2apply(~coordinates.BdG_u) = -potential2apply(~coordinates.BdG_u);
				else
					potential2apply(~coordinates.BdG_u, ~coordinates.BdG_u) = -conj(potential2apply(~coordinates.BdG_u, ~coordinates.BdG_u));
				end
           	end
           	Surface_tmp.AddPotential( potential2apply );
        end 
    end

%% ApplyPotentialInScatter %NEW input for the potential function handle
%> @brief Applies the potential in the scattering region
%> @param CreateH An instance of class CreateHamiltonians containing the Hamiltonian  of the scattering center.
%> @param PotInScatter A function handle for the potential to be applied in the Hamiltonian. Does not need to be translational invariant along the unit cells of the scattering center.
    function ApplyPotentialInScatter( obj, CreateH, PotInScatter )
        coordinates = CreateH.Read('coordinates');        
        if nargin( PotInScatter ) == 1
            potential = PotInScatter( coordinates );
        elseif nargin( PotInScatter ) == 2
            potential = PotInScatter( coordinates, CreateH );
            coordinates = CreateH.Read('coordinates');   % coordinates might be changed in function PotInScatter
        else
            error('Ribbon:ApplyPotentialInScatter', 'To many input arguments in function handle PotInScatter');
        end
        
        Hscatter = CreateH.Read('Hscatter');
        if isvector(potential) && length(potential) == size(Hscatter,1)
            %> in case of diagonal on-site potential
            if isfield(coordinates, 'BdG_u')
                potential(~coordinates.BdG_u) = -potential(~coordinates.BdG_u);
            end
			potential = sparse(1:size(Hscatter,1), 1:size(Hscatter,1), potential, size(Hscatter,1), size(Hscatter,1));
        elseif length(size(potential)) == 2 && norm( size(potential)-size(Hscatter) ) == 0
            if isfield(coordinates, 'BdG_u')
                potential(~coordinates.BdG_u, ~coordinates.BdG_u) = -potential(~coordinates.BdG_u, ~coordinates.BdG_u);
            end
        else
            error('Ribbon:ApplyPotentialInScatter', 'Wrong output of the function handle PotInScatter');
        end
        Hscatter = Hscatter + potential;

        CreateH.Write('Hscatter', Hscatter);

    end



% Creates handles for geometry operations
    function CreateHandles( obj )  
        
        CreateHandles@TwoTerminal( obj )
        
        obj.Surface_tmp = obj.FL_handles.SurfaceGreenFunctionCalculator([], 'createCore', 1, 'q', obj.q);
 
    end

%% calculate_lead_attach_points
%> @brief Determines the lead attach points
    function calculate_lead_attach_points( obj )
        for idx = 1:length(obj.param.Leads)
            obj.param.Leads{idx}.M = obj.param.scatter.shape.width;
        end
    end

%% createShape
%> @brief Creates the geometry data of the ribbon shaped scattering region.
    function createShape( obj )
        
        if ~isempty( obj.width ) && ~isempty( obj.height )
            obj.param.scatter.shape.width = obj.width;
            obj.param.scatter.shape.height = obj.height;
        end
        
        if ~isempty(obj.param.scatter.shape.width)
            obj.width = obj.param.scatter.shape.width;
        else
            err = MException('Ribbon:createShape', 'Shape is not given correctly, width is missing');
            throw(err)
        end
        
        if ~isempty(obj.param.scatter.shape.height)
            obj.height = obj.param.scatter.shape.height;
        else
            err = MException('Ribbon:createShape', 'Shape is not given correctly, height is missing');
            throw(err)
        end
        
        obj.calculate_lead_attach_points();
       
    end
    
end % protected methods
    
    
methods (Access=private)        


%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin )
    
        p = inputParser;
        p.addParamValue('width', obj.width);
        p.addParamValue('height', obj.height);
        p.addParamValue('filenameIn', [pwd, filesep, 'Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd, filesep, 'Basic_Input_running_parameters.xml'], @ischar);
        p.addParamValue('WorkingDir', pwd);
        p.addParamValue('E', obj.E, @isscalar);
        p.addParamValue('EF', obj.EF);
        p.addParamValue('phi', obj.phi);  
        p.addParamValue('silent', obj.silent);   
        p.addParamValue('transversepotential', obj.transversepotential);
        p.addParamValue('leadmodel', obj.leadmodel); %individual physical model for the contacts
        p.addParamValue('Opt', []);
        p.addParamValue('param', []);
        p.addParamValue('q', []);
        
        p.parse(varargin{:});

       
        obj.width        = p.Results.width;
        obj.height       = p.Results.height;
        obj.filenameIn   = p.Results.filenameIn;
        obj.filenameOut  = p.Results.filenameOut;
        obj.WorkingDir   = p.Results.WorkingDir;
        obj.E            = p.Results.E;
        obj.EF           = p.Results.EF;
        obj.phi          = p.Results.phi;
        obj.silent       = p.Results.silent;
        obj.transversepotential    = p.Results.transversepotential; 
        obj.leadmodel    = p.Results.leadmodel;
        obj.q            = p.Results.q;
        obj.Opt             = p.Results.Opt;
		obj.param           = p.Results.param;       
                
		if isempty(obj.Opt) || isempty(obj.param)   
			[ obj.Opt, obj.param ] = parseInput( obj.filenameIn );
        end
        
    end
    
end % methdos private



end
