%%    Eotvos Quantum Transport Utilities - SNSJosephson
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for the calculations of the DC Josephson effect.
%%
classdef SNSJosephson < handle & CommonFunctions & FermiDirac & Messages

    properties
		%> The magnetic field in Tesla
        B = 10e-10; %in Tesla      
        %>an instance of class TwoTerminal (or a derived class)        
        ribbon
        %>An instance of structure BandWidth
        BandWidth
        %>logical value. Set true (default) for solve the Dyson equation with the self energies, or false otherwise.
        useSelfEnergy
        %>Function handle pot = f(coordinates) for the additional potential to be applied in the scattering region
        scatterPotential   
        %>Height of the contact potential
        ContactPotInterface
        %>logical value. Set true for calculate the Greens function of the scattering region from Hamiltonian, or false (default) otherwise.
        gfininvfromHamiltonian
		%> cell array of optional parameters.
        varargin
    end



methods (Access=public)
    
%% Contructor of the class
%> @brief Constructor of the class.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
    function obj = SNSJosephson( varargin )        

        obj.ribbon             = [];
        obj.BandWidth          = [];
        obj.useSelfEnergy      = true;
        obj.ContactPotInterface = [];
        obj.scatterPotential   = [];
        obj.gfininvfromHamiltonian = [];

        obj.varargin = varargin;
        obj.InputParsing();
        
        
    end

%% Transport
%> @brief Calculates the transport of the BdG particles through the two terminal setup. Use for development pupose only.
%> @param Energy The energy value.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return Conductance The conductance tensor
%> @return ny Array of the open channel in the leads.
%> @return DeltaC Error of the unitarity.
%> @return S The scattering matrix.
    function [C,ny,DeltaC] = Transmission( obj, E )   
        
        if exist('E', 'var')
            obj.SetEnergy( E )
        end
        
        
        gfininv = create_scatter_GreensFunction('onlyAtEnds', true, 'gauge_trans', true);
        recalculateSurface = [1 2];
        %Dysonfunc = @()CustomDysonFunc();
        Dysonfunc = @()(obj.ribbon.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', false, 'gfininv', gfininv, 'constant_channels', false, 'SelfEnergy', true));
        obj.ribbon.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        
        
        [S,ny] = obj.ribbon.FL_handles.SmatrixCalc();
        norma = norm(S*S'-eye(sum(ny)));
        if norma >= 1e-3
            display( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        ny
        if ny(1) ~= ny(2)
           display( ['openchannels do not match: ',num2str(ny)] )
        end
        
        
        conductance = obj.ribbon.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(1,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        
        display( ['E = ', num2str(E), ' conductance = ', num2str(C)])
        
    end



%% CurrentCalc_discrete
%> @brief Calculates the Josephson current using the method in PRB 93, 224510 (2016)
%> @param DeltaPhi_vec Array of phase diffrencies between the superconducting contacts.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return currentvec An array of the calculated current corresponding to the phase differencies DeltaPhi_vec
    function currentvec = CurrentCalc_discrete( obj, DeltaPhi_vec, varargin )
        
        p = inputParser;
        p.addParamValue('range', 'ABS' ); % NBS: normal bound states, ABS: andreev bound states, CONT: scattering range, ALL: all contributions at once
		p.addParamValue('Edb', 511 ); % number of energy points on the contour 
        p.addParamValue('DeltaPhi', 0.5*pi );
        p.addParamValue('Evec', [] );
        p.addParamValue('ProximityOn', false ); %New if proxymity effect is taken into account, the current operator must be recalculated for each phase difference
        
        p.parse(varargin{:});       
        range   = p.Results.range;
        Edb      = p.Results.Edb;
        Evec     = p.Results.Evec;
        DeltaPhi = p.Results.DeltaPhi;
        ProximityOn = p.Results.ProximityOn;
        
        DeltaPhi_vec = [0, DeltaPhi_vec];
        DeltaPhi_vec = diff(DeltaPhi_vec);
        
        obj.InputParsing( 'forced_phase', [obj.ribbon.phi(1) obj.ribbon.phi(1)+DeltaPhi_vec(1)])

        if ProximityOn
            obj.gfininvfromHamiltonian = true;
        end
        
        
        phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * DeltaPhi + (pi-DeltaPhi)/2;
        
        if strcmp( range, 'NBS' ) && isempty(Evec)
            obj.display('Calculating current for the normal bound states.', 1)
            obj.getBandWidth();
            Emax = -obj.BandWidth.Lead.Emax;
            Emin = -obj.BandWidth.Scatter.Emax;
%            phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;
            %fact = -fact; %the contour integral is in the opposite direction as for the ABS
        elseif strcmp( range, 'ABS' ) && isempty(Evec)
            obj.display('Calculating current for the Andreev bound states.', 1)
            obj.getBandWidth();
            param = obj.ribbon.FL_handles.Read( 'param' );         
            pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
            Emin = -pair_potential;
            Emax = -0;
            %phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;
        elseif strcmp( range, 'CONT' ) && isempty(Evec)
            obj.display('Calculating current for the scattering states.', 1)
            obj.getBandWidth();
            param = obj.ribbon.FL_handles.Read( 'param' );         
            pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
            Emin = -obj.BandWidth.Lead.Emax;
            Emax = -pair_potential;
            %phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;    
        elseif strcmp( range, 'ALL' ) && isempty(Evec)
            obj.display('Calculating current for all the states.', 1)
            obj.getBandWidth();        
            Emin = min([-obj.BandWidth.Lead.Emax,-obj.BandWidth.Scatter.Emax]);
            Emax = -0;
            %phivec = (0.5 + 0.5*sin( -0.5*pi:pi/(Edb+1):0.5*pi )) * pi;        
        elseif ~isempty(Evec)
            obj.display('Calculating current for custom energy contour.', 1)  
            Emin = min(real(Evec));
            Emax = max(real(Evec));           
        else
            err = MException('SNSJosephson:CurrentCalc_discrete', 'Unrecognized energy range');
            save('Error_SNSJosephson_CurrentCalc_discrete.mat');
            throw(err); 
        end
        
        if Emin >= Emax
            currentvec = zeros(size(DeltaPhi_vec));
            return
        end    
        
        % E -> -E symmetry for both zero and nonzero temperature.
        fact = 2;
               
        
        radius = abs( Emax-Emin) /2;  
        R = radius/sin(DeltaPhi/2);
        if isempty(Evec)
            Evec = R*(cos(phivec) + 1i*sin(phivec)) - radius + Emax - 1i*R*sin((pi-DeltaPhi)/2);
        end
        
       obj.display(['Calculating the current between Emin = ', num2str(Emin),' and Emax = ',num2str(Emax),],1);
%        b = min(1, 0.01/radius);
%        Evec = radius*(cos(phivec) + b*1i*sin(phivec)) - radius + Emax;
		deltaEvec = [0, diff(Evec)];
        currentsurf = complex(zeros( length(Evec), length(obj.T), length(DeltaPhi_vec) ));
        
        poolmanager = Parallel( obj.ribbon.FL_handles.Read('Opt') );
        poolmanager.openPool(); 

        hdisplay = @obj.display;
        hSetEnergy = @obj.SetEnergy;
        hcreateCurrentOperator = @obj.createCurrentOperator;
        hincreasePhaseDifference = @obj.increasePhaseDifference;
        hcomplexCurrent = @complexCurrent;
        
        ribbon_loc = obj.ribbon;%.CreateClone();
        parfor iidx = 1:length(Evec)
            
if isempty(ribbon_loc.q)            
            tic		                
            feval( hdisplay, ['Currently evaluating ', num2str(iidx), ' from ', num2str(length(Evec))], 1);
end
            
            
			recalculateSurface = [1 2];
            feval(hSetEnergy, Evec(iidx), 'ribbon', ribbon_loc );
            
            try
                [current_op, gfininv] = feval(hcreateCurrentOperator, 'ribbon', ribbon_loc);
            catch errCause
                feval(hdisplay, ['Error in SNSJosephson:CurrentCalc_discrete at energy point E = ', num2str(Evec(iidx)), ' caused by:'],1);
                feval(hdisplay, errCause.identifier, 1 );
				feval(hdisplay, errCause.stack(1), 1 );
                feval(hdisplay, 'Giving NaN for the current at the given energy point', 1); 
                currentsurf( iidx, :, : ) = NaN;
            	continue
            end 
            	
            currentvec2int = zeros( length( obj.T ), length( DeltaPhi_vec ) );
            for jjdx = 1:length( DeltaPhi_vec )                
                try 
                	feval(hincreasePhaseDifference, DeltaPhi_vec(jjdx), ribbon_loc );
                    if ProximityOn && jjdx > 1
                         % if proximity effect is taken into account, the current operator must be always recalculated
                         [current_op, gfininv] = feval(hcreateCurrentOperator, 'ribbon', ribbon_loc);
                    end
                	currentvec2int( :, jjdx ) = feval(hcomplexCurrent, ribbon_loc, gfininv, current_op, recalculateSurface);
                    recalculateSurface = []; % second lead is recalculated within increasePhaseDifference
                catch errCause
                	feval(hdisplay, ['Error in SNSJosephson:CurrentCalc_discrete at energy point E = ', num2str(Evec(iidx)), 'and at phase difference ', num2str(DeltaPhi_vec(jjdx)), ' caused by:'], 1);
                	feval(hdisplay, errCause.identifier, 1 );
                    for jjjdx = 1:length(errCause.stack)
                        feval(hdisplay, errCause.stack(jjjdx), 1 );
                    end
                	feval(hdisplay, 'Giving NaN for the calculated current at the given energy point and phase difference.', 1); 
                    currentvec2int( jjdx ) = NaN;
					%throw(errCause)
                end
                
            end 
            currentsurf( iidx, :, : ) = currentvec2int;
            
if isempty(ribbon_loc.q)                  
            toc
end
        end

        poolmanager.closePool();
		
        
		% calculating the contour integrals
        currentvec = cell( size(obj.T) );
       	for kkdx = 1:length( obj.T )
			currentvec{kkdx} = zeros(size(DeltaPhi_vec));		
	        for jjdx = 1:length( DeltaPhi_vec )
				currentvec2integrate = currentsurf(:,kkdx,jjdx);
				% omitting NaNs by substituting interpolates
            	indexes = isnan(currentvec2integrate); 
				indexesall = 1:length(currentvec2integrate);
			        
            	currentvec2integrate(indexes) = interp1(indexesall(~indexes), currentvec2integrate(~indexes), indexesall(indexes), 'spline' );
				current = fact*imag( obj.SimphsonInt( currentvec2integrate.*transpose(deltaEvec) ))/(pi);
				currentvec{kkdx}( jjdx ) = currentvec{kkdx}( jjdx ) + current;
			end
        end	
		if length(obj.T) == 1
			currentvec = currentvec{1};
		end
        
        obj.display('Process finished.', 1)
        
        %reset the system  for a new calculation
        obj.InputParsing();






		%------------------------------------------------
		% calculates current for complex energy
		function current = complexCurrent( ribbon_loc, gfininv, current_op, recalculateSurface )
            
            
            if isfield(ribbon_loc, 'Ribbons')
                ribbon_loc = ribbon_loc.Ribbons{1}; %for the CombineRibbons interface
            end                    
                    		
            
            G = ribbon_loc.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', true, 'gfininv', gfininv, 'constant_channels', false, ...
                'SelfEnergy', obj.useSelfEnergy, 'keep_sites', 'scatter');
            
			G_size = size(G);							
			gfininv_size = size(gfininv);				
			if G_size(1) ~= gfininv_size(1)
				warning( 'wrong size of Greens function' )
				current = 0;
				return
			end

			fE = obj.Fermi(ribbon_loc.E);
	        coordinates = ribbon_loc.getCoordinates();
            scatter_sites = ribbon_loc.CreateH.Read('kulso_szabfokok');
            if ~isempty(scatter_sites) % if the Green operator is calculated from the full Hamiltonian, the coordinates contain all sites.
                BdG_indexes = coordinates.BdG_u(scatter_sites);
            else
                BdG_indexes = coordinates.BdG_u();
            end
            
			current = zeros(length( fE ), 1 );

            for idx = 1:length( fE )
				G_tmp = G;
 				G_tmp( BdG_indexes, BdG_indexes) = G_tmp( BdG_indexes, BdG_indexes)*fE(idx);
				G_tmp( ~BdG_indexes, ~BdG_indexes) = G_tmp( ~BdG_indexes, ~BdG_indexes)*(1-fE(idx));

				current(idx) = trace( current_op*G_tmp );

            end
            
		end
      
    end


%% CurrentCalc_continuum
%> @brief Calculates the Josephson current of the continous scattering states.
%> @param DeltaPhi_vec Array of phase diffrencies between the superconducting contacts.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return currentvec An array of the calculated current corresponding to the phase differencies DeltaPhi_vec
%> @return current_surf Energy resolved Josephson current.
    function [currentvec, current_surf] = CurrentCalc_continuum( obj, DeltaPhi_vec, varargin  )
        obj.display('Calculating current in the continuum range from the scattering matrix.', 1)
        p = inputParser;
		p.addParamValue('Edb', 511 ); % number of energy points on the contour 
        p.parse(varargin{:});       
		Edb     = p.Results.Edb;

        DeltaPhi_vec = [0, DeltaPhi_vec];
        DeltaPhi_vec = diff(DeltaPhi_vec);        
    	currentvec = zeros(size(DeltaPhi_vec));
        
        obj.InputParsing( 'forced_phase', [obj.ribbon.phi(1) obj.ribbon.phi(1)+DeltaPhi_vec(1)])
    	
    	obj.getBandWidth();
        Emax = -obj.BandWidth.Lead.Emin;
        Emin = -obj.BandWidth.Lead.Emax;
        Evec = (0.5+0.5*sin((0:Edb-1)/(Edb-1)*pi - pi/2))*(Emax-Emin) + Emin;
        deltaEvec = [0, diff(Evec) ];
        
        current_surf = zeros(length(DeltaPhi_vec), length(Evec));
        % E -> -E symmetry for both zero and nonzero temperature
        fact = 2;
        
        hCurrentCalc_continuum_forGivenEnergy = @CurrentCalc_continuum_forGivenEnergy;
        ribbon_loc = obj.ribbon;%;.CreateClone();    
        parfor kdx = 1:length( Evec )
            current_surf(:,kdx) = feval(hCurrentCalc_continuum_forGivenEnergy, DeltaPhi_vec, Evec(kdx), ribbon_loc ); 
            obj.display( ['Evaluated ', num2str(kdx), ' from ', num2str(length(Evec))], 1);
        end
        
        for jdx = 1:length( DeltaPhi_vec )
            currentvec2integrate =  current_surf(jdx,:);
            % omitting NaNs by substituting interpolates
        	indexes = isnan(currentvec2integrate); 
			indexesall = 1:length(currentvec2integrate);
			        
            currentvec2integrate(indexes) = interp1(indexesall(~indexes), currentvec2integrate(~indexes), indexesall(indexes), 'spline' );
            current_surf(jdx,:) = currentvec2integrate;
			currentvec(jdx) = currentvec(jdx) + fact*obj.SimphsonInt( currentvec2integrate.*(deltaEvec) )/(2*pi);            
        end
        
        %reset the system  for a new calculation
        obj.InputParsing();
        
        obj.display('Process finished.', 1)
        
        %-----------------------------------------------------------
        function current = CurrentCalc_continuum_forGivenEnergy( DeltaPhi_vec, E, ribbon_loc )
        
        %if abs(abs(E)-0.6) <= 1e-8
        %    current = zeros(size( DeltaPhi_vec ) );
        %    return
        %end
       obj.SetEnergy( E, 'ribbon', ribbon_loc  );  
        
        param = ribbon_loc.FL_handles.Read( 'param' );         
        pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
        
        current = NaN( size( DeltaPhi_vec ) );
        if abs(obj.ribbon.E) <= abs(pair_potential)
    	    return
        end
        
        [current_op,gfininv] = obj.createCurrentOperator('ribbon', ribbon_loc); 
        if isnan(current_op)
            return
        end	 
              
        recalculateSurface = [1 2];
        for idx = 1: length( DeltaPhi_vec )
            % setting phi2 to phi2+delta_phi 
			try   
	            obj.increasePhaseDifference( DeltaPhi_vec(idx)-DeltaPhi_vec(1), ribbon_loc );
 	   	        current_tmp = CurrentCalc_continuum_for_GivenDeltaPhi( ribbon_loc, gfininv, current_op, recalculateSurface) ;
            catch errCause
              	obj.display( 'Error in SNSJosephson:CurrentCalc_continuum_forGivenEnergy caused by:')
                obj.display( errCause.identifier, 1 );
               	obj.display( errCause.stack(1), 1 );
               	obj.display( 'Giving NaN for the calculated current')
               	current_tmp = NaN;
            end
            recalculateSurface = [2];
    	    current(idx) = current_tmp;
        end       
        
            
    
        %-------------------------------------------------
        function current = CurrentCalc_continuum_for_GivenDeltaPhi( ribbon_loc, gfininv, current_op, recalculateSurface )
  
            %Dysonfunc = @()CustomDysonFunc('recalculateSurface', recalculateSurface);
            Dysonfunc = @()ribbon_loc.CustomDysonFunc('recalculateSurface', recalculateSurface, ...
                'decimate', false, 'gfininv', gfininv, 'constant_channels', false, 'SelfEnergy', true);
            
            G = ribbon_loc.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
            
            %[S,ny] = obj.ribbon.FL_handles.SmatrixCalc();
            M = ribbon_loc.FL_handles.Read('M');
            
            % incomming states from left (1) and right (2) 
            Surface = ribbon_loc.FL_handles.Read('Surface');
            vcsop_p = cell(length(Surface),1);
            k_p = cell(length(Surface),1);
            k_m = cell(length(Surface),1);
            modusmtx_p = cell(length(Surface),1);
            Normamtx = cell(length(Surface),1);
            modusmtx_m = cell(length(Surface),1);
            d_modusmtx_m = cell(length(Surface),1);
            for iidx = 1:length(Surface)
                vcsop_p{iidx}    = Surface{iidx}.Read('vcsop_p');
                k_p{iidx}        = Surface{iidx}.Read('k_p');
                k_m{iidx}        = Surface{iidx}.Read('k_m');
                modusmtx_p{iidx} = Surface{iidx}.Read('modusmtx_p');
                Normamtx{iidx}   = Surface{iidx}.Read('Normamtx');
                modusmtx_m{iidx} = Surface{iidx}.Read('modusmtx_m');
                d_modusmtx_m{iidx} = Surface{iidx}.Read('d_modusmtx_m');
            end
          
            %propagating indexes
            tolerance = 1e-6;
            prop_indexes{1} = find(abs(imag(k_p{1})) < tolerance);
            prop_indexes{2} = find(abs(imag(k_p{2})) < tolerance);

            current = 0;
            for cell_index = 1:length(Surface)
                for iidx = 1:length(prop_indexes{cell_index})
                    current_tmp = CurrentFromGivenMode( cell_index, prop_indexes{cell_index}(iidx) );
                    current = current + current_tmp;
                end
            end    
       
            if imag(current) > 1e-6
                warning(['Current is complex: ', num2str(current)])
            end
            current = real(current);
                
        
            %--------------------------------------------------
            % calculates current from given incomming mode
            function current_trans = CurrentFromGivenMode( cell_index, idx )
                tolerance = 1e-6;
                modus_in = zeros(size(G,1), 1);
                indexes = sum(M(1:cell_index-1)) + (1:M(cell_index));
                modus_in(indexes) = Normamtx{cell_index}*modusmtx_p{cell_index}(:,idx)/sqrt(vcsop_p{cell_index}(idx));
            
                modus_out = G*modus_in;
                if isempty( ribbon_loc.Surface_interface )
                    % in case no interface region is included
                    sc_wavefnc = modus_out(sum(M)+1:end);        
                else
                    % with interface region included in the setup
                    sc_wavefnc = modus_out(2*sum(M)+1:end);                   
                end
            
            

            
%               particle current
                current_trans = sc_wavefnc'*current_op*sc_wavefnc;
                return

                %quasiparticle current
                quasi_current_trans = 1/(1i)*( c2'*H1sc'*c1 - c1'*H1sc*c2)
            
                % ******** unitarity test **********
                % coupling Hamiltonian to calculate the current in the incomming lead
                H1 = Surface{cell_index}.Read('H1');     

                modus_in = modusmtx_p{cell_index}(:,idx)/sqrt(vcsop_p{cell_index}(idx));
                quasi_current_in = 1i*modus_in'*(H1*exp(1i*k_p{cell_index}(idx))-H1'*exp(-1i*k_p{cell_index}(idx)))*modus_in;
            
                modus_ref = modus_out(indexes) - modus_in;
                r = d_modusmtx_m{cell_index}*modus_ref;
                quasi_current_ref = 0;
                for iiidx = 1:M(cell_index)
                    if abs(imag(k_m{cell_index}(iiidx))) <= 1e-6
                        modus_ref_tmp = r(iiidx)*modusmtx_m{cell_index}(:,iiidx);
                        quasi_current_ref = quasi_current_ref + 1i*modus_ref_tmp'*(H1*exp(1i*k_m{cell_index}(iiidx))-H1'*exp(-1i*k_m{cell_index}(iiidx)))*modus_ref_tmp;
                    end
                end
                quasi_current_ref = -quasi_current_ref
            
                delta_quasi_current = quasi_current_in - quasi_current_trans - quasi_current_ref;
            	delta_quasi_current
                if abs(delta_quasi_current) > tolerance
                    display(['unitarity test failed with \Delta I = ',num2str(delta_quasi_current)]);
                end 
                % ******** unitarity test **********
            
            end
        
        end
    
        end
    
    end

%% createCurrentOperator
%> @brief Calculates the charge current operator from the inverse of the Greens function of the scattering region.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return current_op The matrix operator of the current operator.
%> @return gfininv The inverse of the Green operator of the scattering region.
	function [current_op, gfininv] = createCurrentOperator( obj, varargin )
        
        p = inputParser;
        p.addParamValue('ribbon', obj.ribbon);
        p.parse(varargin{:});
        
        ribbon_loc = p.Results.ribbon;
		
		gfininv = obj.create_scatter_GreensFunction('onlyAtEnds', true, 'ribbon', ribbon_loc);
        if isnan(gfininv)
			current_op = NaN;
			return
        end 
        
        % gfininv = E - Heff(E) !!!!!
        current_op = 1/1i*(triu(gfininv) - tril(gfininv));
    end


%% getBandWidth
%> @brief Determines the band width of the leads and of the scattering region.
%> @return ret An instance of structure BandWidth containing the bandwidths.
    function ret = getBandWidth( obj )
    	obj.display('Getting Bandwidths', 1);
        
        if ~isempty(obj.BandWidth)
            return
        end
        
        poolmanager = Parallel( obj.ribbon.FL_handles.Read('Opt') );
        poolmanager.openPool();
        
        Surface_1 = obj.ribbon.FL_handles.SurfaceGreenFunctionCalculator(1, 'Just_Create_Hamiltonians', true, 'q', obj.ribbon.q, 'leadmodel', obj.ribbon.leadmodel, ...
            'CustomHamiltonian', obj.ribbon.cCustom_Hamiltonians);
        W = BandWidthCalculator( Surface_1 );
        
        param = obj.ribbon.FL_handles.Read( 'param' );         
        pair_potential = min([abs(param.Leads{1}.pair_potential), abs(param.Leads{2}.pair_potential)]);
        
        obj.BandWidth.Lead = structures('BandWidthLimits');
        obj.BandWidth.Lead.Emin = abs(pair_potential);
        obj.BandWidth.Lead.Emax = W;
        
        
        if strcmp( class(obj.ribbon), 'Ribbon' )
            obj.ribbon.CreateRibbon('justHamiltonians', true);        
            W = BandWidthCalculator( obj.ribbon.Surface_tmp );
        elseif strcmp( class(obj.ribbon), 'TwoTerminal' )
            obj.ribbon.CreateScatter();
            W = 1.3*BandWidthCalculatorMolecule( obj.ribbon.CreateH ); % multiplied by 1.3 just for sure to include normal bound states
        else
            W = 0;
        end
        obj.BandWidth.Scatter = structures('BandWidthLimits');
        obj.BandWidth.Scatter.Emin = 0;
        obj.BandWidth.Scatter.Emax = W;
        
        poolmanager.closePool();

        ret = obj.BandWidth;
         
         %--------------------------------------------
         function W = BandWidthCalculator( Surface_tmp )

         	H0 = Surface_tmp.Read('H0');
            if ~isempty( obj.ribbon.q )
                H1_transverse = Surface_tmp.Read('H1_transverse');
                H0 = H0 + H1_transverse*exp(1i*obj.ribbon.q) + H1_transverse'*exp(-1i*obj.ribbon.q); 
            end
        	H1 = Surface_tmp.Read('H1'); 
        	u = rand(size(H0,1)); 
        	u = u/norm(u);     
            
            hgetMaxEigenvalue = @getMaxEigenvalue;
            hsecular_H = @secular_H;
            k = -pi:2*pi/50:pi;
            arnoldi = zeros( length(k),1);
            parfor idx = 1:length(k)
                Heff = feval(hsecular_H,H0,H1,k(idx));
                arnoldi(idx) = abs(feval(hgetMaxEigenvalue, Heff ));                
            end
            W = max( arnoldi );
            

         	% --------------------------------------
         	% Hamiltonian for the secular equation
         	function H = secular_H(H0,H1,k)     
         		H = H0 + H1*exp(1i*k) + H1'*exp(-1i*k);    
         	end
         
            %--------------------------------------------
         	% getting the maximal eigenvalue with Arnoldi iteration
         	% Numerical Linear Algebra" by Trefethen and Bau. (p250) or
         	% http://en.wikipedia.org/wiki/Arnoldi_iteration
         	function ret = getMaxEigenvalue( Heff )
         		%u = rand(size(H0,1)); 
        		%u = u/norm(u);
         		maxIter = 1e2;
         		tolerance = 1e-5;
         	    ret = 0;
         		lambda = 1;
         		for iidx=1:maxIter
         			u_new = Heff*u;
         			norm_new = norm(u_new);
         			lambda_new = norm_new/norm(u);
         		
         			if abs((lambda - lambda_new)/lambda) < tolerance
         				ret = lambda;
         				return
         			end         		
         			u = u_new/norm_new;
         			lambda = lambda_new;     	
         		end
         	end
         end
         
         
         %--------------------------------------------
         function W = BandWidthCalculatorMolecule( CreateH )
            Hscatter  = CreateH.Read('Hscatter');
            W = abs(eigs(Hscatter,1));
         end         
        
    end
    
    
    end % public methods
    
    
    methods (Access=protected)    




%% create_scatter_GreensFunction
%> @brief Calculates the surface Green operator of the scattering region.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return gfininv The inverse of the Green operator.
    function gfininv = create_scatter_GreensFunction(  obj, varargin )
        
        p = inputParser;
        p.addParamValue('onlyAtEnds', false ); 
        p.addParamValue('gauge_trans', true); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians
        p.addParamValue('ribbon', obj.ribbon); %for parallel computations
        p.parse(varargin{:});       
        onlyAtEnds     = p.Results.onlyAtEnds;
        gauge_trans     = p.Results.gauge_trans;
        ribbon_loc     = p.Results.ribbon;       
        
        if onlyAtEnds
            if obj.gfininvfromHamiltonian
                %calculating the Greens function from the Hamiltonian
                ribbon_loc.CalcFiniteGreensFunctionFromHamiltonian('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface, 'PotInScatter', obj.scatterPotential );
            else
                % Calculating the Greens function by the fast way optimized for translational invariant systems
                ribbon_loc.CalcFiniteGreensFunction('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface);
            end
            [~, gfininv] = ribbon_loc.GetFiniteGreensFunction();
            coordinates_scatter = ribbon_loc.getCoordinates();
        else            
            z1 = 0; % TODO to use function GetíHeight...
            z2 = ribbon_loc.height-1;
            z3 = round( ribbon_loc.height/2 )+1;
            z3_coords = [z3-1, z3];     
            M = ribbon_loc.Surface_tmp.Read('M');
            gfin = GreensFuncAtRibbonEnds_z1z2z3();
            if isnan(rcond(gfin) ) || abs( rcond(gfin) ) < 1e-16
                gfininv = obj.inv_SVD( gfin );
            else
                gfininv = inv(gfin);
            end    
            coordinates_scatter = ribbon_loc.getCoordinates();
            coordinates_tmp = ribbon_loc.Surface_tmp.Read( 'coordinates' );
            indexes = [1+M:3*M,4*M+1:(4+length(z3_coords))*M];
            gfininv = gfininv( indexes, indexes );
            for kdx = 1:length( z3_coords )
                coordinates_scatter.x = [coordinates_scatter.x; coordinates_tmp.x+coordinates_tmp.a(1)*z3_coords(kdx) ];
                coordinates_scatter.y = [coordinates_scatter.y; coordinates_tmp.x+coordinates_tmp.a(2)*z3_coords(kdx) ];
                if isfield( coordinates_tmp, 'BdG_u' )
                    coordinates_scatter.BdG_u = [coordinates_scatter.BdG_u; coordinates_tmp.BdG_u];
                end
            end
        end
        
        if isfield(ribbon_loc, 'Ribbons')
            ribbon_loc = ribbon_loc.Ribbons{1}; %for the CombineRibbons interface
        end
       
        
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans && ~isempty(  ribbon_loc.PeierlsTransform_Scatter )
            gfininv = ribbon_loc.PeierlsTransform_Scatter.gaugeTransformation( gfininv, coordinates_scatter, ribbon_loc.gauge_field );
            %ribbon_loc.PeierlsTransform_Scatter.gaugeTransformationOnLead( ribbon_loc.Surface_tmp, ribbon_loc.gauge_field );  
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2z3()
            
            ribbon_loc.CalcFiniteGreensFunction('onlyGinv', true, 'gauge_trans', false, 'ContactPotInterface', obj.ContactPotInterface);
            
            gfin = zeros(4*M+length(z3_coords)*M, 4*M+length(z3_coords)*M);
            gfin(1:4*M,1:4*M) = GreensFuncAtRibbonEnds_z1z2();
            
            
            for idx = 1:length(z3_coords) %oszlop index
                z3_indexes = 4*M+(idx-1)*M + (1:M);
                
                for jdx = 1:length(z3_coords) %sor index
                    
                    if idx == jdx
                        gtmp = gfin(1:M,1:M);
                        gfin( z3_indexes, z3_indexes) = gtmp;
                    elseif z3_coords(idx)-1==z3_coords(jdx)
                        gtmp = gfin(1:M,1+M:2*M);
                        gfin( z3_indexes-M, z3_indexes) = gtmp;                        
                    elseif z3_coords(idx)+1==z3_coords(jdx)
                        gtmp = gfin(1+M:2*M,1:M);
                        gfin( z3_indexes+M, z3_indexes) = gtmp;   
                    else
                        z3_indexes2 = 4*M+(jdx-1)*M + (1:M);
                        gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(jdx), z3_coords(idx));
                        gfin(z3_indexes2, z3_indexes)     = gtmp;
                    end                
                end
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z3_coords(idx));
                gfin(1:M, z3_indexes)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1-1);
                gfin(z3_indexes, 1:M)     = gtmp;    
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1, z3_coords(idx));
                gfin(1+M:2*M, z3_indexes)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z1);
                gfin(z3_indexes, 1+M:2*M)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2, z3_coords(idx));
                gfin(1+2*M:3*M, z3_indexes)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2);
                gfin(z3_indexes, 1+2*M:3*M)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z3_coords(idx));
                gfin(1+3*M:4*M, z3_indexes)     = gtmp;
                
                gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z3_coords(idx), z2+1);
                gfin(z3_indexes, 1+3*M:4*M)     = gtmp; 
                
                
            end
            
        end
            
        
        %-------------------------------------------------
        function gfin = GreensFuncAtRibbonEnds_z1z2()
            gfin = zeros(4*M,4*M);
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z1-1);
            gfin(1:M,1:M)              = gtmp;
            gfin(1+M:2*M, 1+M:2*M)     = gtmp;
            gfin(1+2*M:3*M, 1+2*M:3*M) = gtmp;              
            gfin(1+3*M:4*M, 1+3*M:4*M) = gtmp;
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z1);
            gfin(1:M,1+M:2*M) = gtmp;
            gfin(1+2*M:3*M,1+3*M:4*M) = gtmp;
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1, z1-1);
            gfin(1+M:2*M,1:M) = gtmp;
            gfin(1+3*M:4*M, 1+2*M:3*M) = gtmp;    
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z2);
            gfin(1:M,1+2*M:3*M) = gtmp; 
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2, z1-1);
            gfin(1+2*M:3*M,1:M) = gtmp; 
                
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1-1, z2+1);
            gfin(1:M,1+3*M:4*M) = gtmp; 
                
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z1-1);
            gfin(1+3*M:4*M,1:M) = gtmp;   
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1, z2);
            gfin(1+M:2*M,1+2*M:3*M) = gtmp; 
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2, z1);
            gfin(1+2*M:3*M,1+M:2*M) = gtmp; 
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z1, z2+1);
            gfin(1+M:2*M,1+3*M:4*M) = gtmp; 
        
            gtmp = ribbon_loc.Surface_tmp.InfGreenFunction(z2+1, z1);
            gfin(1+3*M:4*M,1+M:2*M) = gtmp;  
            
        end     
        
    end




%> @brief Increase the phase difference between the Leads by delta_phi (apply gauge transformation on the second lead)
%> @param delta_phi The phase difference increment between the leads.
%> @param ribbon_loc An instance of class TwoTerminal (or its subclass) containing the data of the contacts.
    function increasePhaseDifference( obj, delta_phi, ribbon_loc )
        
            
        if isfield(ribbon_loc, 'Ribbons')
            ribbon_loc = ribbon_loc.Ribbons{1}; %for the CombineRibbons interface
        end
        
        Surface = ribbon_loc.FL_handles.Read( 'Surface' ); 
        
        if isempty( Surface )
            return
        else
            gaugetrans( Surface{2} ) 
        end    
        ribbon_loc.SyncParam();
        
        % gauge transformation in the interface region
        if isempty( ribbon_loc.Surface_interface )
            return
        else
            gaugetrans( ribbon_loc.Surface_interface{2} ) 
        end               
        
        
        %--------------------------------------
        % function that performs tha gauge transformation on the pair
        % potential
        function gaugetrans( Surface_tmp )
            if abs(delta_phi ) < 1e-16
                return
            end
                        
            coordinates = Surface_tmp.Read('coordinates');
            
            if isempty(coordinates)
                % Hamiltonians are not created yet  
                return
            end
        
            Umtx_diag = zeros(size(coordinates.BdG_u));
            if isempty(Umtx_diag)
                % not a BdG calculation
                warning('Not a BdG calculation. Gauge transformation of the pair potential is skipped');
                return
            end
            
        
            Umtx_diag( coordinates.BdG_u ) = exp(1i*delta_phi/2);
            Umtx_diag( ~coordinates.BdG_u ) = exp(-1i*delta_phi/2);        
            Umtx = diag(Umtx_diag);
        
            names = { 'H0', 'H1', 'H1adj'};
            
            if ~strcmp( class(Surface_tmp), 'InterfaceRegion' )
                if obj.useSelfEnergy 
                    names{end+1} = 'Sigma';
                else
                    names{end+1}  = 'gsurfinv';
                end
            end
        
            for idx = 1:length(names)
                matrix2transform = Surface_tmp.Read( names{idx} );
                if isempty( matrix2transform )
                    continue;
                end
                matrix2transform = Umtx*matrix2transform*Umtx';
                Surface_tmp.Write( names{idx}, matrix2transform );
            end   
            
            params = Surface_tmp.Read( 'params' );
            params.pair_potential = params.pair_potential*exp(1i*delta_phi);
            Surface_tmp.Write( 'params', params );
            
            
            if ~strcmp( class(Surface_tmp), 'InterfaceRegion' )
                return
            end
            
            % Interface region needs further transformation
            coordinates2 = Surface_tmp.Read('coordinates2');
            Umtx_diag2 = zeros(size(coordinates2.BdG_u));
            Umtx_diag2( coordinates2.BdG_u ) = exp(1i*delta_phi/2);
            Umtx_diag2( ~coordinates2.BdG_u ) = exp(-1i*delta_phi/2);        
            Umtx2 = diag(Umtx_diag2);
            
            matrix2transform = Surface_tmp.Read( 'Hcoupling' );
            if ~isempty( matrix2transform )
                matrix2transform = Umtx*matrix2transform*Umtx2';
                Surface_tmp.Write( 'Hcoupling', matrix2transform );
            end
            
            matrix2transform = Surface_tmp.Read( 'Hcouplingadj' );
            if ~isempty( matrix2transform )
                matrix2transform = Umtx2*matrix2transform*Umtx';
                Surface_tmp.Write( 'Hcouplingadj', matrix2transform );
            end            

            
            
        end
        
        
        
    end





%% setEnergy
%> @brief Sets the energy for the calculations
%> @param newE The value of the energy in the same units as the Hamiltonian.
%> @param varargin Optional parameters, see the documenattion for details.
    function SetEnergy( obj, newE, varargin ) 
        p = inputParser;
        p.addParamValue('ribbon', obj.ribbon);
        p.parse(varargin{:});
        
        ribbon_loc = p.Results.ribbon;
        ribbon_loc.setEnergy( newE ) 
    end


    end % protected methods
    
    
    methods (Access=private)    
    

%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin )
        
        p2 = inputParser;
        p2.addParamValue('forced_phase',  []);
        p2.parse(varargin{:});
        forced_phase = p2.Results.forced_phase;
        
        p = inputParser;
        p.addParamValue('width', 1, @isscalar);
        p.addParamValue('height', 8, @isscalar);
        p.addParamValue('E', 0.637, @isscalar);
        p.addParamValue('EF', 0, @isscalar);
        p.addParamValue('phi', [[],[]]);
        p.addParamValue('T', obj.T);
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_zigzag_leads.xml'], @ischar);
        p.addParamValue('silent', 0);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('transversepotential', []);
        p.addParamValue('leadmodel', []); %individual physical model for the contacts
        p.addParamValue('scatterPotential', []);
        p.addParamValue('gfininvfromHamiltonian', false);
        p.addParamValue('ribbon', []);     
        p.addParamValue('q', []);
        
        p.parse(obj.varargin{:});

       
        width               = p.Results.width;
        height              = p.Results.height;
        E                   = p.Results.E;
        EF                  = p.Results.EF;
        if isempty( forced_phase )
            phi = p.Results.phi;
        else
            phi = forced_phase;
        end
        obj.T               = p.Results.T;
        silent              = p.Results.silent;
        filenameIn          = p.Results.filenameIn;
        filenameOut         = p.Results.filenameOut;
        obj.ContactPotInterface = p.Results.ContactPotInterface;
        transversepotential = p.Results.transversepotential;
        leadmodel           = p.Results.leadmodel;
        obj.scatterPotential    = p.Results.scatterPotential;
        obj.gfininvfromHamiltonian = p.Results.gfininvfromHamiltonian;
        q                  = p.Results.q;
        obj.ribbon             = p.Results.ribbon;
        
%         if mod(width,2) == 1 && width ~= 1
%             display('correcting width to be even')
%             width = width + 1;
%         end
        
        
        obj.setTemperature(obj.T);  
        
        if isempty(obj.ribbon)
            obj.ribbon = Ribbon('width', width, 'height', height, 'E', E, 'EF', EF, 'phi', phi, 'silent', silent, 'filenameIn', filenameIn, 'filenameOut', filenameOut, ...
                'q', q, 'transversepotential', transversepotential, 'leadmodel', leadmodel);
        else
            if ~strcmp( class(obj.ribbon), 'TwoTerminal' )        
                supClasses = superclasses(obj.ribbon);
                if sum( strcmp( supClasses, 'TwoTerminal' ) ) == 0
                    error('EQuUs:Utils:SNSJosephson:InputParsing', 'Element cTwoTerminal or its superclass is not of type TwoTerminal.')
                end
            end
        end
        
         obj.ribbon.FL_handles.LeadCalc('createCore', true, 'q', q );
               

    end % private methods
    
    
end % methods
end
