%%    Eotvos Quantum Transport Utilities - TwoTerminal
%    Copyright (C) 2016 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an object for calculations on a two terminal setup.
%%
classdef TwoTerminal < handle & Messages & CommonFunctions

    
    properties (Access = protected )   
        %> An instance of strucutre param
        param
        %> Green operator of the scattering region
        G 
        %> The inverse of the Green operator G
        Ginv
    end
    
    
    properties (Access = public)
        %> An instance of CreateHamiltonians object to manipulate the Hamiltonian of the Scattering region
        CreateH
        %> An instance of Transport_Interface object to calculate the scattering region
        FL_handles
        %> A vector of Surface_Green_Function object to describe the interface region between the leads and the scattering region
        Surface_interface
        %> An instance of Peierls object to describe the peirls substitution in the scattering region
        PeierlsTransform_Scatter
        %> An instance of Peierls object to describe the peirls substitution in the leads
        PeierlsTransform_Leads
        %> function handle for the scalar field to transform the vector potential in the scattering region
        gauge_field
        %> function handle for individual physical model of the leads
        leadmodel
        %> input filename containing the computational parameters
        filenameIn
        %> output filename to export the computational parameters
        filenameOut
        %> The working directory
        WorkingDir
        %> An instance of class Custom_Hamiltonians object to load the Hamiltonians from external source %NEW moved from private elements
        cCustom_Hamiltonians         
        %> Function handle to create custom Hamiltonians
        CustomHamiltoniansHandle
        %> if true, no output messages are print
        silent
        %> Two component array of pair potential phases in the left (1) and right (2) superconductor        
        phi
        %> The energy used in the calculations
        E
        %> The Fermi energy
        EF
        %> The transverse momentum quantum number
        q
        %> Planck contant
        h = 6.626e-34;
        %> The charge of the electron
        qe = 1.602e-19;
    end


methods ( Access = public )
    
    
%% Contructor of the class
%> @brief Constructor of the class.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return An instance of the class
function obj = TwoTerminal( varargin )
    obj = obj@Messages();
    
    obj.G = []; % Greens function of the scattering region
    obj.Ginv = []; % inverse of G
    obj.param = [];
    
    obj.CreateH                  = [];
    obj.FL_handles               = [];
    obj.Surface_interface        = [];
    obj.PeierlsTransform_Scatter = [];
    obj.PeierlsTransform_Leads   = [];
    obj.gauge_field              = [];  
    obj.leadmodel                = []; 
    obj.filenameIn = [];
    obj.filenameOut = [];
    obj.WorkingDir = [];
    obj.silent     = false;        
    obj.phi = [[],[]]; 
    obj.E = 0;
    obj.EF = [];
    obj.q  = [];
    
    if ~strcmpi( class(obj), 'TwoTerminal') 
        % The constructor is called from within a subclass
        return
    end
    
    obj.InputParsing( varargin{:} );
    
    obj.cCustom_Hamiltonians = [];

    obj.display('Creating TwoTerminal interface')


    obj.setParameters();
    
    createOutput( obj.filenameOut, obj.Opt, obj.param );            
            
    obj.CreateHandles();
    obj.CreateTwoTerminalHamiltonians();

end




%% Transport
%> @brief Calculates the transport through the two terminal setup. Use for development pupose only.
%> @param Energy The energy value.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return Conductance The conductance tensor
%> @return ny Array of the open channel in the leads.
%> @return DeltaC Error of the unitarity.
%> @return S The scattering matrix.
    function [Conductance,ny,DeltaC,S] = Transport(obj, Energy, varargin)
        
        p = inputParser;
        p.addParamValue('constant_channels', false);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('FiniteGreensFunctionFromHamiltonian', false);
        p.addParamValue('decimateDyson', true);
        p.addParamValue('PotInScatter', [])
        p.addParamValue('selfEnergy', false)
        p.parse(varargin{:});
        constant_channels     = p.Results.constant_channels;
        ContactPotInterface   = p.Results.ContactPotInterface;
        FiniteGreensFunctionFromHamiltonian = p.Results.FiniteGreensFunctionFromHamiltonian;
        PotInScatter               = p.Results.PotInScatter;
        decimateDyson          = p.Results.decimateDyson;
        selfEnergy             = p.Results.selfEnergy; 
        
        obj.CalcSpectralFunction(Energy, 'constant_channels', constant_channels, 'ContactPotInterface', ContactPotInterface, 'FiniteGreensFunctionFromHamiltonian', FiniteGreensFunctionFromHamiltonian, ...
            'decimateDyson', decimateDyson, 'PotInScatter', PotInScatter, 'SelfEnergy', selfEnergy );
        
        
        [S,ny] = obj.FL_handles.SmatrixCalc();
        
        norma = norm(S*S'-eye(sum(ny)));
            
        if norma >= 1e-3
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if ny(1) ~= ny(2)
           disp( ['openchannels do not match: ',num2str(ny(1)), ' ', num2str(ny(2))] )
        end
        
        
        conductance = obj.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(2,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        Conductance = C;
        
        disp( [' conductance = ', num2str(C), ' open_channels= ', num2str(ny(1))])        
        %2/pi
        
    end


%% CalcSpectralFunction
%> @brief Calculates the spectral density and the Green operator.
%> @param Energy The energy value.
%> @param varargin Optional parameters, see the documenattion for details.
%> @return A The spectral density.
%> @return G The Green operator.
    function [A,G] = CalcSpectralFunction(obj, Energy, varargin)
        obj.setEnergy( Energy )
        
        p = inputParser;
        p.addParamValue('constant_channels', false);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('FiniteGreensFunctionFromHamiltonian', false);
        p.addParamValue('decimateDyson', true);
        p.addParamValue('PotInScatter', []);
        p.addParamValue('selfEnergy', false)
        p.parse(varargin{:});
        constant_channels     = p.Results.constant_channels;
        ContactPotInterface   = p.Results.ContactPotInterface;
        FiniteGreensFunctionFromHamiltonian = p.Results.FiniteGreensFunctionFromHamiltonian;
        PotInScatter           = p.Results.PotInScatter;
        decimateDyson          = p.Results.decimateDyson;
        selfEnergy             = p.Results.selfEnergy; %NEW
        
        if ~FiniteGreensFunctionFromHamiltonian
            obj.CalcFiniteGreensFunction('gauge_trans', true, 'ContactPotInterface', ContactPotInterface, 'onlyGinv', true); 
        else
            obj.CalcFiniteGreensFunctionFromHamiltonian('gauge_trans', true, 'ContactPotInterface', ContactPotInterface, 'PotInScatter', PotInScatter, 'onlyGinv', true); 
        end
        
        Dysonfunc = @()(obj.CustomDysonFunc( 'constant_channels', constant_channels, 'decimate', decimateDyson, 'SelfEnergy', selfEnergy ));
        try
            G = obj.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        catch errCause
            err = MException([class(obj),':CalcSpectralFunction'], 'Error occured in calculating the Greens function');
            err = addCause(err, errCause);
            save(['Error_', class(obj), '_CalcSpectralFunction.mat']);
            obj.display(errCause.identifier, 1 );
		    obj.display( errCause.stack(1), 1 );            
            throw( errCause );
        end
        
        A = -imag( trace(G))/pi;
        
        
    end

%% getCoordinates
%> @brief Gets the coordinates of the central region
%> @return coordinates Coordinates of the central region.
%> @return coordinates_interface Coordinates of the interface region.
function [coordinates, coordinates_interface] = getCoordinates( obj )
    try
        coordinates = obj.CreateH.Read('coordinates');
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the ribbon from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end
    
    
    try
        if isempty( obj.Surface_interface )
            coordinates_interface = [];
            return
        end
        
        coordinates_interface = cell( length(obj.Surface_interface), 1);
        for idx = 1:length( obj.Surface_interface )
            coordinates_interface{idx} = obj.Surface_interface{idx}.Read('coordinates');
        end
        
    catch errCause
        err = MException('Ribbon:getCoordinatesFromRibbon', 'Error occured when retriving the coordinates of the interface region from a Ribbon interface');
        err = addCause(err, errCause);
        save('Error_Ribbon_getCoordinatesFromRibbon.mat');
        throw( err );
    end    
        
end


%% CreateScatter
%> @brief Creates CreateHamiltonians object for storing and manipulate the Hamiltonian of the the scattering region. The created object is stored in attribute CreateH.
    function CreateH = CreateScatter( obj ) 
        
        if ~strcmp( class(obj.CreateH), 'CreateHamiltonians')
            CreateH = CreateHamiltonians(obj.Opt, obj.param, 'q', obj.q);
            obj.CreateH = CreateH; 
        end
        
        if ~(obj.CreateH.Read('HamiltoniansCreated') && (~obj.CreateH.Read('HamiltoniansDecimated')))            
            obj.CreateH.CreateScatterH( 'CustomHamiltonian', obj.cCustom_Hamiltonians );    
        end
        

           
        
        % apply magnetic field in scatter
        if ~isempty( obj.PeierlsTransform_Scatter ) && isempty(obj.q) %for finite q vector potential must be parallel to q, and perpendicular to the unit cell vector
           	obj.display('Applying magnetic field in the scattering region')
           	obj.PeierlsTransform_Scatter.PeierlsTransfor( obj.CreateH );
        end     
        
    end

%% setEnergy
%> @brief Sets the energy for the calculations
%> @param Energy The value of the energy in the same units as the Hamiltonian.
    function setEnergy( obj, Energy )
        obj.E = Energy;

        if ~isempty( obj.FL_handles ) && strcmp(class(obj.FL_handles), 'Transport_Interface')        
            obj.FL_handles.setEnergy( obj.E ); 
        end
        
        if ~isempty( obj.CreateH )
            obj.CreateH.Reset();
        end        
         
        
        for idx = 1:length(obj.Surface_interface)
        	obj.Surface_interface{idx}.Reset(); 
        end  
        
        if ~isempty( obj.CreateH )  && strcmpi( class(obj), 'TwoTerminal' )      
            obj.CreateTwoTerminalHamiltonians();
        end
        
    end





%% CustomDysonFunc
%> @brief Custom Dyson function for a general two terminal arrangement.
%> @param varargin Optional parameters, see the documenattion for details.
    function Gret = CustomDysonFunc( obj, varargin )
        
    p = inputParser;
    p.addParamValue('gfininv', []);
    p.addParamValue('constant_channels', true);
    p.addParamValue('onlyGinverz', false );
    p.addParamValue('recalculateSurface', [1 2] );
    p.addParamValue('decimate', true );
    p.addParamValue('kulso_szabfokok', []); %The list of sites to be left after the decimation procedure
    p.addParamValue('SelfEnergy', false); %set true to calculate the Dyson equation with the self energy
    p.addParamValue('keep_sites', 'lead'); %NEW identification of sites to be kept (scatter, interface, lead)
    p.parse(varargin{:});
    gfininv     = p.Results.gfininv;
    constant_channels = p.Results.constant_channels;
    onlyGinverz        = p.Results.onlyGinverz;
    recalculateSurface = p.Results.recalculateSurface;  
    decimate           = p.Results.decimate; 
    kulso_szabfokok = p.Results.kulso_szabfokok;
    SelfEnergy         = p.Results.SelfEnergy;
    keep_sites        = p.Results.keep_sites;
    
    if isempty(gfininv)
        if ~isempty( obj.Ginv )
            gfininv = obj.Ginv;
        else
            rcond_G = rcond(obj.G);
            if isnan(rcond_G) || abs( rcond_G ) < 1e-15
                gfininv = obj.inv_SVD( obj.G );
            else
                gfininv = inv(obj.G);
            end
        end       
    end

	if ~isempty(recalculateSurface)
    
        % creating interfaces for the Leads
        if constant_channels
            shiftLeads = ones(length(obj.param.Leads),1)*obj.E;
        else
            shiftLeads = ones(length(obj.param.Leads),1)*0;
        end

        % creating objects for Leads
        Surface = obj.FL_handles.LeadCalc('shiftLeads', shiftLeads, ...
            'SelfEnergy', SelfEnergy, 'SurfaceGreensFunction', ~SelfEnergy, 'gauge_field', obj.gauge_field, 'leads', recalculateSurface, 'q', obj.q, ...
            'leadmodel', obj.leadmodel, 'CustomHamiltonian', obj.cCustom_Hamiltonians);
	else
		Surface = obj.FL_handles.Read( 'Surface' );
	end
    

    %creating matrices for the Dyson Eq
    Ms = obj.FL_handles.Read('M');

    
    if SelfEnergy
        Ginverz = CalcGinverzwithSelfEnergy();
    else
        Ginverz = CalcGinverz();
    end
    
    if decimate
        if isempty(kulso_szabfokok)
            if strcmpi(keep_sites, 'scatter')
                kulso_szabfokok = size(Ginverz,1)-size(gfininv,1)+1:size(Ginverz,1);
            elseif strcmpi(keep_sites, 'interface')
                kulso_szabfokok = 1+sum(Ms):size(Ginverz,1)-size(gfininv,1);
            elseif strcmpi(keep_sites, 'lead')
                kulso_szabfokok = 1:sum(Ms);
            else
                kulso_szabfokok = 1:sum(Ms);
            end
        end
        Ginverz = obj.DecimationFunction( kulso_szabfokok, Ginverz);
    end
    
    
    
    if onlyGinverz
        Gret = [];
        return
    end
 	
    rcond_Ginverz = rcond(Ginverz);
    if isnan( rcond_Ginverz )
 		err = MException('Ribbon:CustomDysonFunc', 'NaN is the condition number for Ginverz');
        save('Error_Ribbon_CustomDysonFunc.mat');
        throw(err);
    end
    
    if abs( rcond_Ginverz ) < 1e-15
        obj.display( 'Regularizing Ginverz by SVD',1);
        Gret = obj.inv_SVD( Ginverz );
    else
        Gret = inv( Ginverz );
    end  
    
        %-------------------------------
        % calculate the inverse Greens function
        function Ginverz = CalcGinverz()
            
            Gsurfinverz = cell(length(Ms),1);
            H1  = cell(length(Ms),1);
            H1adj = cell(length(Ms),1);
            H_interface = cell(length(Ms),1);
            S_interface = cell(length(Ms),1);
            S1_sc = cell(length(Ms),1);
            S1adj_sc = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            for idx = 1:length(Ms)
                Gsurfinverz{idx} = Surface{idx}.Read('gsurfinv');
                if Surface{idx}.Read('Lead_Orientation') == 1
                    H1{idx}       = Surface{idx}.Read('H1');
                    H1adj{idx}    = Surface{idx}.Read('H1adj');
                elseif Surface{idx}.Read('Lead_Orientation') == -1
                    H1{idx}       = Surface{idx}.Read('H1adj');
                    H1adj{idx}    = Surface{idx}.Read('H1');
                else
                    error('Wrong Lead Orientation');
                end
                H_interface{idx} = obj.Surface_interface{idx}.Read('H0');      
                if ~isempty( obj.q ) && ~obj.Surface_interface{idx}.Read('HamiltoniansDecimated')
                    H1_transverse = obj.Surface_interface{idx}.Read('H1_transverse');
                    H_interface{idx} = H_interface{idx} + H1_transverse*exp(-1i*obj.q) + H1_transverse'*exp(1i*obj.q); 
                end
                S_interface{idx} = obj.Surface_interface{idx}.Read('S0');  
                H1_sc{idx} = obj.Surface_interface{idx}.Read('Hcoupling');
                H1adj_sc{idx} = obj.Surface_interface{idx}.Read('Hcouplingadj');
                S1_sc{idx} = obj.Surface_interface{idx}.Read('Scoupling');
                S1adj_sc{idx} = obj.Surface_interface{idx}.Read('Scoupling')';                
            end
            
    
   
            % The Dyson Eq
            GsurfinverzAll = zeros(sum(Ms), sum(Ms));
            H1_sc_All = zeros(sum(Ms), size(gfininv,2) );
            H1adj_sc_All = zeros(size(gfininv,1), sum(Ms));
            if ~isempty( obj.Surface_interface )
                H_interface_All = zeros(sum(Ms), sum(Ms));
                S_interface_All = zeros(sum(Ms), sum(Ms));
                H1_All = zeros(sum(Ms), sum(Ms));
                H1adj_All = zeros(sum(Ms), sum(Ms));
            end
    
            for idx = 1:length(Ms)
                GsurfinverzAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = Gsurfinverz{idx};
                Gsurfinverz{idx} = [];
                if isempty( S1_sc{idx} )
                    H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), : ) = H1_sc{idx};
                    H1_sc{idx} = [];
                    H1adj_sc_All( :, sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};
                    H1adj_sc{idx} = [];
                else
                    H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), : ) = H1_sc{idx} - obj.E*S1_sc{idx};
                    H1_sc{idx} = [];
                    S1_sc{idx} = [];
                    H1adj_sc_All( :, sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx} - obj.E*S1adj_sc{idx};
                    H1adj_sc{idx} = [];  
                    S1adj_sc{idx} = [];
                end
                H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                H_interface{idx} = [];
                if ~isempty(S_interface{idx})
                    S_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = S_interface{idx};
                    S_interface{idx} = [];
                else
                    S_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = speye(Ms(idx));
                end
                H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
                H1{idx} = [];
                H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
                H1adj{idx} = [];                
            end
            
            Ginv_interface = obj.E*S_interface_All - H_interface_All;
      
            sumM = sum(Ms);
            Ginverz = zeros(2*sumM+size(gfininv,1));
            Ginverz(1:sumM,1:sumM) = GsurfinverzAll;
            Ginverz(1:sumM,sumM+1:2*sumM) = -H1_All;
            Ginverz(sumM+1:2*sumM, 1:sumM) = -H1adj_All;
            Ginverz(sumM+1:2*sumM, sumM+1:2*sumM) = Ginv_interface;
            Ginverz(sumM+1:2*sumM,2*sumM+1:end) = -H1_sc_All;
            Ginverz(2*sumM+1:end, sumM+1:2*sumM) = -H1adj_sc_All;    
            Ginverz(2*sumM+1:end, 2*sumM+1:end) = gfininv;
        end
        
        
        %-------------------------------
        % calculate the inverse Greens function with the self energy
        function Ginverz = CalcGinverzwithSelfEnergy()
            SelfEnergy = cell(length(Ms),1);
            for iidx = 1:length(Ms)
                SelfEnergy{iidx} = Surface{iidx}.Read('Sigma');
            end
    
    
            H_interface = cell(length(Ms),1);
            S_interface = cell(length(Ms),1);
            S1_sc = cell(length(Ms),1);
            S1adj_sc = cell(length(Ms),1);
            H1_sc = cell(length(Ms),1);
            H1adj_sc = cell(length(Ms),1);
            for idx = 1:length( Ms )
                H_interface{idx} = obj.Surface_interface{idx}.Read('H0');
                S_interface{idx} = obj.Surface_interface{idx}.Read('S0');  
                if ~isempty( obj.q ) && ~obj.Surface_interface{idx}.Read('HamiltoniansDecimated')
                    H1_transverse = obj.Surface_interface{idx}.Read('H1_transverse');
                    H_interface{idx} = H_interface{idx} + H1_transverse*exp(-1i*obj.q) + H1_transverse'*exp(1i*obj.q); 
                end
                H1_sc{idx} = obj.Surface_interface{idx}.Read('Hcoupling');
                H1adj_sc{idx} = obj.Surface_interface{idx}.Read('Hcouplingadj');
                S1_sc{idx} = obj.Surface_interface{idx}.Read('Scoupling');
                S1adj_sc{idx} = obj.Surface_interface{idx}.Read('Scoupling')'; 
            end  

            % The Dyson Eq
            SelfEnergyAll = zeros(sum(Ms), sum(Ms));
            H_interface_All = zeros(sum(Ms), sum(Ms));
            H1_sc_All = zeros(sum(Ms), size(gfininv,2));
            H1adj_sc_All = zeros(size(gfininv,1), sum(Ms));
            S_interface_All = zeros(sum(Ms), sum(Ms));
            for idx = 1:length(Ms)
                    SelfEnergyAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = SelfEnergy{idx};
                    SelfEnergy{idx} = [];

                    H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                    if ~isempty(S_interface{idx})
                        S_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = S_interface{idx};
                        S_interface{idx} = [];
                    else
                        S_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = speye(Ms(idx));
                    end
                    H_interface{idx} = [];
                    
                    if isempty( S1_sc{idx} )
                        H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), : ) = H1_sc{idx};
                        H1_sc{idx} = [];                        
                        H1adj_sc_All( :, sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx};
                        H1adj_sc{idx} = [];                         
                    else
                        H1_sc_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), : ) = H1_sc{idx} - obj.E*S1_sc{idx};
                        H1_sc{idx} = [];
                        S1_sc{idx} = [];
                        H1adj_sc_All( :, sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_sc{idx} - obj.E*S1adj_sc{idx};
                        H1adj_sc{idx} = [];  
                        S1adj_sc{idx} = [];
                    end

            end
    
            Ginv_interface = obj.E*S_interface_All - H_interface_All;

            sumM = sum(Ms);
            Ginverz = zeros(sumM + size(gfininv,1) );
            Ginverz(1:sumM,1:sumM) = Ginv_interface - SelfEnergyAll;
            Ginverz(1:sumM,sumM+1:end) = -H1_sc_All;
            Ginverz(sumM+1:end, 1:sumM) = -H1adj_sc_All;    
            Ginverz(sumM+1:end, sumM+1:end) = gfininv;           
            
        end        
        
    end


%% DecimationFunction
%> @brief Performs the decimation procedure on the inverse Green operator.
%> @param kulso_szabfokok Array of sites to be kept after the decimation.
%> @param ginv The inverse of the Green operator.
%> @param varargin Optional parameters, see the documenattion for details. 
    function [ret, coordinates_ret] = DecimationFunction( obj, kulso_szabfokok, ginv, varargin)
        p = inputParser;
        p.addParamValue('coordinates', [] );        
        p.parse(varargin{:});       
        coordinates    = p.Results.coordinates;
        
        CreateH = CreateHamiltonians(obj.Opt, obj.param);
        
        Hamiltoni = eye(size(ginv))*obj.E - ginv;
        ginv = [];
    
        CreateH.Write('Hscatter', Hamiltoni);
        clear Hamiltoni;
     
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', coordinates);
        CreateH.Write('HamiltoniansCreated', 1);
        CreateH.Write('HamiltoniansDecimated', 0);
        
        Decimation_Procedure = Decimation(obj.Opt); 
        Decimation_Procedure.DecimationFunc(obj.E, CreateH, 'Hscatter', 'kulso_szabfokok');
    
        Hamiltoni = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');
    
        ret = eye(size(Hamiltoni))*obj.E - Hamiltoni;
        Hamiltoni = [];
        
        
        coordinates_ret = CreateH.Read('coordinates');
        
        
    end


%% GetFiniteGreensFunction
%> @brief Query fo the calculated Green operator of the scattering center.
%> @return [Gret, Ginvret] The Green operator and the inverse Green operator.
    function [Gret, Ginvret] = GetFiniteGreensFunction( obj )
        Gret = obj.G;
        Ginvret = obj.Ginv;
    end

%% CalcFiniteGreensFunction
%> @brief Calculates the Green operator of the scattering region.
%> @param varargin Optional parameters, see the documenattion for details. 
    function CalcFiniteGreensFunction( obj, varargin )
    
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians     
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('onlyGinv', false);
        p.parse(varargin{:});
        gauge_trans     = p.Results.gauge_trans;
        ContactPotInterface  = p.Results.ContactPotInterface;
        onlyGinv        = p.Results.onlyGinv;     
        
      	obj.CalcFiniteGreensFunctionFromHamiltonian('gauge_trans', gauge_trans, 'ContactPotInterface', ContactPotInterface, 'onlyGinv', onlyGinv);
        
    end


%% CalcFiniteGreensFunctionFromHamiltonian
%> @brief Calculates the Green operator of the scattering region from the whole Hamiltonian.
%> @param varargin Optional parameters, see the documenattion for details. 
    function CalcFiniteGreensFunctionFromHamiltonian( obj, varargin )
        
        
        
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians     
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('onlyGinv', false);
        p.addParamValue('PotInScatter', [])
        p.parse(varargin{:});
        gauge_trans              = p.Results.gauge_trans;
        ContactPotInterface      = p.Results.ContactPotInterface;
        onlyGinv                 = p.Results.onlyGinv;
        PotInScatter             = p.Results.PotInScatter;
        
        
        obj.CreateScatter();
%**************************************** 
        
        Hscatter = obj.CreateH.Read('Hscatter');
        Sscatter = obj.CreateH.Read('Sscatter');
        coordinates = obj.CreateH.Read('coordinates');
        kulso_szabfokok = obj.CreateH.Read( 'kulso_szabfokok' );
                
        if ~isempty(PotInScatter)            
            potential = sparse(1:size(Hscatter,1), 1:size(Hscatter,1), PotInScatter( coordinates ), size(Hscatter,1), size(Hscatter,1));
            if isfield(coordinates, 'BdG_u')
                potential(~coordinates.BdG_u, ~coordinates.BdG_u) = -potential(~coordinates.BdG_u, ~coordinates.BdG_u);
            end
            Hscatter = Hscatter + potential;
        end
        
                        
              
        q = obj.CreateH.Read('q');
        if ~isempty( q ) && ~obj.CreateH.Read('HamiltoniansDecimated')
            Hscatter_transverse = obj.CreateH.Read('Hscatter_transverse');
            Hscatter = Hscatter + Hscatter_transverse*diag(exp(-1i*q)) + Hscatter_transverse'*diag(exp(1i*q)); 
            obj.CreateH.Clear('Hscatter_transverse');
        end
        
        % Applying overlap matrices
        if isempty( Sscatter )
            Hscatter = (sparse(1:size(Hscatter,1), 1:size(Hscatter,1), obj.E, size(Hscatter,1),size(Hscatter,1))   - Hscatter);
        else
            Hscatter = obj.E*Sscatter - Hscatter;
        end      

        indexes = false( size(Hscatter,1), 1);
        indexes(kulso_szabfokok) = true;
        Hscatter = [ Hscatter( ~indexes, ~indexes) , Hscatter(~indexes, indexes);
                     Hscatter(indexes, ~indexes), Hscatter(indexes, indexes)];

        obj.G = obj.partialInv( Hscatter, length(kulso_szabfokok) );
        
        % truncating the coupling Hamiltonians to fit the scattering region
        for idx = 1: length( obj.Surface_interface )
            H1    = obj.Surface_interface{idx}.Read('Hcoupling');
            H1adj = obj.Surface_interface{idx}.Read('Hcouplingadj');
            H1    = H1(:,kulso_szabfokok);
            H1adj = H1adj(kulso_szabfokok,:);
            S1    = obj.Surface_interface{idx}.Read('Scoupling');
            if ~isempty( S1 )           
                S1    = S1(:,kulso_szabfokok);
            end
            obj.Surface_interface{idx}.Write('Hcoupling', H1);
            obj.Surface_interface{idx}.Write('Hcouplingadj', H1adj);
            obj.Surface_interface{idx}.Write('Scoupling', S1);
            
            coordinates2 = obj.Surface_interface{idx}.Read('coordinates2');
            names = fieldnames(coordinates2);
            for iidx = 1:length(names)
                name = names{iidx};
                if strcmp( name, 'a') || strcmp(name, 'b')
                    continue
                end
                if ~isempty( coordinates2.(name) )
                    coordinates2.(name) = coordinates2.(name)(kulso_szabfokok);
                end
            end
            obj.Surface_interface{idx}.Write('coordinates2', coordinates2);
        end          
          
%         Hscatter = obj.CreateH.Read('Hscatter');
%         Hscatter = (sparse(1:size(Hscatter,1), 1:size(Hscatter,1), obj.E, size(Hscatter,1),size(Hscatter,1))   - Hscatter);
%         obj.G = inv(Hscatter);
%**************************************************        
        
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans
            try
            	% gauge transformation on Green's function
                if ~isempty(obj.G) && ~isempty(obj.PeierlsTransform_Scatter) && isempty(obj.q)
                    % gauge transformation on the inverse Green's function
                    obj.G = obj.PeierlsTransform_Scatter.gaugeTransformation( obj.G, coordinates_scatter, obj.gauge_field );
                end
            catch  errCause  
                err = MException('Ribbon:CalcFiniteGreensFunctionFromHamiltonian', 'Unable to perform gauge transformation');
                err = addCause(err, errCause);
                save('Error_Ribbon_CalcFiniteGreensFunctionFromHamiltonian.mat')
                throw(err);
            end
        end
              
        if onlyGinv
            rcond_G = rcond(obj.G);
            if isnan(rcond_G ) || abs( rcond_G ) < 1e-15
                obj.display( 'Regularizing Ginv by SVD',1);
                obj.Ginv = obj.inv_SVD( obj.G );
            else
                obj.Ginv = inv(obj.G);
            end       
            obj.G = [];
        else
            obj.Ginv = [];
        end
        
    end




%% setInterfaceRegions
%> @brief Replaces the attribute Surface_interface with the given value.
%> @param Surface_interface A two component array of classes Surface_Green_function 
    function setInterfaceRegions( obj, Surface_interface )
       if length( Surface_interface ) ~= 2
           err = MException('Ribbon:setInterfaceRegions', 'The length of Surface_interface must be 2.');
           save('Error_Ribbon_setInterfaceRegions.mat');
           throw(err);          
       end
       
       if ~iscell( Surface_interface )
           err = MException('Ribbon:setInterfaceRegions', 'The Surface_interface must be a cell containing of classes InterfaceRegion.');
           save('Error_Ribbon_setInterfaceRegions.mat');
           throw(err);          
       end 
       
       obj.Surface_interface = Surface_interface;
        
    end

%% CreateInterface
%> @brief Creates the Hamiltonians for the interface regions between the leads and scattering center.
%> @param idx Identification number of the interface region. 
	function CreateInterface( obj, idx )
        
       

        if ~obj.Surface_interface{idx}.Read( 'HamiltoniansCreated' )   
        	obj.display(['Creating interface region', num2str(idx)]) 
            H0 = obj.cCustom_Hamiltonians.Read( 'H0' );
            S0 = obj.cCustom_Hamiltonians.Read( 'S0' );
            H1 = obj.cCustom_Hamiltonians.Read( 'H1' );
            S1 = obj.cCustom_Hamiltonians.Read( 'S1' );
            H_coupling = obj.cCustom_Hamiltonians.Read( 'Hcoupling' );
            S_coupling = obj.cCustom_Hamiltonians.Read( 'Scoupling' );
            obj.Surface_interface{idx}.Write( 'H0', H0{idx} );
            if ~isempty( S0 )
                obj.Surface_interface{idx}.Write( 'S0', S0{idx} );
            end
            obj.Surface_interface{idx}.Write( 'Hcoupling', H_coupling{idx} );
            obj.Surface_interface{idx}.Write( 'Hcouplingadj', H_coupling{idx}' ); 
            if ~isempty( S_coupling )
                obj.Surface_interface{idx}.Write( 'Scoupling', S_coupling{idx} );  
            end
            obj.Surface_interface{idx}.Write( 'H1', H1{idx} );
            obj.Surface_interface{idx}.Write( 'H1adj', H1{idx}' ); 
            if ~isempty( S1 )
                obj.Surface_interface{idx}.Write( 'S1', S1{idx} );  
            end            
            obj.Surface_interface{idx}.Write( 'M', size(H0{idx},1) );  
            coordinates = structures('coordinates');
            obj.Surface_interface{idx}.Write( 'coordinates', coordinates ); 
            obj.Surface_interface{idx}.Write( 'coordinates2', obj.CreateH.Read('coordinates') );
            obj.Surface_interface{idx}.Transform2BdG();
        end
        
        
        % apply magnetic field in the interface region
        if ~isempty( obj.PeierlsTransform_Leads )      
            % In superconducting lead one must not include nonzero magnetic
            % field.
            % Hamiltonians in transverse computations must remain
            % traslational invariant. 
            if ~obj.Surface_interface{idx}.isSuperconducting() %&& isempty( obj.Surface_interface{idx}.Read('q') )
                obj.display(['Applying magnetic field in the Hamiltonians of interface region ', num2str(idx)])
               	obj.PeierlsTransform_Leads.PeierlsTransformLeads( obj.Surface_interface{idx} );  
            else %&& isempty( obj.Surface_interface{idx}.Read('q') )
                obj.display(['Applying gauge transformation in the Hamiltonians of interface region ', num2str(idx)])
               	obj.PeierlsTransform_Leads.gaugeTransformationOnLead( obj.Surface_interface{idx}, obj.gauge_field );
            end
        end  
	end


%% DecimateInterface
%> @brief Decimates the interface region (between the leads and the scattering region) for connecting the leads to the scattering center.
%> @param idx Identification number of the interface region. 
%> @param varargin Optional parameters, see the documenattion for details. 
    function DecimateInterface( obj, idx, varargin )
        
        p = inputParser;
        p.addParamValue('E', obj.E);
        p.addParamValue('ContactPotInterface', []);
        p.addParamValue('Surface_sc', obj.Surface_tmp);
        p.parse(varargin{:});
        E  = p.Results.E;
        ContactPotInterface  = p.Results.ContactPotInterface;
        Surface_sc           = p.Results.Surface_sc.CreateClone();
    
        if obj.Surface_interface{idx}.Read( 'HamiltoniansDecimated' )
    		obj.display('Hamiltonians of the interface region are already decimated');
    		return
        end
        
        obj.display(['Decimating Hamiltonians of interface region', num2str(idx)])
    
        if ~obj.Surface_interface{idx}.Read( 'HamiltoniansCreated' )
            err = MException('Ribbon:DecimateInterface', 'Hamiltonians in Surface_tmp are not created. Thus, unable to perform the decimation.');
            save('Error_Ribbon_DecimateInterface.mat');
            throw(err); 
        end  
        
        H = obj.Surface_interface{idx}.Read('Hamiltonian2Dec');          
        H0 = Surface_sc.Read('H0');        
        if ~isempty( obj.q ) && ~Surface_sc.Read('HamiltoniansDecimated')
            H1_transverse = Surface_sc.Read('H1_transverse');
            q = Surface_sc.Read('q');
            H0 = H0 + H1_transverse*diag(exp(-1i*q)) + H1_transverse'*diag(exp(1i*q)); 
        end
        
        % tranform the gauge in H0 if magnetic field is used
%         if ~isempty( WS.PeierlsTransform_Scatter ) && isempty(WS.q) %for finite WS.q the gauge must remain translational invariant in the transverse direction
%             WS.PeierlsTransform_Scatter.gaugeTransformationOnLead( Surface_sc, WS.gauge_field );
%             H0 = Surface_sc.Read('H0');
%         end
        
        if ~isempty(ContactPotInterface) && ~obj.Surface_interface{idx}.Read( 'HamiltoniansDecimated' )
            coordinates = Surface_sc.Read('coordinates');
            kulso_szabfokok = obj.Surface_interface{idx}.Read('kulso_szabfokok');
            kulso_szabfokok = kulso_szabfokok(1:length(kulso_szabfokok)/2 );
            contactpotential = sparse( 1:size(H0,1), 1:size(H0,1), ContactPotInterface( coordinates.x, coordinates.y ), size(H0,1), size(H0,1) );    
            contactpotential( kulso_szabfokok, kulso_szabfokok ) = 0;
            if isfield(coordinates, 'BdG_u')
                contactpotential(~coordinates.BdG_u, ~coordinates.BdG_u) = -contactpotential(~coordinates.BdG_u, ~coordinates.BdG_u);
            end
            H0 = H0 + contactpotential;
        end


        
        % Hscattering -> H1 -> Hinterface     
        Hsize = size(H,1);
        H( Hsize/2+1:Hsize, Hsize/2+1:Hsize ) = H0;
        %H( 1:Hsize/2, 1:Hsize/2 ) = H0;
                
        kulso_szabfokok = obj.Surface_interface{idx}.Read('kulso_szabfokok');
        coordinates = obj.Surface_interface{idx}.Read('coordinates');
        
        if length(kulso_szabfokok) == size(H,1)
            %nothing to decimate, but alternative coupling to the
            %scattering region might be used
            obj.Surface_interface{idx}.Write('H1', Surface_sc.Read('H1') );
            obj.Surface_interface{idx}.Write('H1adj', Surface_sc.Read('H1adj') );
            return
        end
        
        CreateH = CreateHamiltonians(obj.Opt, obj.param);
        %CreateH = WS.CreateH;        
        CreateH.Write('Hscatter', H);
        H = [];
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', coordinates);
        CreateH.Write('HamiltoniansCreated', 1);
        CreateH.Write('HamiltoniansDecimated', 0);

        Decimation_Procedure = Decimation(obj.Opt); 
        Decimation_Procedure.DecimationFunc(E, CreateH, 'Hscatter', 'kulso_szabfokok');
        Decimation_Procedure.DecimationFunc(E, Surface_sc, 'Hamiltonian2Dec', 'kulso_szabfokok');  
        
        H = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');    
        
        
        coordinates = CreateH.Read('coordinates');
        obj.Surface_interface{idx}.Write('coordinates', coordinates);
        obj.Surface_interface{idx}.Write('Hamiltonian2Dec', H); 
        obj.Surface_interface{idx}.Write('H1', Surface_sc.Read('H1') );
        obj.Surface_interface{idx}.Write('H1adj', Surface_sc.Read('H1adj') );
        obj.Surface_interface{idx}.Write('HamiltoniansDecimated', CreateH.Read('HamiltoniansDecimated')); 
        
    end



%% setHandlesForMagneticField
%> @brief Sets the function handles of the vector potential and gauge transformation.
%> @param varargin Optional parameters, see the documenattion for details. % 
    function setHandlesForMagneticField( obj, varargin )     
    
        p = inputParser;
        p.addParamValue('scatter', [] );
        p.addParamValue('lead', [] );
        p.addParamValue('gauge_field', [] ); 
        
        p.parse(varargin{:});

       
        hscatter    = p.Results.scatter;
        hlead       = p.Results.lead;
        obj.gauge_field = p.Results.gauge_field;
        
        
        obj.display('Adding handles of the magnetic field to the scattering region') 
        if obj.Opt.magnetic_field && isempty( obj.PeierlsTransform_Scatter )
        	obj.PeierlsTransform_Scatter = Peierls(obj.Opt, 'Vectorpotential', hscatter);
        elseif ~isempty( obj.PeierlsTransform_Scatter ) && ~isempty(hscatter)
        	obj.PeierlsTransform_Scatter.setVectorPotential( hscatter );
        end
        
        if obj.Opt.magnetic_field && isempty( obj.PeierlsTransform_Leads )
        	obj.PeierlsTransform_Leads   = Peierls(obj.Opt, 'Vectorpotential', hlead);
        elseif ~isempty( obj.PeierlsTransform_Leads ) && ~isempty(hlead)
        	obj.PeierlsTransform_Leads.setVectorPotential( hlead );
       	end
       	
    end



%% CreateClone
%> @brief Creates a clone of the present object.
%> @return Returns with the cloned object.
    function ret = CreateClone( obj )
        
        ret = TwoTerminal( ...
            'filenameIn', obj.filenameIn, ...
            'filenameOut', obj.filenameOut, ...
            'E', obj.E, ...
            'phi', obj.phi, ...
            'silent', obj.silent, ...
			'Opt', obj.Opt, ...
			'param', obj.param, ...
            'q', obj.q, ...
            'leadmodel', obj.leadmodel );
        
        ret.CreateH = obj.CreateH.CreateClone();
        ret.FL_handles = obj.FL_handles.CreateClone();
        ret.Surface_interface = cell(size(obj.Surface_interface));
        for idx = 1:length(obj.Surface_interface)
            ret.Surface_interface{idx} = obj.Surface_interface{idx}.CreateClone();
        end
        if ~isempty( obj.PeierlsTransform_Scatter )
            ret.PeierlsTransform_Scatter = obj.PeierlsTransform_Scatter.CreateClone();
        end
        
        if ~isempty( obj.PeierlsTransform_Leads )
            ret.PeierlsTransform_Leads   = obj.PeierlsTransform_Leads.CreateClone();    
        end
        ret.gauge_field              = obj.gauge_field;  % function handle for the scalar field to transform the vector potential from Landauy to Landaux
        
    end   
    
    
%% SyncParam  %NEW moved here from Ribbon
%> @brief Synchronizes the param structrure with the parameter values of the basic interfaces
    function SyncParam(obj)
      Surface = obj.FL_handles.Read( 'Surface' ); 
      for idx = 1:length(Surface)
      	obj.param.Leads{idx} = Surface{idx}.Read('params');
      end

      % TODO scatter params sync...
    end    
      
    
end %methods public    

methods ( Access = private )
    
%% CreateTwoTerminalHamiltonians
%> @brief Loads the Hamiltonians from the appropriate DFT interface or files
    function CreateTwoTerminalHamiltonians( obj )
        % instead of CreateRibbon
        if ~strcmp( class(obj.cCustom_Hamiltonians), 'Custom_Hamiltonian' )
            obj.cCustom_Hamiltonians =  Custom_Hamiltonians( obj.Opt, obj.param, 'EF', obj.EF, 'CustomHamiltoniansHandle', obj.CustomHamiltoniansHandle );
        end
        
        if ~obj.cCustom_Hamiltonians.Read( 'Hamiltonians_loaded' )
            obj.cCustom_Hamiltonians.LoadHamiltonians( 'WorkingDir', obj.WorkingDir );
        end
        
        obj.CreateScatter();
        
        for idx = 1:length(obj.Surface_interface)
			obj.CreateInterface( idx );
        end
    end
    

end %methods private

methods ( Access = protected )
     
%% CreateHandles
%> @brief Initializes the attributes of the class.
    function CreateHandles( obj )  
        
        obj.setHandlesForMagneticField();
        
        obj.CreateH = CreateHamiltonians(obj.Opt, obj.param, 'q', obj.q);
        obj.FL_handles = Transport_Interface(obj.E, obj.Opt, obj.param, 'CreateH', obj.CreateH, 'PeierlsTransform', obj.PeierlsTransform_Leads );
        obj.Opt = obj.FL_handles.Read( 'Opt' );
        obj.Surface_interface = cell(length(obj.param.Leads),1);
        for idx = 1:length(obj.Surface_interface)
            obj.Surface_interface{idx} = InterfaceRegion(obj.Opt, obj.param, 'Hanyadik_Lead', idx, 'q', obj.q);
        end
        
    end






%% setParameters
%> @brief Sets some special computing parameters
    function setParameters( obj )
        
        if isvector(obj.phi) && length(obj.phi) == length(obj.param.Leads)
            for idx = 1:length(obj.param.Leads)
                obj.param.Leads{idx}.pair_potential = abs(obj.param.Leads{idx}.pair_potential)*exp(1i*obj.phi(idx));
            end
        end
    end

    
end % protected methods
       
methods (Access=private)    


%% InputParsing
%> @brief Parses the optional parameters for the class constructor.
%> @param varargin Optional parameters, see the web documantation.
    function InputParsing( obj, varargin)
    
        p = inputParser;
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_running_parameters.xml'], @ischar);
        p.addParamValue('WorkingDir', pwd);
        p.addParamValue('CustomHamiltoniansHandle', []); %NEW function handle for the custom Hamiltonians
        p.addParamValue('E', obj.E, @isscalar);
        p.addParamValue('EF', obj.EF);
        p.addParamValue('phi', obj.phi);  
        p.addParamValue('silent', obj.silent);   
        p.addParamValue('leadmodel', obj.leadmodel); %individual physical model for the contacts
        p.addParamValue('Opt', []);
        p.addParamValue('param', []);
        p.addParamValue('q', []);
        
        p.parse(varargin{:});

       
        obj.filenameIn   = p.Results.filenameIn;        
        obj.filenameOut  = p.Results.filenameOut;
        obj.WorkingDir   = p.Results.WorkingDir;
        obj.CustomHamiltoniansHandle = p.Results.CustomHamiltoniansHandle;
        obj.E            = p.Results.E;
        obj.EF           = p.Results.EF;
        obj.phi          = p.Results.phi;
        obj.silent       = p.Results.silent;
        obj.leadmodel    = p.Results.leadmodel;
        obj.q            = p.Results.q;
		obj.Opt          = p.Results.Opt;
		obj.param        = p.Results.param;       
        
                
		if isempty(obj.Opt) || isempty(obj.param)   
			[ obj.Opt, obj.param ] = parseInput( obj.filenameIn );
		end

    end

end % methods private

end
